/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
                         
//*****************************************************************************
//
// Filename : TPROCESSWFS.CPP/MAK
// Description : Process workflow in WorkFlow/WorkBasket
//
// Required setup:  
//          
//    - Item ID of a Document or Folder (e.g., "N5LA94C7BCH8HZA8")
//    - Item ID of a WorkFlow (e.g., "HI7MOPALUPFQ1U47") containing 2 WorkBaskets             
//    - Item ID of a WorkBasket (e.g., "E3PP1UZOZUFQ1U3M") which is the second 
//      WorkBasket in the above WorkFlow           
//*****************************************************************************

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDL.hpp>
#include <DKWorkFlowServiceDL.hpp>
#include <DKException.hpp> 
#include <DKString.hpp>
#include <DKDDO.hpp>
#include <DKSequentialCollection.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;
    DKDatastoreDL dsDL; // must be outside try/catch block

    try {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    DKString str; 
    DKAny input_option = DK_SS_CONFIG;
    DKAny output_option;
    DKWorkFlowServiceDL wfDL(&dsDL);

    DKString itemID   = DKString("N5LA94C7BCH8HZA8");
    DKString itemIDWF = DKString("HI7MOPALUPFQ1U47");
    DKString itemIDWB = DKString("E3PP1UZOZUFQ1U3M");

    cout << "Datastore DL created" << endl;
    cout << "connecting to datastore" << endl;
    dsDL.connect("lsmufasa", "FRNADMIN", "PASSWORD");
    cout << "datastore connected" << endl;

    if (wfDL.isConnected())
	cout << "workflow service has been connected" << endl;
    else
	cout << "workflow service is not connected" << endl;	

    cout << "starting Item ID " << itemID << " ... " << endl;
    wfDL.startWorkFlowItem(itemID,                    //itemID
                           itemIDWF,                  //itemIDWF
                           NULL,                      //default (the first workbasket)
                           TRUE,                      //overload
                           DK_WIP_DEFAULT_PRIORITY    //initial_priority
						  );
    cout << "Item ID " << itemID << " started" << endl;

    cout << "routing Item ID " << itemID << " to WorkBasket " << itemIDWB << endl;
    wfDL.routeWipItem(itemID,                //itemID 
                      itemIDWB,              //itemIDWB, 
                      TRUE,                  //overload 
                      DK_NO_PRIORITY_CHANGE  //priority
					 );	 
    cout << "Item ID " << itemID << " routed" << endl;
	
    cout << "completing Item ID " << itemID << " ... " << endl;
    wfDL.completeWorkFlowItem(itemID);
    cout << "Item ID " << itemID << " completed" << endl;

    dsDL.disconnect();
    cout << "datastore disconnected" << endl;
    }
 
    catch(DKException &exc)
    {
      cout << "Error id "     << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }

    cout << "done ..." << endl;
    
}
