/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TExecMSTFed.cpp
 * Description    : Create search template in memory and execute a query.
 *                : Need to run the following program first
 *                  TADLServersFed <cmbdb> <userid> <pw> add
 *                  TADLsmFed      <cmbdb> <userid> <pw> add
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKString.hpp>
#include <DKAny.hpp>
#include <DKDatastoreFed.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKException.hpp>
#include <DKEnvironment.hpp>
#include <DKAttrDefFed.hpp>
#include <DKEntityDefFed.hpp>
#include <DKAttrMappingFed.hpp>
#include <DKAttrMapDef.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>
#include <DKSearchTemplateFed.hpp>
#include <DKSearchCriteria.hpp>

void displayDDO(DKDDO* ddo);



void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //-------------------------------------------------------------- 
   // Checking for input parameters
   //--------------------------------------------------------------   
   if (argc < 4)  {
      cout <<"Usage syntax: "<<endl;
      cout <<"\t TExecMSTFed <feddb> <userid> <pw> " << endl;
      return;
   }
   
   // ### get input parameters
   DKString libsrv(argv[1]);
   DKString userid(argv[2]);
   DKString pw    (argv[3]);
   DKDatastoreFed dsFed; // must be outside try block to avoid exception trap error.

   try {
   
      // All set; Connect and execute the compound query expression
      cout << " *** connecting to " << libsrv << " *** " << endl;
      dsFed.connect(libsrv, userid, pw);
      cout << " \t -----> connected " << endl;
      cout << "     user " << dsFed.userName() << ", dsName " << dsFed.datastoreName() << endl;
   
      // set up search criteria memory variable.
	  short ops[] = { DK_CM_OPCODE_EQ,      DK_CM_OPCODE_NEQ,
			          DK_CM_OPCODE_LIKE,    DK_CM_OPCODE_NOTLIKE,
			          DK_CM_OPCODE_IN,      DK_CM_OPCODE_NOTIN };

      short ops2[] = { DK_CM_OPCODE_EQ,     DK_CM_OPCODE_NEQ,
			           DK_CM_OPCODE_BETWEEN,DK_CM_OPCODE_NOTBETWEEN,
			           DK_CM_OPCODE_IN,     DK_CM_OPCODE_NOTIN,
			           DK_CM_OPCODE_LT,     DK_CM_OPCODE_LEQ,
			           DK_CM_OPCODE_GT,     DK_CM_OPCODE_GEQ };

      //===================================================================================================
      // Defines search criterias for FedDemo
      //===================================================================================================

      // Create search criteria for FedUserId
	  DKSearchCriteria *scUserID = new DKSearchCriteria("FedDemo", "FedUserId");
                        scUserID->setName("UserId");
	                    scUserID->setDisplayName("Document UserID");
	                    scUserID->setDefaultOperator(DK_CM_OPCODE_LIKE);
	                    scUserID->setValidOperators(ops, 6);
                        scUserID->setDisplayPosition((short)1);
	                    scUserID->setDisplayWidth((short)40);
	                    scUserID->setQueryable(true);
	                    scUserID->setDisplayable(true);
                        scUserID->setOperator(DK_CM_OPCODE_LIKE);
                        scUserID->setValue("'Prog%'");

      // Create search criteria for FedTitle
	  DKSearchCriteria *scTitle = new DKSearchCriteria("FedDemo", "FedTitle");
	                    scTitle->setName("Title");
	                    scTitle->setDisplayName("Document Title");
	                    scTitle->setDefaultOperator(DK_CM_OPCODE_LIKE);
	                    scTitle->setValidOperators(ops, 6);
	                    scTitle->setDisplayPosition((short)2);
	                    scTitle->setDisplayWidth((short)40);
	                    scTitle->setQueryable(true);
	                    scTitle->setDisplayable(true);
                        scTitle->setOperator(DK_CM_OPCODE_EQ);
                        scTitle->setValue("BASUKI");

      // Create search criteria for FedDate
	  DKSearchCriteria* scDate = new DKSearchCriteria("FedDemo", "FedDate");
	                    scDate->setName("Date");
	                    scDate->setDisplayName("Document Date");
	                    scDate->setDefaultOperator(DK_CM_OPCODE_EQ);
	                    scDate->setValidOperators(ops2, 10);
	                    scDate->setDisplayPosition((short)3);
	                    scDate->setDisplayWidth((short)20);
	                    scDate->setQueryable(true);
	                    scDate->setDisplayable(true);
                        // sets the value:
                        scDate->setOperator(DK_CM_OPCODE_NEQ);
                        scDate->setValue("NULL");

      // Create search criteria for FedUserId
	  DKSearchCriteria *scUserID1 = new DKSearchCriteria("FedDemo", "FedUserId");
                        scUserID1->setName("FedUserId");
	                    scUserID1->setDisplayName("Document Author");
	                    scUserID1->setDefaultOperator(DK_CM_OPCODE_LIKE);
	                    scUserID1->setValidOperators(ops, 6);
                        scUserID1->setDisplayPosition((short)1);
	                    scUserID1->setDisplayWidth((short)40);
	                    scUserID1->setQueryable(true);
	                    scUserID1->setDisplayable(true);
                        scUserID1->setOperator(DK_CM_OPCODE_LIKE);
                        scUserID1->setValue("'Alan%'");

      // Create search criteria for FedTitle
	  DKSearchCriteria *scTitle1 = new DKSearchCriteria("FedDemo", "FedTitle");
	                    scTitle1->setName("FedTitle");
	                    scTitle1->setDisplayName("Document Title");
	                    scTitle1->setDefaultOperator(DK_CM_OPCODE_LIKE);
	                    scTitle1->setValidOperators(ops, 6);
	                    scTitle1->setDisplayPosition((short)2);
	                    scTitle1->setDisplayWidth((short)40);
	                    scTitle1->setQueryable(true);
	                    scTitle1->setDisplayable(true);
                        scTitle1->setOperator(DK_CM_OPCODE_NEQ);
                        scTitle1->setValue("NULL");

      // Create search criteria for FedDate
	  DKSearchCriteria* scDate1 = new DKSearchCriteria("FedDemo", "FedDate");
	                    scDate1->setName("FedDate");
	                    scDate1->setDisplayName("Subject Heading");
	                    scDate1->setDefaultOperator(DK_CM_OPCODE_EQ);
	                    scDate1->setValidOperators(ops2, 10);
	                    scDate1->setDisplayPosition((short)3);
	                    scDate1->setDisplayWidth((short)20);
	                    scDate1->setQueryable(true);
	                    scDate1->setDisplayable(true);
                        scDate1->setOperator(DK_CM_OPCODE_LIKE);
                        scDate1->setValue("'Contents%'");
      
      // set up search template memory variable.
      DKNVPair optL[2];
      optL[0].set("MAX_RESULTS", DKString("6"));
      optL[1].set("CONTENT", DKString("ATTRONLY"));
      int size = 2;
                        
      //===================================================================================================
      // Defines memory only search templates for FedDemo
      //===================================================================================================
         
      // Create search template: FedUserId AND FedTitle AND FedDate
      DKSearchTemplateFed *stANDS = new DKSearchTemplateFed(&dsFed);
	                       stANDS->setName("FedDemoST_ANDS");
                           stANDS->setEntityName("FedDemo");
                           stANDS->setDescription("All ANDS in memory search template for FedDemo.");
                           stANDS->setParametricOp(DK_CM_OP_ALL_ORS);
                           stANDS->setOptionList(optL, size);
                           stANDS->addCriteria(scUserID);     // Add search criterias to memory only.
                           stANDS->addCriteria(scTitle);
                           stANDS->addCriteria(scDate);
                                       
              
      // Create search template: FedUserId OR FedTitle OR FedDate
      DKSearchTemplateFed *stORS = new DKSearchTemplateFed(&dsFed);
                           stORS->setName("FedDemoST_ORS");
                           stORS->setDescription("All ORS in memory search template for FedDemo.");
                           stORS->setEntityName("FedDemo");
                           stORS->setParametricOp(DK_CM_OP_ALL_ORS);
                           stORS->setOptionList(optL, size);
                           stORS->addCriteria(scUserID1);     // Add search criterias to memory only.
                           stORS->addCriteria(scTitle1);
                           stORS->addCriteria(scDate1);

      //===================================================================================================
      // Execute search template.
      //===================================================================================================

      // Execute ALL ANDS search template
      DKCQExpr* cqe = stANDS->createQueryExpr();

      cout << " *** execute query expression for all ANDS search template ****" << endl;
      cout << "    Criteria: FedUserId AND FedTitle AND FedDate" << endl;
      
      dkResultSetCursor* rsc = dsFed.execute(cqe);

      //  get result set cursor
      if (rsc == NULL) 
         cout << " *** cursor is null" << endl;
      else {
         cout << " hit count = " << rsc->cardinality() << endl;
         DKDDO* ddo;
     
         while (rsc->isValid())  {
            cout << "before fetch, position = " << rsc->getPosition() << endl;
            ddo = rsc->fetchNext();
            cout << "after fetch, position = " << rsc->getPosition() << endl;
     
            if (ddo != NULL) {  
               cout << " pidString : " << ((DKPid&)ddo->getPid()).pidString() << endl;
               displayDDO(ddo);
               delete ddo;
            } 
            else 
               break;
         }
         delete rsc;
      }
      delete cqe;
   

      // Execute ALL ORS search template
      cqe = stORS->createQueryExpr();

      cout << " *** execute query expression for all ORS search template ****" << endl;
      cout << "    Criteria: FedUserID OR FedTitle OR FedDate" << endl;
         
      rsc = dsFed.execute(cqe);

      //  get result set cursor
      if (rsc == NULL) 
         cout << " *** cursor is null" << endl;
      else {
         cout << " hit count = " << rsc->cardinality() << endl;
         DKDDO* ddo;
     
         while (rsc->isValid())  {
            cout << "before fetch, position = " << rsc->getPosition() << endl;
            ddo = rsc->fetchNext();
            cout << "after fetch, position = " << rsc->getPosition() << endl;
     
            if (ddo != NULL) {  
               cout << " pidString : " << ((DKPid&)ddo->getPid()).pidString() << endl;
               displayDDO(ddo);
               delete ddo;
            } 
            else 
               break;
         }
         delete rsc;
      }
      delete cqe;      
     
      delete stANDS;
      delete stORS;

      // disconnects
      cout << "\n\n *** disconnect from the datastore Fed. ***" << endl;
      dsFed.disconnect();
      cout << " *** datastore Fed disconnected ***" << endl;
     
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
         cout << "Error text: " << exc.text(i) << endl;
     
      for (unsigned long g=0; g< exc.locationCount();g++) {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
     
      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}


// simple method to show DDO hit 
void displayDDO(DKDDO* ddo) 
{
   DKString   dataName;
   short      cnt;
   DKAny      any;
   
   if (ddo != NULL) {
      cnt = ddo->dataCount();
      for (short i = 1; i <= cnt; i++) {
          any = ddo->getData(i);
          dataName = ddo->getDataName(i);
          cout << i << ": Attribute Name :" << dataName << endl;
          if (!any.isNull())
               cout << ",\t   Value :" << any << endl;
          else cout << ",\t   Value : null"  << endl;
      }  
   }
}

