/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/******************************************************************************
 *
 * Source File Name = TCallbackOD.CPP
 *
 * TCallbackOD - an class that represents a callback
 *               for OD
 *
 *****************************************************************************/
#include "TCallbackOD.hpp"
#ifdef WIN32
#include <io.h>
#include <conio.h>
#endif
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobOD.hpp>
#include <DKPidXDOOD.hpp>
#include <DKDatastoreExtOD.hpp>


//**********************************************************************
// constructor
//**********************************************************************
TCallbackOD::TCallbackOD(DKDatastoreOD* dsOD,
                         long           thresholdValue,
                         long           iterationsValue,
                         BOOL           retrieveHits)
{
  _dsOD = dsOD;
  _threshold = thresholdValue;
  _iterationsValue = iterationsValue;
  _retrieveHits = retrieveHits;

  _iterations = 0;
  _cancel     = FALSE;
  _isDone     = FALSE;
  _displayed  = FALSE;
  _cnt        = 0;
  _exc        = NULL;

  _pColHitList = new DKSequentialCollection();

  cout << "*** callback created " << endl;
}

//**********************************************************************
// destructor
//**********************************************************************
TCallbackOD::~TCallbackOD()
{
  if (_exc != NULL)
  {
    delete _exc;
  }
  delete _pColHitList;
}

//**********************************************************************
// setThreshold - Sets the number of returned hits for each data callback
//**********************************************************************
void TCallbackOD::setThreshold(long nValue)
{
  _threshold = nValue;
}

//**********************************************************************
// getThreshold - Gets the number of returned hits that the server returns
//**********************************************************************
long TCallbackOD::getThreshold()
{
  return _threshold;
}

//**********************************************************************
// cancelQuery - Indicates user has canceled the query
//**********************************************************************
void TCallbackOD::cancelQuery()
{
  _cancel = TRUE;
}

//**********************************************************************
// dataCallback - Return nHits from the backend datastore
//**********************************************************************
DKBoolean TCallbackOD::dataCallback(dkCollection* ddoList, DKBoolean isDone)
{
  _iterations++;
  if (!isDone)
    cout << "\nnumber of hits returned is: " << ddoList->cardinality() << endl;
  else
    cout << "done value is: " << isDone << endl;

  // show the world we got hits!!!
  dkIterator* pIter = ddoList->createIterator();
  while (pIter->more())
  {
    try
    {
      _cnt++;
      DKDDO* ddo = (DKDDO*)(pIter->next()->value());

      displayHit(_cnt, ddo);

      if (_retrieveHits && ddo)
      {
        // save the ddo to the end of the hitlist
        _pColHitList->addElement(DKAny(ddo));
      }
    }
    catch (DKException exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for (unsigned long i = 0; i< exc.textCount();i++)
      {
        cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
        const DKExceptionLocation* p = exc.locationAtIndex(g);
        cout << "Filename: " << p->fileName() << endl;
        cout << "Function: " << p->functionName() << endl;
        cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
  }
  delete pIter;
  ddoList->removeAllElements();
  delete ddoList;
  if (_retrieveHits == FALSE)
    cout << "Number of items processed: "  <<  _cnt  << endl;

  if (_iterationsValue != -1 && _iterations == _iterationsValue)
  {
    cancelQuery();
  }
  if (_cancel || isDone)
  {
    _isDone = TRUE;
  }

  if (_isDone && !_displayed)
  {
    _displayed = TRUE;
    cout << "Total number of hits/DDOs received: "  <<  _cnt  << endl;
    cout << "TCallbackOD done ..." << endl;
  }
  return _cancel;
}

//**********************************************************************
// reportException - Report an exception from the backend datastore
//**********************************************************************
void TCallbackOD::reportException(DKException* exc)
{
  if (_exc != NULL)
  {
    delete _exc;
  }
  _exc = exc;
}

//**********************************************************************
// getException - Get an exception from the backend datastore
//**********************************************************************
DKException* TCallbackOD::getException()
{
  return _exc;
}

//**********************************************************************
// resetException - Call after the exception has been deleted, this method
// will clear the exception variable.
//**********************************************************************
void TCallbackOD::resetException()
{
  _exc = NULL;
}

//**********************************************************************
// isDone -
//**********************************************************************
DKBoolean TCallbackOD::isDone()
{
  return _isDone;
}

//**********************************************************************
// retrieve DDOs
//**********************************************************************
void TCallbackOD::retrieveDDO()
{
  DKPid*      pid;
  DKDDO*      ddo = 0;
  dkIterator* pIter = 0;
  int         cnt = 0;

  cout << "_pColHitList->cardinality()=" << _pColHitList->cardinality() << endl;
  if (_pColHitList->cardinality() > 0)
  {
    pIter = _pColHitList->createIterator();
    while (pIter->more() == TRUE)
    {
      cnt++;
      DKDDO* ddoold = (DKDDO*)(pIter->next()->value());

      DKString pidStr = ((DKPid*)ddoold->getPidObject())->pidString();
      //cout << " pidString : " << pidStr << endl;

      // the retrieval action
      pid = new DKPid(pidStr);

      short id, docType = 0;
      DKAny a;
      if ((id = ddoold->propertyId(DK_CM_PROPERTY_ITEM_TYPE)) > 0)
      {
        a = ddoold->getProperty(id);
        if (a.typeCode() == DKAny::tc_ushort)
          docType = (short)(USHORT)a;
        else
          docType = a;
      }
      if (docType == DK_CM_DOCUMENT)
      {
        cout << "create the DDO from the pidstring..." << endl;
        ddo = _dsOD->createDDO(ddoold->getObjectType(), DK_CM_DOCUMENT);
        ddo->setPidObject(pid);

        // This step is needed for all callback testcases
        _dsOD->retrieveObject((dkDataObject*)ddo);

        displayDDO(cnt, _dsOD, ddo, FALSE);
      }
      delete pid;
      delete ddoold;
    }
    delete pIter;
  }
  cout << "Total number of DDOs processed: "  <<  cnt  << endl;
}

// --------------------------------------------------------------------
// display hits
// --------------------------------------------------------------------
void TCallbackOD::displayHit(int cnt, DKDDO* p)
{
  DKAny    a;
  short    j = 0;

  if (p != NULL)
  {
    try
    {
      int numDataItems = numDataItems = p->dataCount();
      // display attribute (field) names (1st time only)
      if (cnt == 1)
      {
        cout << "" << endl;
        for (j = 1; j <= numDataItems; j++)
        {
          DKString strDataName = p->getDataName(j);
          if (strDataName != DK_CM_DKPARTS   &&
              strDataName != "DKResource"    &&
              strDataName != "DKViews"       &&
              strDataName != "DKLargeObject" &&
              strDataName != "DKPermissions" &&
              strDataName != "DKFixedView"   &&
              strDataName != "DKAnnotations")
          {
            cout << "  \t\"" << strDataName << "\"";
          }
        }
        cout << "" << endl;
      }
      // display attribute (field) values
      for (j = 1; j <= numDataItems; j++)
      {
        a = p->getData(j);
        DKString strDataName = p->getDataName(j);
        if (strDataName != DK_CM_DKPARTS   &&
            strDataName != "DKResource"    &&
            strDataName != "DKViews"       &&
            strDataName != "DKLargeObject" &&
            strDataName != "DKPermissions" &&
            strDataName != "DKFixedView"   &&
            strDataName != "DKAnnotations")
        {
          if (j == 1)
            cout << " " << cnt << ".\t";
          else
            cout << "\t";

          if ((a.typeCode() == DKAny::tc_string) ||
              (a.typeCode() == DKAny::tc_long)   ||
              (a.typeCode() == DKAny::tc_short)  ||
              (a.typeCode() == DKAny::tc_date)   ||
              (a.typeCode() == DKAny::tc_time)   ||
              (a.typeCode() == DKAny::tc_timestamp))
          {
            cout << "\"" << a << "\"";
          }
          else
            cout << "";
        }
      }
      cout << "" << endl;
    }
    catch (DKException exc)
    {
      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for (unsigned long i = 0; i< exc.textCount();i++)
      {
        cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
        const DKExceptionLocation* p = exc.locationAtIndex(g);
        cout << "Filename: " << p->fileName() << endl;
        cout << "Function: " << p->functionName() << endl;
        cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
  }
}

//**********************************************************************
// Display results
//**********************************************************************
void TCallbackOD::displayDDO(int            cnt,
                             DKDatastoreOD* dsOD,
                             DKDDO*         p,
                             DKBoolean      fAfp2Html)
{
  DKString DocFile, DataFile, ResourceFile, TocFile, LogViewFile;

  // create the output directory if it does not exist
#ifdef WIN32
  DKString OutDir = DKString("c:\\cmbuild\\output\\");
#else
  DKString OutDir = DKString("/archive/bulldog/eip/samples/output/");
#endif
  int rc = access( OutDir, 0 );
  if ( rc == -1 )  // does not exist
  {
    // create the directory
    rc = DKDatastoreOD::createDirectory(OutDir);
    // if we can't create the directory, quit now!
    if (rc == FALSE)
    {
      cout << "\n***Cannot create output directory \"" << OutDir << "\"!!!\n" << endl;
      return;
    }
  }

  dkDataObjectBase * pDO = NULL;
  dkCollection* pCol = NULL;
  dkIterator* pIter = NULL;
  DKAnnotationOD* annotation = NULL;

  DKDDO *pDDO = 0;
  DKBlobOD *pXDO = 0;

  DKPidXDOOD *pidXDO = 0;
  DKPid *pid = 0;

  DKAny a;
  int i = 0;
  short k = 0;
  short j = 0;
  short val = 0;
  int numDataItems = 0;
  long pidIdCnt = 0;
  long pidIndex = 0;

  DKString strData = "";
  DKString strDataName = "";
  DKString strPid;

  DKString DataString;
  FILE *myLogViewFile,
       *myResourceFile,
       *myTocFile,
       *myDataFile,
       *myDocFile;
  char* DataBuffer = 0;
  //CsvNote * pODNote;
  int numBytes = 0;
  int numNotes = 0;
  DKString noteString;
  DKString extension("");
  DKString MimeType("");
  DKString segmentFile;
  DKString cnt_str;

  cout << "\n\n==========> " << "Item " << cnt << " <=========\n" << endl;
  numDataItems = p->dataCount();
  pid = (DKPid*)p->getPidObject();
  strPid = pid->pidString();
  cout << "pid string " << strPid << endl;
  cout << "pid id string " << pid->getId() << endl;
  strPid = pid->getIdString();
  cout << "pid idString " << strPid << endl;
  pidIdCnt = pid->getIdStringCount();
  cout << "pid idString cnt " << pidIdCnt << endl;
  strPid = pid->getPrimaryId();
  cout << "pid primary id " << strPid << endl;
  pidIndex = 0;
  strPid = pid->getIdString(pidIndex);
  cout << "pid item id " << strPid << endl;

  // Retrieve privileges
  DKDatastoreExtOD* dsODExt = (DKDatastoreExtOD*)dsOD->getExtension(DK_CM_DATASTORE_EXT);
  DKCommonPrivilege* priv = dsODExt->getCommonPrivilege(p);
  if (priv != NULL)
  {
    cout << "privileges are:" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_PRINT) == DK_CM_PRIV_YES)
       cout << " +++can print document" << endl;
    else
       cout << " ---cannot print document" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_VIEW_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can view annotations" << endl;
    else
       cout << " ---cannot view annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_CREATE_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can create annotations" << endl;
    else
       cout << " ---cannot create annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_DELETE_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can delete annotations" << endl;
    else
       cout << " ---cannot delete annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_EDIT_TEXT_ANNOTATION) == DK_CM_PRIV_YES)
       cout << " +++can edit annotations" << endl;
    else
       cout << " ---cannot edit annotations" << endl;
    if (priv->checkAuthorization(DK_CM_PRIV_MODIFY_MULTI_VIEWDATA) == DK_CM_PRIV_YES)
       cout << " +++can change logical views" << endl;
    else
       cout << " ---cannot change logical views" << endl;
  }

  k = p->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
  if (k > 0)
  {
    a = p->getProperty(k);
    if (a.typeCode() == DKAny::tc_ushort)
      val = (short)(USHORT)a;
    else
      val = a;
    cout << "******************************" << endl;
    switch (val)
    {
      case DK_CM_DOCUMENT :
        cout << "Item is a document " << endl;
        break;
      case DK_CM_ITEM :
        cout << "Item is an item " << endl;
        break;
    }
    cout << "******************************" << endl;
  }

  cnt_str = DKString(cnt);
  DataFile     = OutDir + DKString("docdata")  + cnt_str;
  ResourceFile = OutDir + DKString("resource") + cnt_str + DKString(".dat");
  TocFile      = OutDir + DKString("Toc") + cnt_str + DKString(".dat");
  LogViewFile  = OutDir + DKString("LogView")  + cnt_str + DKString(".dat");

  // Removed existing output files
  if (access( LPCTSTR(ResourceFile), 0 ) != -1 )
    remove(LPCTSTR(ResourceFile));
  if (access( LPCTSTR(TocFile), 0 ) != -1 )
    remove(LPCTSTR(TocFile));
  if (access( LPCTSTR(LogViewFile), 0 ) != -1 )
    remove(LPCTSTR(LogViewFile));

  cout << "  Number of Data Items: " << numDataItems << endl;
  for (j = 1; j <= numDataItems; j++)
  {
    a = p->getData(j);
    strDataName = p->getDataName(j);

    cout << "    " << j << ". Attribute Name: " << strDataName << endl;
    cout << "       type: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_TYPE) << endl;
    cout << "       nullable: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE) << endl;

    if (strDataName != DK_CM_DKPARTS   &&
        strDataName != "DKResource"    &&
        strDataName != "DKViews"       &&
        strDataName != "DKLargeObject" &&
        strDataName != "DKPermissions" &&
        strDataName != "DKFixedView"   &&
        strDataName != "DKAnnotations")
    {
      cout << "       attribute id: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID) << endl;
    }

    if (a.typeCode() == DKAny::tc_string)
    {
      DKString astring = a;
      cout << "       Attribute Value (string): =>" << astring << "<=" << endl;
    }
    else if (a.typeCode() == DKAny::tc_long /*Integer*/)
    {
      long along = a;
      cout << "       Attribute Value (long): " << along << endl;
    }
    else if (a.typeCode() == DKAny::tc_short)
    {
      short ashort = a;
      cout << "       Attribute Value (short): " << ashort << endl;
    }
    else if (a.typeCode() == DKAny::tc_date)
    {
      DKDate adate = a;
      cout << "      Attribute Value (date): " << adate << endl;
    }
    else if (a.typeCode() == DKAny::tc_time)
    {
      DKTime atime = a;
      cout << "       Attribute Value (time): " << atime << endl;
    }
    else if (a.typeCode() == DKAny::tc_timestamp)
    {
      DKTimestamp atimestamp = a;
      cout << "       Attribute Value (timestamp): " << atimestamp << endl;
    }
    else if (a.typeCode() == DKAny::tc_collection)
    {
      cout << "       Attribute Value is collection" << endl;
      pCol = a;
      pIter = pCol->createIterator();
      i = 0;

      while (pIter->more() == TRUE)
      {
        i++;
        a = *(pIter->next());
        pDO = a;

        if (pDO->protocol() == DK_CM_XDO)
        {
          cout << "         dkXDO object " << i << " in collection" << endl;
          pXDO = (DKBlobOD*)pDO;
          pidXDO = (DKPidXDOOD*)pXDO->getPidObject();
          cout << "           XDO pid string: " << pidXDO->pidString() << endl;

          MimeType = pXDO->getMimeType();
          cout << "MimeType = " << MimeType << endl;

          // Set the EBCDIC->ASCII conversion
          if ( MimeType == "application/ondemand line" )
          {
            cout << "Setting the EBCDIC->ASCII conversion option..." << endl;
            a = (long)DK_CM_TEXT_ASCII;
            pXDO->setOption( DK_CM_OPT_XDO_TEXT_CONVERSION, a);
          }
          else if (fAfp2Html == TRUE && MimeType == "application/afp" )
          {
            cout << "Setting the AFP->HTML conversion option..." << endl;
            a = (long)DK_CM_AFP_HTML;
            pXDO->setOption(DK_CM_OPT_XDO_AFP_CONVERSION, a);
            a = (long)DK_CM_APPLET;
            pXDO->setOption(DK_CM_OPT_XDO_VIEWER, a);
          }

          // Retrieve and open instance handler for an XDO
          pXDO->retrieve();
          cout << "Content Begin:" << endl;

          DataString = pXDO->getContent();
          long DataSize = DataString.length();
          DataBuffer = (char*)LPCTSTR(DataString);
          long item_type = ((DKPidXDOOD*)pXDO->getPidObject())->getType();
          long part_id = ((DKPidXDOOD*)pXDO->getPidObject())->getPartId();
          switch (item_type)
          {
            case DK_CM_ANNOTATION:
              // binary
              cout << "======DK_CM_ANNOTATION (Notes)======" << endl;
              annotation = (DKAnnotationOD*)((DKBlobOD*)pXDO)->getExtension("dkAnnotationExt");
              noteString  = "NOTE: " + annotation->getTimestamp();
              noteString += " by " + annotation->getUserID();
              noteString += " for page " + DKString(annotation->getPageNumber());
              noteString += "\n";
              noteString += annotation->isPublic() ? "Public" : "Private";
              noteString += "--";
              noteString += annotation->isOkToCopy() ? "Can" : "Cannot";
              noteString += " be copied to another server";
              noteString += DKString("\n") + annotation->getText();
              cout << noteString << endl;

              // ======== method #2 ========
              //cout << "====Note====" << endl;
              //cout << "isPublic     = " << annotation->isPublic()      << endl;
              //cout << "isOkToCopy   = " << annotation->isOkToCopy()    << endl;
              //cout << "Color        = " << annotation->getColor()      << endl;
              //cout << "Page Number  = " << annotation->getPageNumber() << endl;
              //cout << "X coordinate = " << annotation->getX()          << endl;
              //cout << "Y coordinate = " << annotation->getY()          << endl;
              //cout << "UserID       = " << annotation->getUserID()     << endl;
              //cout << "Timestamp    = " << annotation->getTimestamp()  << endl;
              //cout << "GroupName    = " << annotation->getGroupName()  << endl;
              //cout << "NoteID       = " << annotation->getPart()       << endl;
              //cout << "Text         = " << annotation->getText()       << endl;
              break;
            case DK_OD_VIEW:
              // binary
              cout << "======DK_OD_VIEW (Logical Views)======" << endl;
              myLogViewFile = fopen(LogViewFile,"ab+");
              numBytes = fwrite(DataBuffer,1,DataSize,myLogViewFile);
              fclose(myLogViewFile);
              break;
            case DK_OD_SEGMENT:
              // binary
              if ( extension == "" )
              {
                if ( MimeType == "application/afp" )
                  extension = "afp";
                else if ( MimeType == "application/ondemand line" )
                  extension = "lin";
                else if ( MimeType == "text/plain" )
                  extension = "txt";
                else if ( MimeType == "image/bmp" )
                  extension = "bmp";
                else if ( MimeType == "image/gif" )
                  extension = "gif";
                else if ( MimeType == "image/jpeg" )
                  extension = "jpg";
                else if ( MimeType == "image/pcx" )
                  extension = "pcx";
                else if ( MimeType == "image/tif" )
                  extension = "tif";
                else if ( MimeType == "application/pdf" )
                  extension = "pdf";
                else
                  extension = "dat";
              }
              cout << "======DK_OD_SEGMENT (Doc Data) part_id=" << part_id << "======" << endl;
              if (fAfp2Html == TRUE && extension == "afp")
                extension = "htm";
              segmentFile = DataFile + "." + extension;
              if ( part_id == 0 )
              {
                if (access( LPCTSTR(segmentFile), 0 ) != -1 )
                  remove(LPCTSTR(segmentFile));
              }
              // The first segment starts at part_id 0
              if ( part_id >= 0)
              {
                myDataFile = fopen(segmentFile,"ab+");
                numBytes = fwrite(DataBuffer,1,DataSize,myDataFile);
                fclose(myDataFile);
              }
              break;
          }
          cout << endl << "Content End" << endl;
          //pXDO->open();
        }
      }
      if (pIter != 0)
        delete pIter;
    }
    else if (a.typeCode() != DKAny::tc_null)
    {
      cout << "       Attribute Value (non NULL): " << a << endl;

      if (strDataName == "DKResource"    ||
          strDataName == "DKFixedView"   ||
          strDataName == "DKLargeObject")
      {
        pDO = (dkDataObjectBase*)a;

        if (pDO->protocol() == DK_CM_XDO)
        {
          cout << "         dkXDO object " << endl;
          pXDO = (DKBlobOD*)pDO;
          pidXDO = (DKPidXDOOD*)pXDO->getPidObject();
          cout << "           XDO pid string: " << pidXDO->pidString() << endl;

          // Retrieve and open instance handler for an XDO
          pXDO->retrieve();
          //pXDO->open();

          DataString = pXDO->getContent();
          long DataSize = DataString.length();
          DataBuffer = (char*)LPCTSTR(DataString);
          long item_type = ((DKPidXDOOD*)pXDO->getPidObject())->getType();

          if (strDataName == "DKResource" )
          {
            // binary
            cout << "======DK_OD_RESOURCE (AFP Resource Group)======" << endl;
            // I don't think resource is needed for html files!
            if (fAfp2Html != TRUE)
            {
              myResourceFile = fopen(ResourceFile,"ab+");
              numBytes = fwrite(DataBuffer,1,DataSize,myResourceFile);
              fclose(myResourceFile);
            }
          }
          else if (strDataName == "DKFixedView")
          {
            // string
            cout << "======DK_OD_FIXED_VIEW (Fixed View)======" << endl;
            cout << DataString << endl;
          }
          else if (item_type == DK_OD_TOC)
          {
            // binary
            cout << "======DK_OD_TOC (Large Object table of contents)======" << endl;
            myTocFile = fopen(TocFile,"ab+");
            numBytes = fwrite(DataBuffer,1,DataSize,myTocFile);
            fclose(myTocFile);
          }
        }
      }
    }
    else cout << "       Attribute Value is NULL" << endl;
  }

  DocFile = OutDir + DKString("sample") + cnt_str + DKString(".") + extension;
  if (access( LPCTSTR(DocFile), 0 ) != -1 )
    remove(LPCTSTR(DocFile));

  delete p;

  // If the resource file exists, concatenate the resource with data
  BOOL ResourceExists = ( access( LPCTSTR(ResourceFile), 0 ) != -1 );
  if ( ResourceExists )
  {
    char * buffer = new char[100000];
    myDocFile = fopen(DocFile, "wb");
    myResourceFile = fopen(ResourceFile,"rb");
    if ( myResourceFile )
    {
      while ( !feof( myResourceFile ) )
      {
        numBytes = fread( buffer, sizeof( char ), 100000, myResourceFile );
        if ( ferror( myResourceFile ) )
        {
          perror( "Resource read error" );
          break;
        }
        numBytes = fwrite(buffer,1,numBytes,myDocFile);
      }
      fclose(myResourceFile);
      remove(LPCTSTR(ResourceFile));
    }
    segmentFile = DataFile + "." + extension;
    myDataFile = fopen(segmentFile,"rb");
    // copy data
    if ( myDataFile )
    {
      while ( !feof( myDataFile ) )
      {
        numBytes = fread( buffer, sizeof( char ), 100000, myDataFile );
        if ( ferror( myDataFile ) )
        {
          perror( "Data read error" );
          break;
        }
        numBytes = fwrite(buffer,1,numBytes,myDocFile);
      }
      fclose(myDataFile);
      remove(LPCTSTR(segmentFile));
    }
    fclose(myDocFile);
    delete [] buffer;
  }
}

