/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// test list servers/ list schema / list schema attributes from OD

/*
 * Filename       : TListFolders.java
 * Description    : test DKDatastoreOD for list functions
 * Change History :
 *
 * Date        Name  Description
 * -------    -----  ---------------------------
 * 1999/05/10  bkh   Creation of test cases
 * 1999/10/10  nnc   Ported to C++
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKServerDefOD.hpp>
#include <DKDatastoreDefOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;
  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TListFoldersOD <libsrv> <userid> <pw> <connect_string> <configuration_string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  if (argc > 4 )
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];

  // must be outside try/catch block
  DKDatastoreOD* dsOD;

  if (configuration_string != "")
  {
    dsOD  = new DKDatastoreOD(configuration_string);
  }
  else
  {
    dsOD  = new DKDatastoreOD();
  }
  cout << "Datastore OD created" << endl;
  cout << "connect_string=>" << connect_string << "<=" << endl;
  cout << "configuration string=>" << configuration_string << "<=" << endl;

  DKSequentialCollection * pCol = NULL;
  dkIterator * pIter = NULL;
  DKServerDefOD * pSV = NULL;
  DKDatastoreDefOD * dsDef = NULL;

  try
  {
    DKString strServerName;
    DKString strServerType;
    DKString folderName;
    int i = 0;
    int j = 0;

    cout << "list servers" << endl;
    pCol = (DKSequentialCollection*)dsOD->listDataSources();
    pIter = pCol->createIterator();
    while (pIter->more() == TRUE)
    {
      i++;
      pSV = (DKServerDefOD*)((void*)(*pIter->next()));
      strServerName = pSV->getName();
      strServerType = pSV->getServerType();
      cout << "Server Name [" << i << "] - " << strServerName << " Server Type - " << strServerType << endl;
      delete pSV;
    }
    cout << i << " servers listed" << endl;

    cout << "connecting to datastore" << endl;
    dsOD->connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected libsrv=" << libsrv << ", userid=" << userid << endl;
    cout << "connect_string=" << connect_string << endl;
    cout << endl;

    cout << "list folders" << endl;
    dsDef = (DKDatastoreDefOD*)dsOD->datastoreDef();
    DKString* folderNames = NULL;
    long arraySize = 0;
    folderNames = dsDef->listSearchTemplateNames(arraySize);
    if (folderNames != NULL)
    {
      for (i = 0; i < arraySize; i++)
      {
        cout << "folder name #" << i+1 << " - " << folderNames[i] << endl;
      }
      cout << i << " folders listed" << endl;
      delete [] folderNames;
    }

    dsOD->disconnect();
    cout << "datastore disconnected" << endl;
  }
  catch (DKException &exc)
  {
    cout << "Error id " << exc.errorId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }

    cout << "Exception Class Name: " << exc.name() << endl;
  }

  // cleanup
  if ( pCol )
    delete pCol;
  if ( pIter )
    delete pIter;
  if ( dsOD )
    delete dsOD;

  cout << "done ..." << endl;
}

