/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

IMPORTANT:  Each sample uses commandline parameters to enable each sample to immediately
            work on any system without editing the sample file.  This sample must call
            each of those samples, passing all necessary commandline parameters.  In order
            to make this particular sample as easy to run as possible, all Resource Manager
            Configuration related constants have been hard-coded in this particular smaple.
            
            Therefore, this file must be modified before being run to match the Resource
            Manager setup preferences and configuration.  Please modify the 
            "Defaults for resource manager" section at the beginning of the "main" funciton
            in this sample.

This sample calls all of the necessary Samples to define the Sample Data Model.  The 
Sample Data Model features a Real-World Business Model Example, Insurance Model.  

In order to create Items of the Insurance Model, SInsuranceModelItemCreationICM may
be used.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include "DKConstant.h"
#include "DKConstantICM.h"
#include <DKDatastoreICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeDefinitionCreationICM.hpp"
#include "SAttributeGroupDefCreationICM.hpp"
#include "SReferenceAttrDefCreationICM.hpp"
#include "SItemTypeCreationICM.hpp"
#include "SResourceMgrDefCreationICM.hpp"
#include "SResourceMgrDefSetDefaultICM.hpp"
#include "SSMSCollectionDefCreationICM.hpp"
#include "SSMSCollectionDefSetDefaultICM.hpp"
#include "SLinkTypeDefinitionCreationICM.hpp"
#include "SSampleModelBuildICM.hpp"

/************************************************************************************************
 *          FILENAME: SSampleModelBuildICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Defines the Sample Data Model, including the Real-World Business Model
 *                    Example, Insurance Model, using all necessary Samples.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Samples
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SSampleModelBuildICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: This file must be modified before being run to match the Resource 
 *                    Manager setup preferences and configuration.  Please modify the
 *                    "Defaults for resource manager" section at the beginning of the 
 *                    "main" funciton in this sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeDefinitionCreationICM.hpp/cpp
 *                    SAttributeGroupDefCreationICM.hpp/cpp
 *                    SAttributeGroupDefRetrievalICM.hpp/cpp
 *                    SItemTypeCreationICM.hpp/cpp
 *                    SLinkTypeDefinitionCreationICM.hpp/cpp
 *                    SReferenceAttrDefCreationICM.hpp/cpp
 *                    SResourceMgrDefCreationICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SResourceMgrDefSetDefaultICM.hpp/cpp
 *                    SSMSCollectionDefCreationICM.hpp/cpp
 *                    SSMSCollectionDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefSetDefaultICM.hpp/cpp
 *                    SSampleModelBuildICMICM.hpp/cpp
 *                    SSampleModelBuildICMICM_launch.hpp/cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSampleModelBuildICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // Defaults for resource manager
        char * rmUserName       = "icmadmin";
        char * rmPassword       = "password";
        char * rmReferenceName  = "MY_RM";
        short  rmPlatform       = RM_PLATFORM_WIN;
        char * rmINETAddress    = "taj.stl.ibm.com";
        short  rmPort           = 80;
        short  rmAccessType     = RM_ACCESSTYPE_HTTP;
        char * rmAccessData     = "/icmrm/ICMResourceManager";
        short  rmTokenDuration  = 0;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                   << endl;
            cout << "  SSampleModelBuildICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."    << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SSampleModelBuildICM"           << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{

            char ** args = new char*[4];
            args[0] = new char[256];
            args[1] = new char[256];
            args[2] = new char[256];
            args[3] = new char[256];
            int  argsc = 4;
            // specify the arguments to pass to object's main method
            strcpy(args[0],argv[0]);
            strcpy(args[1],database);
            strcpy(args[2],userName);
            strcpy(args[3],password);

            char ** rmDefArgs = new char*[13];
            rmDefArgs[0] = new char[256];
            rmDefArgs[1] = new char[256];
            rmDefArgs[2] = new char[256];
            rmDefArgs[3] = new char[256];
            rmDefArgs[4] = new char[256];
            rmDefArgs[5] = new char[256];
            rmDefArgs[6] = new char[256];
            rmDefArgs[7] = new char[256];
            rmDefArgs[8] = new char[256];
            rmDefArgs[9] = new char[256];
            rmDefArgs[10]= new char[256];
            rmDefArgs[11]= new char[256];
            rmDefArgs[12]= new char[256];
            int  rmDefArgc = 12;
            // specify the arguments to pass to object's main method
            strcpy(rmDefArgs[0] ,argv[0]);
            strcpy(rmDefArgs[1] ,rmReferenceName);
            strcpy(rmDefArgs[2] ,rmINETAddress);
            strcpy(rmDefArgs[3] ,DKString(rmPort));
            strcpy(rmDefArgs[4] ,DKString(rmPlatform));
            strcpy(rmDefArgs[5] ,DKString(rmAccessType));
            strcpy(rmDefArgs[6] ,rmAccessData);
            strcpy(rmDefArgs[7] ,DKString(rmTokenDuration));
            strcpy(rmDefArgs[8] ,rmUserName);
            strcpy(rmDefArgs[9] ,rmPassword);
            strcpy(rmDefArgs[10],database);
            strcpy(rmDefArgs[11],userName);
            strcpy(rmDefArgs[12],password);

            char ** rmArgs = new char*[5];
            rmArgs[0] = new char[256];
            rmArgs[1] = new char[256];
            rmArgs[2] = new char[256];
            rmArgs[3] = new char[256];
            rmArgs[4] = new char[256];
            int  rmArgc = 5;
            // specify the arguments to pass to object's main method
            strcpy(rmArgs[0],argv[0]);
            strcpy(rmArgs[1],rmReferenceName);
            strcpy(rmArgs[2],database);
            strcpy(rmArgs[3],userName);
            strcpy(rmArgs[4],password);

            char ** smsDefArgs = new char*[6];
            smsDefArgs[0] = new char[256];
            smsDefArgs[1] = new char[256];
            smsDefArgs[2] = new char[256];
            smsDefArgs[3] = new char[256];
            smsDefArgs[4] = new char[256];
            smsDefArgs[5] = new char[256];
            int  smsDefArgc = 6;
            // specify the arguments to pass to object's main method
            strcpy(smsDefArgs[0],argv[0]);
            strcpy(smsDefArgs[1],rmReferenceName);
            strcpy(smsDefArgs[2],rmUserName);
            strcpy(smsDefArgs[3],database);
            strcpy(smsDefArgs[4],userName);
            strcpy(smsDefArgs[5],password);

            // Run Samples
            SAttributeDefinitionCreationICM::main(argsc,args);
            SAttributeGroupDefCreationICM::main(argsc,args);
            SReferenceAttrDefCreationICM::main(argsc,args);
            SItemTypeCreationICM::main(argsc,args);
            
            // Resource Mgr Setup
            SResourceMgrDefCreationICM::main(rmDefArgc,rmDefArgs);   
            SResourceMgrDefSetDefaultICM::main(rmArgc,rmArgs);   
            SSMSCollectionDefCreationICM::main(smsDefArgc,smsDefArgs);
            SSMSCollectionDefSetDefaultICM::main(rmArgc,rmArgs);
            
            // Link Type Definition
            SLinkTypeDefinitionCreationICM::main(argsc,args);
            
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
