// TEvaluateDD.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "TEvaluateDD.h"

#include "DKDatastoreDD.hpp"
#include "dkQueryableCollection.hpp"
#include "DKDocumentDefDD.hpp"
#include "DKBlobDD.hpp"
#include "DKAny.hpp"
#include "DKResults.hpp"
#include "DKEntityDefDD.hpp"
#include "DKAttrDefDD.hpp"
#include "DKException.hpp"


/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

void printDDO(DKDDO* ddo)
{
	if(ddo != NULL)
	{
		DKPid p = ddo->getPid();
		cout << "************PID************" << endl;
		cout << "PID ID: " << p.getId() << endl;
		cout << "PID DS: " << p.getDatastoreType() << endl;
		cout << "PID DS: " << p.getDatastoreName() << endl;
		cout << "PID ObjType: " << p.getObjectType() << endl;
		cout << "ID String Count: " << p.getIdStringCount() << endl;
		cout << "************DDO************" << endl;
		cout << "Object Type: " << ddo->getObjectType() << endl;
		cout << "Data Count: " << ddo->dataCount() << endl;
		for(int i = 1; i <= ddo->dataCount(); i++)
		{
			DKAny any = ddo->getData(i);
			DKString dataName = ddo->getDataName(i);
			cout << dataName << ":\t\t" << any << endl;
		}
	}
}


void printResults(DKResults* results)
{
	cout << "Cardinality: " << results->cardinality() << endl;
	
	DKSequentialIterator* iterator = NULL;
	dkIterator* it = NULL;
	it = results->createIterator();
	iterator = (DKSequentialIterator*)it;
	while(iterator->more())
	{
		DKAny* any = iterator->next();
		dkDataObjectBase* doB = (dkDataObjectBase*)any->value();
		DKDDO* doc = (DKDDO*)doB;
		if(doc != NULL)
		{
			printDDO(doc);
		}
	}
	if(iterator != NULL)
		delete iterator;
}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	for( int nbrArgs = 1; nbrArgs < argc; nbrArgs++ )
	{
		cout << "TListAttrsDD parm " << nbrArgs << "= "<< argv[nbrArgs] << endl;
	}
	if( nbrArgs < 4 )
	{
	    cout << "TListAttrsDD usage = <serverName> <userId> <pw>" << endl;
		return -1;
	}

	DKDatastoreDD* dd;
	try
	{
		DKString server = argv[1];
		DKString uid = argv[2];
		DKString pw = argv[3];

		dd = new DKDatastoreDD();
		dd->connect(server, uid, pw,"");
		if( dd->isConnected() )
		{

			//DKString qString2("((NOT([\"DocFormat\"]CONTAINS\"NOTES\"))  AND  ([\"Description\"]CONTAINS\"Request*\"))");
			//DKString queryParms("ENTITY=Diane Cabinet ;QUERY=");
			//Connecting to "http://smbdoc.austin.ibm.com/domdoc/webmaster02lib.nsf cmbadmin password"
			//Query string need be changed if we connect to the different dom doc server
			DKString qString2("([\"Title\"]CONTAINS\"Home*\")");
			DKString queryParms("ENTITY=WebPublishing ;QUERY=");
			DKString queryString = queryParms + qString2;
			cout << "Query: " << queryString << endl;
			DKAny any = dd->evaluate(queryString, DK_PARAMETRIC_QL_TYPE, NULL);
		
			DKResults* results = (DKResults*)any.value();
			printResults(results);

			dd->disconnect();
      delete results;
		}
	}
	catch( DKException dke )
	{
		for( long i = 0; i < dke.textCount(); i++ )
		{
			const char* text = dke.text(i);
			if( text != 0 )
				cout << text << endl;
		}
	}
	catch( ... )
	{
		cout << "TEvaluateDD FAILURE! Caught unexpected exception." << endl;
	}


	delete dd;
	return nRetCode;
}


