/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TdkCallback.cpp
 * Description    : dkCallback implementation for executeWithCallback()
 *
 * Change History
 * Date        Name  Description     
 * ----------  ----- -----------------------------------------------------
 * 2000/07/28  CSP   created
 *------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>

#include "TdkCallback.hpp"


/**
 * constructor
 */
TdkCallback::TdkCallback()
{
	init();
}
TdkCallback::TdkCallback(TLogUtils* aParent)
{
	init();
	setParent(aParent);
}
void TdkCallback::init()
{
	_threshold = 5;
	_cancel    = FALSE;
	_isDone    = FALSE;
	_numCalled = 0;
}
/**
 * destructor
 */
TdkCallback::~TdkCallback()
{
}

 /**
  * Sets the number of returned hits for each data callback.
  * This method should be set by applications for the datastore.
  * @param   nValue the number of hits in the result collection
  *          for each callback.
  */
void TdkCallback::setThreshold(long nValue)
{
	_threshold = nValue;
}
 /**
  * Gets the number of returned hits that the server return
  * for each callback.  The datastore will call this method for
  * each query.
  * @return the threshold value,
  */
long TdkCallback::getThreshold()
{
	//logmsg("--> TdkCallback  : getThreshold() called ");
	return _threshold;
}
 /**
  * Indicates that the user has canceled the query.
  * It should be called by the application to let the server
  * knows to stop the query.
  */
void TdkCallback::cancelQuery()
{
	_cancel = TRUE;
}
 /**
  * Query done check
  * @return	true when done
  */
DKBoolean TdkCallback::isDone()
{
	return _isDone;
}
 /**
  * Returns nHits from the backend datastore
  * @param  aDDOList a pointer to collection of DKDDO objects
  * @param  isDone   a boolean value indicating whether the server is
  *         done with the query.  That is, there is more hits.
  * @return true if query is done or user has canceled the query.
  */
DKBoolean TdkCallback::dataCallback(dkCollection* aptrDDOList, DKBoolean isDone)
{
	DKBoolean fDone = FALSE;
	_numCalled++;
	//logmsg("--> TdkCallback  : dataCallback() " + _numCalled + " time called");

	if (_cancel == TRUE || isDone == TRUE) {
		fDone   = TRUE;
		_isDone = TRUE;
	}

	if(aptrDDOList != NULL && (aptrDDOList->cardinality() > 0 || fDone != TRUE)) {
		logmsgdkCollection(aptrDDOList);
		delete aptrDDOList;
	}

	return fDone;
}
