/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TdkDatastore.cpp
 * Description    : FVT/SVT Testing program to test Query String/Objects
 *                  for execute/executeWithCallback/evaluate
 *
 * Change History
 * Date        Name  Description     
 * ----------  ----- -----------------------------------------------------
 * 2000/07/15  CSP   created
 *------------------------------------------------------------------------
 */

#include "TdkDatastore.hpp"

#include <DKDatastoreDL.hpp>
#include <DKDatastoreFed.hpp>
//#include <DKDatastoreV4.hpp>
//#include <DKDatastoreQBIC.hpp>
//#include <DKDatastoreOD.hpp>
//#include <DKDatastoreDES.hpp>


TdkDatastore::TdkDatastore()
{
	_dkDatastore = NULL;

	// Indicate this is a Console Application on Windows NT/95 for error
	// message code page conversion to OEM code page. If this was a Windows
	// GUI(Graphic User Interface) Application this would not be necessary
	// because DK_SS_WINDOWS is the default.
	DKEnvironment::setSubSystem(DK_SS_CONSOLE);
}
TdkDatastore::~TdkDatastore()
{
	setdkDatastore((dkDatastore*)NULL);
}
void TdkDatastore::setdkDatastore(dkDatastore* adkDatastore)
{
	if(_dkDatastore != NULL) {
		if(_dkDatastore->isConnected()) {
			_dkDatastore->disconnect();
		}
		delete(_dkDatastore);
		_dkDatastore = NULL;
	}
	_dkDatastore = adkDatastore;
}
void TdkDatastore::setdkDatastore(const DKString& aDBType)
{
	if(     equalsIgnoreCase(aDBType, "Fed")) {
		
		_strCurDSName = "DKDatastoreFed";
		setdkDatastore(new DKDatastoreFed());
	}
	else if(equalsIgnoreCase(aDBType, "DL")) {
		_strCurDSName = "DKDatastoreDL";
		setdkDatastore(new DKDatastoreDL());
	}
	/*
	else if(equalsIgnoreCase(aDBType, "V4")) {
		_strCurDSName = "DKDatastoreV4";
		setdkDatastore(new DKDatastoreV4());
	}
	else if(equalsIgnoreCase(aDBType, "QBIC")) {
		_strCurDSName = "DKDatastoreQBIC";
		setdkDatastore(new DKDatastoreQBIC());
	}
	*/
	setDBType(aDBType);
}
const DKString TdkDatastore::getdkDatastoreClassName()
{
	return _strCurDSName;
}
dkDatastore* TdkDatastore::getdkDatastore()
{
	if(_dkDatastore == NULL) {
		setdkDatastore("Fed");
	}
	return _dkDatastore;
}
void TdkDatastore::connect(int argc, char* args[])
{
	if( argc < 4) {
		logmsg("usage :\n  connect DBName UID   PWD   connectString");
		logmsg(    "  e.g. connect feddb  edms1 edms1 add");
	}
	else {
		char* lConnectStr = (argc >= 4) ? args[4] : "";
		getdkDatastore()->connect(args[1], args[2], args[3], lConnectStr);
		logmsg("                 Connected successfully");
	}
}
void TdkDatastore::disconnect()
{
	getdkDatastore()->disconnect();
	logmsg("                 Disconnected successfully");
}
void TdkDatastore::listDatastoreInfo()
{
	logmsg("-->Loaded      : ", _strCurDSName);
	logmsg("-->DBName UID  : ", getdkDatastore()->datastoreName() + " " + 
			getdkDatastore()->userName());
	logmsg("-->Connected?  : ", DKString(getdkDatastore()->isConnected() ? "true" : "false"));
}

void TdkDatastore::execute(const DKString& aQueryString,
		const short nQLType, const DKNVPair* aParms)
{
	logmsg("-->execute(QueryString) called");
	dkResultSetCursor* lCursor = getdkDatastore()->execute(aQueryString, nQLType, aParms);
	logmsgdkResultSetCursor(lCursor);
}
void TdkDatastore::execute(dkQuery* aQuery)
{
	logmsg("-->execute(QueryObject) called");
	dkResultSetCursor* lCursor = getdkDatastore()->execute(aQuery);
	logmsgdkResultSetCursor(lCursor);
}
void TdkDatastore::executeWithCallback(const DKString& aQueryString,
		const short nQLType, const DKNVPair* aParms)
{
	logmsg("-->executeWithCallback(QueryString) called");
	TdkCallback* ptrTdkCallback = getTdkCallback();
	getdkDatastore()->executeWithCallback(aQueryString, nQLType, aParms, ptrTdkCallback);
	waitForTdkCallback(ptrTdkCallback);
}
void TdkDatastore::executeWithCallback(dkQuery* aQuery)
{
	logmsg("-->executeWithCallback(QueryObject) called");
	TdkCallback* ptrTdkCallback = getTdkCallback();
	getdkDatastore()->executeWithCallback(aQuery, ptrTdkCallback);
	waitForTdkCallback(ptrTdkCallback);
}
void TdkDatastore::evaluate(const DKString& aQueryString,
		const short nQLType, const DKNVPair* aParms)
{
	logmsg("-->evaluate(QueryString) called");
	dkCollection* lCollection = getdkDatastore()->evaluate(aQueryString, nQLType, aParms);
	logmsgdkCollection(lCollection);
}
void TdkDatastore::evaluate(dkQuery* aQuery)
{
	logmsg("-->evaluate(QueryObject) called");
	dkCollection* lCollection = getdkDatastore()->evaluate(aQuery);
	logmsgdkCollection(lCollection);
}
void TdkDatastore::waitForTdkCallback(TdkCallback* aTdkCallback)
{
	if(aTdkCallback != NULL) {
		// need to wait until it finishes
		DKBoolean fDone;

		unsigned long lstartTime  = clock();
		unsigned long lcurTime    = 0;
		unsigned long lelapseTime = 0;

		while((fDone = aTdkCallback->isDone()) != TRUE) {
			lcurTime    = clock();
			lelapseTime = (lcurTime - lstartTime) / CLOCKS_PER_SEC; // seconds

			if( lelapseTime > CALLBACK_WAIT_TIME_SECS ) {
				sprintf(getMsgBuff(), "callback timedout.  Waited for : %d seconds", 
						CALLBACK_WAIT_TIME_SECS);
				logmsgError( getMsgBuff());
				break;
			}
		}
		delete aTdkCallback;
	}
}
const DKString TdkDatastore::getAvailableCommands()
{
	//super.getAvailableCommands();
	char* rc = 
			"    TdkDatastore--------------------------------------------------\n\t"
			"listDatastoreInfo   () \n\t"
			"setdkDatastore      (Fed/DL/TS/QBIC) \n\t"
			"connect             (_DatabaseName_, _uid_, _pwd_, _ConnectStr_) \n\t"
			"disconnect          () \n\t"
			"setMapServer        (_MapServerName_) \n\t"
			"enableQueryMapServer(true/false) \n\t"
			"execute             (STR/OBJ, _QueryString_, PQ/TQ/PQorTQ/PQandTQ) \n\t"
			"executeWithCallback (STR/OBJ, _QueryString_, PQ/TQ/PQorTQ/PQandTQ) \n\t"
			"evaluate            (STR/OBJ, _QueryString_, PQ/TQ/PQorTQ/PQandTQ) \n\t"
			"executeTemplates    () \n\t"
			"executeTemplates    (RSC/CBK/RCL) \n"
			"";
	return rc;
}
void TdkDatastore::processCommand(const DKString& aCmd)
{
	dkCollection* ldkCollection = getTokens(aCmd);
	dkIterator*   lIter         = ldkCollection->createIterator();
	int           nArgc         = ldkCollection->cardinality();
	char**        args          = new char*[nArgc];
	DKAny         lObjAny;

	try {
		for(short nInd = 0; lIter->more(); nInd++) {
			lObjAny = *(lIter->next());
			args[nInd] = (char*)((DKString)lObjAny);
		}

		short nQLType = DK_UNKNOWN_QL_TYPE;
		if(nArgc > 3) {
			nQLType = getQLType(args[3]);
		}
	
		if(    equalsIgnoreCase(args[0], "setdkDatastore")) {
			// command format
			//     setdkDatastore client/server Fed/DL/QBIC/DES
			//     to be in sync with java code args[1] is ignored
			setdkDatastore(args[2]);
			if(getdkDatastore() != NULL) {
				sprintf(getMsgBuff(), "                 %s loaded successfully\n", 
						(const char*)_strCurDSName);
				logmsgBuff();
			}
			else {
				logmsgError("creating datastore");
			}
		}
		else if(equalsIgnoreCase(args[0], "connect")) {
			connect(nArgc, args);
		}
		else if(equalsIgnoreCase(args[0], "listDatastoreInfo")) {
			listDatastoreInfo();
		}
		else if(equalsIgnoreCase(args[0], "disconnect")) {
			disconnect();
		}
		else if(equalsIgnoreCase(args[0], "execute")) {
			strErrInput = args[2];
			if(     equalsIgnoreCase(args[1], "STR")) {
				execute(args[2], nQLType);
			}
			else if(equalsIgnoreCase(args[1], "OBJ")) {
				execute(convertToQuery(nArgc, args));
			}
		}
		else if(equalsIgnoreCase(args[0], "evaluate")) {
			strErrInput = args[2];
			if(     equalsIgnoreCase(args[1], "STR")) {
				evaluate(args[2], nQLType);
			}
			else if(equalsIgnoreCase(args[1], "OBJ")) {
				evaluate(convertToQuery(nArgc, args));
			}
		}
		else if(equalsIgnoreCase(args[0], "executeWithCallback")) {
			strErrInput = args[2];
			if(     equalsIgnoreCase(args[1], "STR")) {
				executeWithCallback(args[2], nQLType);
			}
			else if(equalsIgnoreCase(args[1], "OBJ")) {
				executeWithCallback(convertToQuery(nArgc, args));
			}
		}
		else {
			TLogUtils::processCommand(aCmd);
		}
	}
	catch(DKException aDKExp) {
		logmsgDKException(aDKExp);
	}
	catch(char* strExp) {
		logmsgError(strExp);
	}
	catch(...) {
		delete[] args;
		delete   ldkCollection;
		logmsgError("Got unknown exception");

		throw;
	}

	delete[] args;
	delete   ldkCollection;
}
short TdkDatastore::getQLType(const DKString& astrQLType)
{
	short shQLType = DK_UNKNOWN_QL_TYPE;
	if(getDBType() == SH_DB_FED) {
		shQLType = DK_CM_FEDERATED_QL_TYPE;
	}
	else {
		if(     equalsIgnoreCase(astrQLType, "TQ")) {
			shQLType = DK_TEXT_QL_TYPE;
		}
		else if(equalsIgnoreCase(astrQLType, "PQ")) {
			shQLType = DK_PARAMETRIC_QL_TYPE;
		}
		else {
			shQLType = DK_COMBINED_QL_TYPE;
		}
	}
	return shQLType;
}
short TdkDatastore::getDBType()
{
	return _nDBType;
}
void TdkDatastore::setDBType(const DKString& aDBType)
{
	_nDBType     = SH_DB_UNKNOWN;

	if(equalsIgnoreCase(aDBType, "FED") ) {
		_nDBType   = SH_DB_FED;
		strcpy(_strDBName, "Fed");
	}
	else {
		if(     equalsIgnoreCase(aDBType, "DD"  ) ) {
			_nDBType   = SH_DB_DD;
		}
		else if(equalsIgnoreCase(aDBType, "DL"  ) ) {
			_nDBType   = SH_DB_DL;
		}
		else if(equalsIgnoreCase(aDBType, "IP"  ) ) {
			_nDBType   = SH_DB_IP;
		}
		else if(equalsIgnoreCase(aDBType, "JDBC") ) {
			_nDBType   = SH_DB_JDBC;
		}
		else if(equalsIgnoreCase(aDBType, "OD"  ) ) {
			_nDBType   = SH_DB_OD;
		}
		else if(equalsIgnoreCase(aDBType, "QBIC") ) {
			_nDBType   = SH_DB_QBIC;
		}
		else if(equalsIgnoreCase(aDBType, "TS"  ) ) {
			_nDBType   = SH_DB_TS;
		}
		else if(equalsIgnoreCase(aDBType, "V4"  ) ) {
			_nDBType   = SH_DB_V4;
		}

		strcpy(_strDBName, aDBType);
		toUpperCase(DKString(_strDBName));
	}
}
dkQuery* TdkDatastore::convertToQuery(int argc, char* args[])
{
	dkQuery* rc = NULL;

	if(argc > 0 && args != NULL) {
		if(     equalsIgnoreCase(args[1], "OBJ")) {
			if(argc >= 4) {
				rc = getdkDatastore()->createQuery(args[2], getQLType(args[3]), NULL);
			}
		}
	}

	if(rc == NULL) {
		logmsgError("creating QueryObject");
	}

	return rc;
}
TdkCallback* TdkDatastore::getTdkCallback()
{
	return new TdkCallback(this);
}

void main(int argc, const char *argv[])
{
	TdkDatastore* lTdkDatastore = new TdkDatastore();
	lTdkDatastore->processCommandLineArgs(argc, argv);

	/*
	TdkDatastore* lTdkDatastore = new TdkDatastore();
	lTdkDatastore->connect(argc, argv);

	if(lTdkDatastore->getdkDatastore()->isConnected()) {
		DKString cmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,MAX_RESULTS=5,";
				cmd += "COND=(DLSEARCH_DocType <> NULL));";
				cmd += "OPTION=(CONTENT=YES;TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";

		lTdkDatastore->execute(cmd);
	}
	*/ 
}
