/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import  com.ibm.mm.beans.*;
import  com.ibm.mm.beans.util.*;

import  java.beans.*;
import  java.util.*;
import  java.io.*;

/**
 * This sample application will retrieve documents and update their
 * attributes values.
 * <p>
 * This application will use the following beans:
 * <pre>
 * CMBConnection
 * CMBSchemaManagement
 * CMBSearchTemplate
 * CMBQueryService
 * CMBSearchResults
 * CMBDataManagement
 * util/CMBTraceLog
 * </pre>
 * Use the following functions:
 * <pre>
 * a.  Connect/disconnect
 * b.  get a list of search templates names
 * c.  pick a search template (nvTemplate), get its search criteria
 * d.  fill in the values and submit a query (asynchronously - query with callback)
 * e.  print out the result in search results bean
 * f.  loop thru the search results, pick the first CM document and update
 *     its attribute values to ones.  Then update it back to its original values.
 * g.  disconnect
 * </pre>
 */
public  class DemoItemAttr
{
    int                 docCounter = 0;
    UtilPersistent      per        = new UtilPersistent();
    UtilSimpleUI        ui         = new UtilSimpleUI();
    CMBSchemaManagement schemaBean = null;
    CMBSearchResults    resultBean = null;
    CMBConnection       connBean   = null;
    CMBQueryService     queryBean  = null;
    CMBSearchTemplate   stBean     = null;
    CMBDataManagement   dataBean   = null;

    public DemoItemAttr()
    {
        try {
            System.out.println("Creating CMBUserManagement bean using " +
                     "ser/class/jar specified in the classpath.");
            connBean = (com.ibm.mm.beans.CMBConnection) Beans.instantiate(null,
                     "com.ibm.mm.beans.CMBConnection");

            System.out.println("Creating CMBSchemaManagement bean from CMBConnection bean...");
            schemaBean = connBean.getSchemaManagement();

            System.out.println("Creating CMBSchemaManagement bean from CMBConnection bean...");
            queryBean  = connBean.getQueryService();
            schemaBean.setCacheEnabled(true);

            System.out.println("Creating CMBDataManagement bean from CMBConnection bean...");
            dataBean   = connBean.getDataManagement();

            resultBean = (com.ibm.mm.beans.CMBSearchResults) Beans.instantiate(null,
                      "com.ibm.mm.beans.CMBSearchResults");
            resultBean.setConnection(connBean);

        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void
    checkConnBeanProperty()
    {
        try {
            if (connBean.getServerName().length() == 0) {
                String srv = ui.getString("Server name is empty, " +
                                        "please enter the server name:");
                connBean.setServerName(srv);
            }
            if (connBean.getUserid().length() == 0) {
                String uid = ui.getString("User ID is empty, please enter the user ID:");
                connBean.setUserid(uid);
            }
            if (connBean.getPassword().length() == 0) {
                String pwd = ui.getString("Password is empty, please enter the password:");
                connBean.setPassword(pwd);
            }
        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void
    connect()
    {
        checkConnBeanProperty();

        try {
            System.out.println("Connecting...");
            connBean.connect();
            System.out.println("OK, connected\n");
        }
        catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    public void listTemplates()
    {
        try {
            String[] stNames = schemaBean.getSearchTemplateNames();
            System.out.println("There are " + stNames.length + " search templates:");
            for (int i=0; i<stNames.length; i++) {
                System.out.println("\t" + stNames[i]);
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void listSTCriteria()
    {
        try {
            String stName = ui.getString("Please enter the search template name:");
            CMBSearchTemplate stObj = schemaBean.getSearchTemplate(stName);
            System.out.println("Search template '" + stName + "' retrieved...");
            String[] critNames = stObj.listSearchCriteriaNames();
            System.out.println("There are " + critNames.length + " criteria: ");
            for (int i=0; i<critNames.length; i++) {
                System.out.println("\t" + critNames[i]);
                CMBSTCriterion critObj = stObj.getSearchCriterion(i);
                System.out.println("\t\tdisplayName:  " + critObj.getDisplayName());
                System.out.println("\t\tattrName:     " + critObj.getAttrName());
                System.out.println("\t\tdisplayWidth: " + critObj.getDisplayWidth());
                System.out.println("\t\ttype:         " + critObj.getType());
                System.out.println("\t\tdefaultValue: " + critObj.getDefaultValue());
                System.out.println("\t\tdefaultOp:    " + critObj.getDefaultOperator());
                short[] validOps = critObj.getValidOperators();
                System.out.println("\t\tvalidOps:");
                for (int j=0; j<validOps.length; j++) {
                    switch((int)validOps[j]) {
                        case CMBBaseConstant.CMB_OP_EQUAL:
                            System.out.println("\t\t\t.EQ."); break;
                        case CMBBaseConstant.CMB_OP_NOT_EQUAL:
                            System.out.println("\t\t\t.NEQ."); break;
                        case CMBBaseConstant.CMB_OP_LESS_EQUAL:
                            System.out.println("\t\t\t.LEQ."); break;
                        case CMBBaseConstant.CMB_OP_LESS:
                            System.out.println("\t\t\t.LT."); break;
                        case CMBBaseConstant.CMB_OP_GREATER_EQUAL:
                            System.out.println("\t\t\t.GEQ."); break;
                        case CMBBaseConstant.CMB_OP_GREATER:
                            System.out.println("\t\t\t.GT."); break;
                        case CMBBaseConstant.CMB_OP_BETWEEN:
                            System.out.println("\t\t\t.BETWEEN."); break;
                        case CMBBaseConstant.CMB_OP_NOT_BETWEEN:
                            System.out.println("\t\t\t.NOTBETWEEN."); break;
                        case CMBBaseConstant.CMB_OP_IN:
                            System.out.println("\t\t\t.IN."); break;
                        case CMBBaseConstant.CMB_OP_NOT_IN:
                            System.out.println("\t\t\t.NOTIN."); break;
                        case CMBBaseConstant.CMB_OP_LIKE:
                            System.out.println("\t\t\t.LIKE."); break;
                        case CMBBaseConstant.CMB_OP_NOT_LIKE:
                            System.out.println("\t\t\t.NOTLIKE."); break;
                        default:  break;
                    }
                }
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void runQuery()
    {
        try {
            boolean runQry = false;
            String stName = ui.getString("Please enter the search template name:");
            CMBSearchTemplate stObj = schemaBean.getSearchTemplate(stName);
            if (stObj == null) {
                System.out.println("Unknown template name, please try again");
                return;
            }

            String[] critNames = stObj.listSearchCriteriaNames();
            System.out.println("There are " + critNames.length + " criteria: ");
            for (int i=0; i<critNames.length; i++) {
                System.out.println("\t" + critNames[i]);
            }

            while (!runQry) {
                System.out.println("Selections:");
                System.out.println("1. Specify a search criterion");
                System.out.println("2. Run query ...");
                switch (ui.getInteger("Enter your selection:")) {
                    case 2: runQry = true;                break;
                    case 1: getSearchCriterion(stObj);    break;
                }
            }

            stObj.addCMBSearchReplyListener(resultBean);
            stObj.setAsynchSearch(false);
            queryBean.setAsynchSearch(false);
	    resultBean.setCurrentSearchTemplate(stObj);

            boolean contQuery = true;
    	    while (contQuery) {
                try {
                    queryBean.runQueryWithCursor(stObj);
                    contQuery = false;
                    System.out.println("returned from the query...");
                }
                catch (CMBConnectFailedException ex) {
                    String serverName = (String)ex.getErrorData();
                    System.out.println("Connection to " + serverName + " with id= " + ex.getUserid() + " failed...");
                    int errCode = ex.getErrorId();
                    if (errCode == CMBConnectFailedException.CMB_CONNECT_INVALID_USERID_PASSWD) {
                        String uid = ui.getString("Please enter userid for server " + serverName + ": ");
                        String pwd = ui.getString("Please enter password: ");
                        connBean.requestConnection(serverName, uid, pwd, "", "");
                    }
                    else {
                        boolean skipServer = ui.getBoolean("Server " + serverName + " is not available, do you want to continue?");
                        if (skipServer) {
                            stObj.skipServer(serverName);
                        }
                    }
                }

            }


            resultBean.newResults(queryBean.getResults());
            CMBItem[] itemList = resultBean.getItem();
            System.out.println("There are " + itemList.length + " hits ....");
            for (int k=0; k<itemList.length; k++) {
                CMBItem hit = (CMBItem)itemList[k];
                System.out.println("Hit item #" + k + ", pidString=" + hit.getPidString());
                try {
                    String[] values = hit.getAttrValues();
                    String[] names  = hit.getAttrNames();
                    for (int l=0; l<values.length; l++)
                        System.out.println("\t" + names[l] + ": " + values[l]);
                    if (docCounter < 1 && hit.getItemType()==CMBBaseConstant.CMB_TYPE_DOCUMENT) {
                        docCounter++;
                        dataBean.setDataObject(hit);
                        CMBObject[] objList = dataBean.getContent();
                        for (int m=0; m<objList.length; m++) {
                            byte[] bdata = objList[m].getData();
                            if (bdata != null)
	                        System.out.println("bdata length = " + bdata.length);
                            try {
                                String cwd = System.getProperty("user.dir");
                                String separator = System.getProperty("file.separator");
                                String tempfileName = cwd.concat(separator + "OBJ" + System.currentTimeMillis());
                                FileOutputStream tempfileStream = new FileOutputStream(tempfileName);
                                tempfileStream.write(bdata);
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                        }

                        int objCount = dataBean.getContentCount();
                        System.out.println("Item has obj count= " + objCount);

                        String entityName = stObj.getEntityName();
                        String[] attrNames = schemaBean.getEntityAttrNames(entityName);
                        short[] attrTypes = new short[attrNames.length];
                        for (int m=0; m<attrNames.length; m++) {
                            System.out.print("\tAttribute Name= " + attrNames[m]);
                            CMBAttribute attr = schemaBean.getEntityAttrs(entityName, attrNames[m]);
                            attrTypes[m] = attr.getType();
                            System.out.println("\tAttribute Type=" + attrTypes[m] + "\tsize=" + attr.getSize());
                        }
                        boolean notdone = true;
                        String[] attrVals = new String[attrNames.length];
                        while (notdone) {
                            String aName = ui.getString("enter attribute name: [-1 when done]" );
                            if (aName.equals("-1"))
                                notdone = false;
                            else {
                                for (int m=0; m<attrNames.length; m++) {
                                    if (aName.equals(attrNames[m])) {
                                        attrVals[m] = ui.getString("Enter attribute value: ");
                                        m = attrNames.length;  //to break out of for loop
                                    }
                                }
                            }
                        }

                        hit.removeAllAttrs();
                        hit.setAttrNames(attrNames);
                        hit.setAttrValues(attrVals);
                        hit.setAttrTypes(attrTypes);

                        //prepare for update item
                        dataBean.setDataObject(hit);
                        dataBean.updateItem();

                        //Now retrieve back to see if the values had been changed in the server
                        dataBean.setDataObject(hit);
                        dataBean.retrieveItem();
                        String[] newValues = hit.getAttrValues();
                        String[] newNames  = hit.getAttrNames();
                        for (int i=0; i<newValues.length; i++) {
                            System.out.println("\tChanged attrValue for " + newNames[i] + ": " + newValues[i]);
                        }
                    }
                }
                catch (CMBException ex) {
                     Object data = ex.getErrorData();
                     if (data != null && data instanceof Exception)
                         ((Exception)data).printStackTrace();
                     else
                        ex.printStackTrace();
                }
            }
        }
        catch (CMBConnectFailedException ex) {
            System.out.println("Connection to back-end server failed...");
            ex.printStackTrace();
        }
        catch (CMBNoConnectionException ex) {
            System.out.println("There is no current connection, please logon..");
            ex.printStackTrace();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void getSearchCriterion(CMBSearchTemplate stObj)
    {
        try {
            String critName  = ui.getString("Please enter criterion name: " );
            Vector critList  = stObj.getSearchCriteria();
            CMBSTCriterion critObj = null;
            for (int i=0; i<critList.size(); i++) {
                critObj = (CMBSTCriterion)critList.elementAt(i);
                if (critObj.getName().equals(critName))
                    break;
            }

            if (critObj == null) {
                System.err.println("Invalid criterion name, please try again...");
                return;
            }

            int    critOp    = ui.getInteger("Please enter search operator: " +
                "\nEQ: 1\t\tNEQ: 2\t\tLEQ: 3\t\tLT: 4" +
                "\nGEQ: 5\t\tGT: 6\t\tBETWEEN: 7\tNOTBETWEEN: 8" +
                "\nIN: 9\t\tNOTIN: 10\tLIKE: 11\tNOTLIKE: 12\n");
            switch (critOp) {
                case 1:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_EQUAL); break;
                case 2:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_EQUAL); break;
                case 3:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LESS_EQUAL); break;
                case 4:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LESS); break;
                case 5:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER_EQUAL); break;
                case 6:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER); break;
                case 7:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_BETWEEN); break;
                case 8:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_BETWEEN); break;
                case 9:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_IN); break;
                case 10:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_IN); break;
                case 11:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LIKE); break;
                case 12:
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_LIKE); break;
                default:
                    break;
            }
            if (critOp == 7 || critOp == 8) {
                String critVal1 = ui.getString("Please enter 1st search value: ");
                String critVal2 = ui.getString("Please enter 2nd search value: ");
                String[] critVals = new String[2];
                critVals[0] = critVal1;
                critVals[1] = critVal2;
                critObj.setValues(critVals);
            }
            else if (critOp == 9 || critOp == 10) {
                Vector vals = new Vector();
                boolean notDone = true;
                while (notDone) {
                    String val = ui.getString("Please enter value: (-1 if done entering)");
                    if (val.equals("-1"))
                        notDone = false;
                    else
                        vals.addElement(val);
                }
                String[] critVals = new String[vals.size()];
                vals.copyInto(critVals);
                critObj.setValues(critVals);
            }
            else {
                String critVal = ui.getString("Please enter search value: ");
                critObj.setValue(critVal);
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public  static void main(String argv[])
    {
        String  testName  ="IBM Enterprise Information Portal Sample Application: DemoItemAttr";
        String  copyright ="(c) Copyright IBM Corp. 1998,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(testName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI    ui          = new UtilSimpleUI();
        DemoItemAttr    psObj       = new DemoItemAttr();
        boolean         loopControl = true;

        System.out.println("Going to connect to the server...");
        psObj.connect();

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. List Search Templates");
            System.out.println("2. List Search Criteria");
            System.out.println("3. Run query and update item attributes...");
            switch (ui.getInteger("Enter your selection:")) {
                case 0: loopControl = false;     break;
                case 1: psObj.listTemplates();   break;
                case 2: psObj.listSTCriteria();  break;
                case 3: psObj.runQuery();        break;
            }
        }
        try {
            psObj.connBean.disconnect();
            System.out.println("Bye.");
            System.exit(0);
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}

