/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// File Name   = DemoSTCriteria.java
// Description = Demonstrate CMBSchemaManagement bean functionality
// Author      = afic
// Date        = 03/29/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;

import java.beans.*;

public class DemoSTCriteria {
    UtilPersistent per = new UtilPersistent();
    UtilSimpleUI ui = new UtilSimpleUI();
    CMBSchemaManagement schemaBean = null;

    DemoSTCriteria() {
        try {
            System.out.println("Creating CMBSchemaManagement bean using " + "ser/class/jar specified in the classpath.");
            schemaBean = (com.ibm.mm.beans.CMBSchemaManagement) Beans.instantiate(null, "com.ibm.mm.beans.CMBSchemaManagement");
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void changeProperty() {
        if (ui.getBoolean("Enable the schema cache capability?"))
            schemaBean.setCacheEnabled(true);
        else
            schemaBean.setCacheEnabled(false);

        if (ui.getBoolean("Enable the schema trace capability?"))
            schemaBean.setTraceEnabled(true);
        else
            schemaBean.setTraceEnabled(false);
    }

    void run() {
        System.out.println("this is my line...");
        int count;
        try {
            // get server names
            System.out.println("Getting server names...");
            String svNames[] = schemaBean.getServerNames();
            count = svNames.length;
            System.out.println("Done, server count is: " + count);
            for (int i = 0; i < count; ++i)
                System.out.println("Server[" + i + "] = " + svNames[i]);
            System.out.println("");

            // get search template names
            System.out.println("Getting search template names...");
            String stNames[] = schemaBean.getSearchTemplateNames();
            count = stNames.length;
            System.out.println("Done, search template count is: " + count + "....");

            for (int i = 0; i < count; ++i) {
                System.out.println("Template[" + i + "] = " + stNames[i]);

                System.out.println("Starting getting each search template and its criteria ...");
                CMBSearchTemplate stObj = schemaBean.getSearchTemplate(i);
                System.out.println("entityName   = " + stObj.getEntityName());
                System.out.println("templateName = " + stObj.getName());

                String[] critNames = stObj.listSearchCriteriaNames();
                for (int k = 0; k < critNames.length; k++) {
                    System.out.println("critName = " + critNames[k]);
                }

                if (critNames == null)
                    System.out.println("critNames is null.....");
                else if (critNames != null) {
                    System.out.println("There are " + critNames.length + " criteria in this search template");
                    java.util.Vector critList = stObj.getSearchCriteria();
                    for (int j = 0; j < critList.size(); j++) {
                        System.out.println("Criteria[" + j + "] = " + critNames[j]);
                        CMBSTCriterion crit = (CMBSTCriterion) critList.elementAt(j);
                        //CMBSTCriterion crit = stObj.getSearchCriterion(j);
                        System.out.println("\t\tCriteria Name:  " + crit.getName());
                        System.out.println("\t\tDisplay  Name:  " + crit.getDisplayName());
                        System.out.println("\t\tDisplay  Width: " + crit.getDisplayWidth());
                        System.out.println("\t\tDefault  Value: " + crit.getDefaultValue());
                        System.out.println("\t\tDefault  Op:    " + crit.getDefaultOperator());
                        System.out.println("\t\tCriteria Type:  " + crit.getType());
                    }
                }
                System.out.println("entityName   = " + stObj.getEntityName());
            }

            System.out.println("");

        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void load() {
        schemaBean = (CMBSchemaManagement) per.load();
    }

    void save() {
        per.save(schemaBean);
    }

    public static void main(String argv[]) {
        String demoName = "IBM CMB Demonstration program: DemoSchema";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(demoName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI ui = new UtilSimpleUI();
        DemoConnect connDemo = new DemoConnect();
        DemoSTCriteria demo = new DemoSTCriteria();
        boolean loopControl = true;

        demo.schemaBean.setConnection(connDemo.connBean);
        System.out.println("Going to connect to the server...");
        connDemo.connect();

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. Re-load the CMBSchemaManagement bean from a file");
            System.out.println("2. Change property values");
            System.out.println("3. Run the bean");
            System.out.println("4. Save this CMBSchemaManagement bean to a file");
            switch (ui.getInteger("Enter your selection:")) {
                case 0 :
                    loopControl = false;
                    break;
                case 1 :
                    demo.save();
                    break;
                case 2 :
                    demo.changeProperty();
                    break;
                case 3 :
                    demo.run();
                    break;
                case 4 :
                    demo.save();
                    break;
            }
        }

        connDemo.disconnect();
        System.out.println("Bye.");
    }
}