/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// Class Name  = SampleGuiTemplate
// Description = Sample visual search template bean
// Author      = mimivo
// Date        = 04/22/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusListener;
import java.beans.*;
import java.io.*;
import java.util.Vector;

//import  javax.swing.*;
//import  javax.swing.border.*;

import javax.swing.*;
import javax.swing.border.*;

//*****************************************************************************
// class/interface definition
//*****************************************************************************
/**
 * Sample GUI bean to prompt for user ID and password. Can be used with CM
 * connection bean to perform the functions.
 *
 * <pre>
 *   standalone properties  timeout, asynchSearch.
 *   source of events       CMBSearchRequestEvent     - to search
 *                          CMBSearchReply
 * </pre>
 */
public class SampleGuiTemplate
    implements Serializable, TemplateChangeListener, CMBSearchReplyListener, CMBUtilConstant {

    String[] opStrArray =
        {
            "",
            "==",
            "<>",
            "> ",
            "< ",
            "like",
            "AND",
            "OR",
            "NOT",
            "not like",
            "between",
            "in",
            "not in",
            ">=",
            "<=",
            "not between" };

    //*****************************************************************************
    // constructor
    //*****************************************************************************
    /**
    *  Default constructor
    */
    public SampleGuiTemplate() {
        requestListenerVector = new Vector();
        replyListenerVector = new Vector();
        initGUI(null);
    }

    /**
    * Constructor with search template
    * @param st search template object
    */
    public SampleGuiTemplate(CMBSearchTemplate st) {
        requestListenerVector = new Vector();
        replyListenerVector = new Vector();
        initGUI(st);
        _template = st;
        addCMBSearchRequestListener(_template);
        _template.addCMBSearchReplyListener(this);
        _template.setAsynchSearch(false);
    }

    /**
    * Destructor,  dispose the window.
    */
    public void finalize() {
        frame.dispose();
    }

    public void onTemplateChange(TemplateChangeEvent evt) {
        String stName = (String) evt.getData();
        CMBSchemaManagement schemaBean = (CMBSchemaManagement) evt.getExtraData();
        System.out.println("onTemplateChange().......");
        System.out.println("templateName = " + stName);
        if (stName == null || stName.length() == 0) {
            System.out.println("Template name is null === returning....");
            return;
        }

        if (evt.getID() == TemplateChangeEvent.SAMPLE_TEMPLATE_SELECTED) {
            try {
                CMBSearchTemplate stObj = schemaBean.getSearchTemplate(stName);
                setTemplate(stObj);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
    * Implementation of search reply service method. Will get the search result
    * from the event. See CMBSearchReplyEvent for result type.
    * @param  evt a CMBSearchReplyEvent
    */
    public synchronized void onCMBSearchReply(CMBSearchReplyEvent evt) {
        System.out.println("##################################################");
        System.out.println("About to post search reply event to CMBSearchResults");
        fireCMBSearchReplyEvent(evt);
    }

    //*****************************************************************************
    // Event source: listener registration, unregistration, and notify methods
    //*****************************************************************************
    // CMBSearchEvent
    //*****************************************************************************
    /**
    * CMBSearchEvent registration method.
    * It ignores a registration request if the listener is already registered.
    * @param listener listener to be registered
    */
    public void addCMBSearchRequestListener(CMBSearchRequestListener listener) {
        if (!requestListenerVector.contains(listener)) {
            System.out.println("adding CMBSearchRequestListener listener....");
            requestListenerVector.addElement(listener);
        }
    }

    /**
    * CMBSearchRequestEvent unregistration method.
    * @param listener listener to be unregistered
    */
    public void removeCMBSearchRequestListener(CMBSearchRequestListener listener) {
        if (requestListenerVector.contains(listener))
            requestListenerVector.removeElement(listener);
    }

    /**
    * CMBSearchRequestEvent unregistration method.
    * @param listener listener to be unregistered
    */
    public void removeAllCMBSearchRequestListeners() {
        System.out.println("Removing all listeners##########");
        if (requestListenerVector != null && requestListenerVector.size() > 0)
            requestListenerVector.removeAllElements();
    }

    /*
    * CMBSearchRequestEvent notification method.
    * The event firing order is based on the order of registration.
    * @param evt the CMBSearchRequestEvent
    */
    private void postCMBSearchRequest(CMBSearchRequestEvent evt) {
        // VisualCafe 3.0 support 04/23/99
        searchRequestEvent = evt;
        // VisualCafe 3.0 support 04/23/99

        Vector v;
        synchronized (this) {
            v = (Vector) requestListenerVector.clone();
        }

        int cnt = v.size();
        CMBSearchRequestListener listener = null;

        System.out.println("There are " + cnt + " listeners for search request...");
        for (int i = 0; i < cnt; ++i) {
            listener = (CMBSearchRequestListener) v.elementAt(i);
            listener.onCMBSearchRequest(evt);
        }
    }

    /*
    * CMBSearchRequestEvent notification method.
    * @param id         the event type id
    */
    private void postCMBSearchRequest(int id) {
        if (id == CMBSearchRequestEvent.CMB_REQUEST_SEARCH_SYNCH_CURSOR) {
            System.out.println("Before posting to search template to search...");
            int nCrit = txtFieldList.size();
            for (int i = 0; i < nCrit; i++) {
                JTextField txtField = (JTextField) txtFieldList.elementAt(i);
                JComboBox opField = (JComboBox) opFieldList.elementAt(i);
                CMBSTCriterion crit = (CMBSTCriterion) critList.elementAt(i);
                String val = txtField.getText();
                if (val != null && val.length() != 0) {
                    crit.setValue(val);
                    String value1 = (String) opField.getSelectedItem();
                    for (int j = 0; j < opStrArray.length; j++) {
                        if (value1.equals(opStrArray[j])) {
                            crit.setOperator((short) j);
                            break;
                        }
                    }
                }
            }
            postCMBSearchRequest(new CMBSearchRequestEvent(this, id, _template));
        } else if (id == CMBSearchRequestEvent.CMB_REQUEST_CANCEL_SEARCH) {
            postCMBSearchRequest(new CMBSearchRequestEvent(this, id));
        }
    }

    /**
     *	Get the CMBSearchRequestEvent object.
     *  @return CMBSearchRequestEvent object
     */
    public CMBSearchRequestEvent getCMBSearchRequestEvent() {
        return searchRequestEvent;
    }

    /**
     *  CMBSearchReplyEvent registration method.
     *	Adds a new listener object to listener list
     *  @param aListener a CMBSearchReplyListener object to be added
     */
    public void addCMBSearchReplyListener(CMBSearchReplyListener aListener) {
        System.out.println("#########################");
        System.out.println("Adding CMBSearchReplyListener ...");
        if (replyListenerVector == null)
            replyListenerVector = new java.util.Vector();
        if (!replyListenerVector.contains(aListener)) {
            replyListenerVector.addElement(aListener);
        }
    }

    /**
     *  CMBSearchReplyEvent unregistration method.
     *  @param aListener a listener to be unregistered
     */
    public void removeCMBSearchReplyListener(CMBSearchReplyListener aListener) {
        System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%%");
        System.out.println("Removing CMBSearchReplyListener ...");

        if (replyListenerVector == null)
            return;

        if (replyListenerVector.contains(aListener))
            replyListenerVector.removeElement(aListener);
    }

    private void fireCMBSearchReplyEvent(CMBSearchReplyEvent evt) {

        // VisualCafe 3.0 support 04/22/99/afic
        searchReplyEvent = evt;
        // VisualCafe 3.0 support 04/22/99/afic

        Vector v;
        synchronized (this) {
            v = (Vector) replyListenerVector.clone();
        }

        int cnt = v.size();
        CMBSearchReplyListener listener = null;

        System.out.println("There are " + cnt + " CMBSearchReplyListener listeners.");
        for (int i = 0; i < cnt; ++i) {
            listener = (CMBSearchReplyListener) v.elementAt(i);
            listener.onCMBSearchReply(evt);
        }
    }

    //*****************************************************************************
    // property accessors
    //*****************************************************************************
    /**
    * Property getter.
    * @return  window title
    */
    public String getWindowTitle() {
        return frame.getTitle();
    }

    /**
    * Property setter.
    * @param   newValue new window title
    */
    public void setWindowTitle(String newValue) {
        if (newValue != null)
            frame.setTitle(newValue);
    }

    /**
    * Property getter.
    * @return  connect button label
    */
    public String getSearchLabel() {
        return searchButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new connect button label
    */
    public void setSearchLabel(String newValue) {
        if (newValue != null)
            searchButton.setText(newValue);
    }

    /**
    * Property getter.
    * @return  disconnect button label
    */
    public String getExitLabel() {
        return exitButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new disconnect button label
    */
    public void setExitLabel(String newValue) {
        if (newValue != null)
            exitButton.setText(newValue);
    }

    /**
    * Property getter.
    * @return  cancel button label
    */
    public String getCancelLabel() {
        return cancelButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new cancel button label
    */
    public void setCancelLabel(String newValue) {
        if (newValue != null)
            cancelButton.setText(newValue);
    }

    public boolean isVisible() {
        return frame.isVisible();
    }

    //*****************************************************************************
    // Private methods
    //*****************************************************************************
    // Serialize transient data
    //*****************************************************************************
    /**
    * @serialData the data is serialized in the following sequence:
    * <pre>
    *    this object
    *    event listeners if the event listener is serializable
    *    a null object   tells the end of serialization
    * </pre>
    */
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.defaultWriteObject();

        Vector v = null;
        synchronized (this) {
            if (requestListenerVector.size() > 0)
                v = (Vector) requestListenerVector.clone();
        }

        if (v != null) {
            int cnt = v.size();
            CMBSchemaReplyListener listener = null;
            for (int i = 0; i < cnt; ++i) {
                listener = (CMBSchemaReplyListener) v.elementAt(i);
                if (listener instanceof Serializable)
                    stream.writeObject(listener);
            }
        }

        stream.writeObject(null); // tells the end of listener
    }

    //*****************************************************************************
    // De-serialize/initialize transient data
    //*****************************************************************************
    /**
    * @serialData the data is deserialized in the following sequence:
    * <pre>
    *    this object
    *    event listeners may be none
    *    a null object   the marker of serialization
    * </pre>
    */
    private void readObject(ObjectInputStream stream) throws Exception {
        requestListenerVector = new Vector(); // always initialize listener list
        stream.defaultReadObject();

        Object o;
        while ((o = stream.readObject()) != null)
            requestListenerVector.addElement((CMBSchemaReplyListener) o);
        frame.setVisible(true);
    }

    public void setTemplate(CMBSearchTemplate st) {
        if (_template == st)
            return;
        _template = st;
        addCMBSearchRequestListener(_template);
        // 04/23/99/afic
        _template.addCMBSearchReplyListener(this);
        // 04/23/99/afic
        _template.setAsynchSearch(false);
        //_template.setAsynchSearch(true);

        int nCrit = 0;

        try {
            critList = st.getSearchCriteria();
            nCrit = critList.size();
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        fieldPane.removeAll();
        _setupTemplatePane(critList);
        frame.setVisible(true);
        frame.requestFocus();
    }

    private void _setupTemplatePane(Vector critList) {
        fieldPane.setVisible(false);
        int nCrit = critList.size();
        fieldPane.setLayout(new GridLayout(nCrit, 3, 4, 4));

        txtFieldList.removeAllElements();
        opFieldList.removeAllElements();

        for (int i = 0; i < nCrit; i++) {
            CMBSTCriterion crit = (CMBSTCriterion) critList.elementAt(i);
            JLabel critLabel = new JLabel(crit.getName());
            JComboBox critOp = new JComboBox();
            JTextField critText = new JTextField();

            txtFieldList.addElement(critText);
            opFieldList.addElement(critOp);

            TextFieldActionListener listnr = new TextFieldActionListener(crit);
            critText.addActionListener(listnr);

            ComboBoxActionListener opListnr = new ComboBoxActionListener(crit);
            critOp.addActionListener(opListnr);

            short[] opList = crit.getValidOperators();
            for (int j = 0; j < opList.length; j++) {
                String opStr = opStrArray[(int) opList[j]];
                critOp.addItem(opStr);
            }
            short defOp = crit.getDefaultOperator();
            critOp.setSelectedItem(opStrArray[(int) defOp]);

            fieldPane.add(critLabel);
            fieldPane.add(critOp);
            fieldPane.add(critText);
        }
        fieldPane.setVisible(true);
        fieldPane.doLayout();
        fieldPane.repaint();
    }
    //*****************************************************************************
    // initGUI
    //*****************************************************************************
    private void initGUI(CMBSearchTemplate st) {

        frame = new JFrame("Search Template");
        mainPane = new CMBBasePanel();

        Border raisedBorder = new SoftBevelBorder(BevelBorder.RAISED);
        Border loweredBorder = new SoftBevelBorder(BevelBorder.LOWERED);

        //*****************************************************************************
        // Widgets creation and event listener setup
        //*****************************************************************************
        // Center panel
        //*****************************************************************************
        fieldPane = new JPanel() {
            public Insets getInsets() {
                return new Insets(20, 20, 20, 20);
            }
        };

        if (st != null) {
            int nCrit = 0;

            try {
                critList = st.getSearchCriteria();
                nCrit = critList.size();
            } catch (Exception ex) {
                ex.printStackTrace();
            }

            _setupTemplatePane(critList);
        }

        //*****************************************************************************
        // Bottom panel
        //*****************************************************************************
        exitButton = new JButton("Exit");
        searchButton = new JButton("Search");
        cancelButton = new JButton("Cancel") {
            public Insets getInsets() {
                return new Insets(6, 12, 6, 12); // make it spacy
            }
        };

        searchButton.setActionCommand(_cmdSearch);
        cancelButton.setActionCommand(_cmdCancel);
        exitButton.setActionCommand(_cmdExit);

        CMBActionAdapter al = new CMBActionAdapter() {
            public void actionPerformed(ActionEvent evt) {
                String cmd = evt.getActionCommand();

                if (cmd.equals(_cmdSearch))
                    postCMBSearchRequest(CMBSearchRequestEvent.CMB_REQUEST_SEARCH_SYNCH_CURSOR);

                else if (cmd.equals(_cmdCancel))
                    postCMBSearchRequest(CMBSearchRequestEvent.CMB_REQUEST_CANCEL_SEARCH);

                else if (cmd.equals(_cmdExit))
                    frame.setVisible(false);
                evt = null;
            }
        };
        exitButton.addActionListener(al);
        searchButton.addActionListener(al);
        cancelButton.addActionListener(al);

        searchButton.setBorder(raisedBorder);
        exitButton.setBorder(raisedBorder);
        cancelButton.setBorder(raisedBorder);

        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new GridLayout(1, 3, 4, 0));
        buttonPane.add(searchButton);
        buttonPane.add(cancelButton);
        buttonPane.add(exitButton);

        //*****************************************************************************
        // Add widgets to window frame
        //*****************************************************************************
        mainPane.addCenterPanel(fieldPane);
        mainPane.addBottomPanel(buttonPane);
        frame.getContentPane().add(mainPane, BorderLayout.CENTER);
        frame.pack();
        CMBUtil.setLocation(frame, CMBUtil.CENTER);
        if (st == null) {
            frame.setVisible(false);
        } else {
            frame.setVisible(true);
            frame.requestFocus();
        }
    }

    //*****************************************************************************
    // private data
    //*****************************************************************************
    private final String _cmdExit = "exit";
    private final String _cmdSearch = "search";
    private final String _cmdCancel = "cancel";

    private VetoableChangeSupport vcSupport = new VetoableChangeSupport(this);
    private PropertyChangeSupport pcSupport = new PropertyChangeSupport(this);

    //*****************************************************************************
    // GUI widgets, all labels are editable properties
    //*****************************************************************************
    private JButton searchButton = null;
    private JButton exitButton = null;
    private JButton cancelButton = null;

    private JFrame frame = null;
    private JPanel fieldPane = null;
    private CMBBasePanel mainPane = null;
    private CMBSearchTemplate _template = null;
    //*****************************************************************************
    // Transient data members
    //*****************************************************************************
    private transient Vector replyListenerVector = new Vector();
    private transient Vector requestListenerVector = new Vector();
    private transient Vector txtFieldList = new Vector();
    private transient Vector opFieldList = new Vector();
    private transient Vector critList = new Vector();

    //*****************************************************************************
    // VisualCafe 3.0 support 04/23/99/afic
    //*****************************************************************************
    private transient CMBSearchRequestEvent searchRequestEvent = null;
    private transient CMBSearchReplyEvent searchReplyEvent = null;

    //*****************************************************************************
    // Copyright
    //*****************************************************************************
    private static final String ibmid = "5648-D03 and 5648-D04  " + "(C) Copyright IBM Corp. 1994,1999";

    //*****************************************************************************
    // Internal class
    //*****************************************************************************

    class TextFieldActionListener implements java.awt.event.ActionListener {
        public TextFieldActionListener(CMBSTCriterion crit) {
            _crit = crit;
        }

        public void actionPerformed(ActionEvent evt) {
            if (evt.getSource() instanceof JTextField) {
                JTextField txtField = (JTextField) evt.getSource();
                String txtValue = txtField.getText();
                _crit.setValue(txtValue);
            }
        }

        private CMBSTCriterion _crit = null;
    }

    class ComboBoxActionListener implements java.awt.event.ActionListener {
        public ComboBoxActionListener(CMBSTCriterion crit) {
            _crit1 = crit;
        }

        public void actionPerformed(ActionEvent evt) {
            if (evt.getSource() instanceof JComboBox) {
                JComboBox choice = (JComboBox) evt.getSource();
                String value = (String) choice.getSelectedItem();
                for (int i = 0; i < opStrArray.length; i++) {
                    if (value.equals(opStrArray[i])) {
                        _crit1.setOperator((short) i);
                        return;
                    }
                }
            }
        }

        private CMBSTCriterion _crit1 = null;
    }

}