/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import com.ibm.mm.beans.*;

/**
 * This sample deletes the note log for a document or folder.
 * <p>
 * Note: CMv7, CMv8, VI/400, IP/390 support note log.   Other servers will throw not supported errors.
 */
public class TDeleteNotelog {

	public static void main(String[] args) throws Exception {
		if (args.length < 6) {
			System.out.print(
				"Deletes the notelog of a document. \n"
					+ "Syntax: \n"
					+ "  java TDeleteNotelog <dstype> <server> <userid> <password> <entity> <condition>\n"
					+ "where: \n"
                    + "  <dstype> is the type of server (Fed, ICM, DL, V4, IP) \n"
                    + "  <server> is the name of the server or database.  This can be \n"
                    + "           followed by a connect string in parenthesis. \n"
                    + "  <userid> is the userid \n"
                    + "  <password> is the password \n"
                    + "  <entity> the name of the entity (aka item type, index class) to search. \n"
					+ "  <condition> is a search condition. \n"
					+ "Examples: \n"
					+ "  java TDeleteNotelog ICM icmnlsdb icmadmin password Document @Title=\"Title\"\n");
			return;
		}

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String condition = "";
        int i;
        for (i = 5; i < args.length && !args[i].equals("/"); i++) {
            condition += " " + args[i];
        }

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		// Search for items to add the note to
        TSearch.search(queryService, dstype, entity, condition, false);
		searchResults.newResults(queryService.getResults());
		System.out.println(""+searchResults.getCount()+" items found.");
		for (i = 0; i < searchResults.getCount(); i++) {
			CMBItem item = searchResults.getItem(i);

			// Delete the notelog
			dataManagement.setDataObject(item);
			if (dataManagement.getNoteLogCount() == 0) {
				System.out.println("  Item "+item.getName()+" has no notelogs.");
			} else {
				System.out.println("  Item "+item.getName()+" has "+dataManagement.getNoteLogCount()+" notelogs.");
				System.out.println("  Deleting notelog of item "+item.getName());
				if (dataManagement.isSupported("checkOut")) { // some servers require checkout
					dataManagement.checkOut();
				}
				dataManagement.deleteNoteLog(0);
				if (dataManagement.isSupported("checkIn")) {
					dataManagement.checkIn();
				}
			}
		}

		System.out.println("Disconnecting from server");
		connection.disconnect();

	}

}