//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample connect/execute/disconnect to DES

/*
 * Filename       : TCallbackExecuteDES.java
 * Description    : Connect to DES Datastore.
 *                  Create query string.
 *                  Specify to retrieve a max result of 5 items.
 *                  Execute the query string with Callback
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                  Disconnect from DES Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TCallbackExecuteDES class
public class TCallbackExecuteDES implements DKConstantDES
{


  // Main method
  public static void main(String argv[])
  {

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    DKDatastoreDES dsDES = null;
    try {
      //*** for input parameters
      String libSrv = new String(""),   // library server
             userid = new String(""),     //userid
  	      pw = new String(""), // pw
             connect_string = ""; // PORT=80;DESAPPID=appid;DESAPPPW=apppw

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length != 4)
      {
       System.out.println("Usage: " );
       System.out.println("java TCallbackExecuteDES " +
        "tcp/ipAddress " +
        "userid " +
        "userpw " +
        "PORT=80;DESAPPID=appid;DESAPPPW=apppw[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>]");
       return;
      }
      else
      {
       libSrv = argv[0];
       userid = argv[1];
       pw = argv[2];
       connect_string = argv[3];
      }

    dsDES = new DKDatastoreDES();
    dkResultSetCursor pCur = null;
    DKNVPair parms[] = null;
    System.out.println("connecting to datastore");
    dsDES.connect(libSrv,userid,pw,connect_string);
    System.out.println("datastore connected libSrv " + libSrv + " userid " +
                        userid );
    String command = "SEARCH=(DATABASE=(Notes 5 Help);COND=((IN \"Subject\" \"key\"));";
    command = command + "OPTION=(SEARCHABLE_FIELD=(Subject,View,Body);" +
                                 "MAX_RESULTS=5;" +
                                 "TIME_LIMIT=10))";

    DKDDO p = null;
    DKDDO pDDO = null;
    dkXDO pXDO = null;
    int i = 0;
    int numDataItems = 0;
    TCallbackDES cbkDES = new TCallbackDES();
    short k = 0;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;
    int cnt = 0;
    InputStreamReader isreader = new InputStreamReader(System.in);
    System.out.println("query string " + command);
    System.out.println("executing query");
    Date date;
    date  = new Date();
    System.out.println("before query time " + date.toString());
    dsDES.executeWithCallback(command,DK_DES_GQL_QL_TYPE,parms,(dkCallback)cbkDES);
    System.out.println("datastore executed query");
    boolean isDone = false;
    DKException ckExc = null;

    while (isDone == false)
    {
     System.out.println("press <enter> to continue...");
     isreader.read();
     isDone = cbkDES.isDone();
    }

    ckExc = cbkDES.getException();
    if (ckExc != null)
    {
     System.out.println("Exception name " + ckExc.name());
     System.out.println("Exception message " + ckExc.getMessage());
     System.out.println("Exception error code " + ckExc.errorCode());
     System.out.println("Exception error state " + ckExc.errorState());
     ckExc.printStackTrace();
     cbkDES.resetException(); // reset since it is not needed any more
    }    

    System.out.println("query results processed");
    dsDES.disconnect();
    System.out.println("datastore disconnected");
    dsDES.destroy();
    }
    catch (DKException exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     System.out.println("Exception error code " + exc.errorCode());
     System.out.println("Exception error state " + exc.errorState());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }
}

