//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample connect/disconnect to DES

/*
 * Filename       : TViewSaveSchQryDES.java
 * Description    : test DKDatastoreDES for connect and disconnect
 *                  List all the Saved query folders, queries and resultIDs
 *                  Display results for the result IDs if any.
 *                  Display DDO
 *                  close connector
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.util.*;

// TViewSaveSchQryDES class
public class TViewSaveSchQryDES implements DKConstantDES {

  // Main method
  public static void main(String argv[]){

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length != 4)
      {
       System.out.println("Usage: " );
       System.out.println("java TViewSaveSchQryDES " +
        "tcp/ipAddress " +
        "userid " +
        "userpw " +
        "PORT=80;DESAPPID=appid;DESAPPPW=apppw[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>]");
       return;
      }

    System.out.println("TViewSaveSchQryDES");
    System.out.println("");
    DKDatastoreDES dsDES = null;
    try {
    dsDES = new DKDatastoreDES();
    String userName = dsDES.userName();
    String dsName = dsDES.datastoreName();
    System.out.println("user " + userName + " dsName " + dsName);
    System.out.println("connecting to datastore");
    dsDES.connect(argv[0], argv[1], argv[2], argv[3]);
    System.out.println("datastore connected " + argv[0] + " userid - " + argv[1]);
    userName = dsDES.userName();
    dsName = dsDES.datastoreName();
    System.out.println("user " + userName + " dsName " + dsName);

    //System.out.println("calling refreshDESRepositories()" );
    //dsDES.refreshDESRepositories();
    //System.out.println("completed refreshDESRepositories()" );
	String [] folders = dsDES.listQueryFolderNames();
	if(folders != null) {
		for (int i =0; i < folders.length; ++i){
			System.out.println("folders=" + folders[i]);
			String [] sQueries= dsDES.listStoredQueryNames(folders[i]);
			if(sQueries != null) {
				for (int j=0; j < sQueries.length; ++j){
					System.out.println("  - sQueries=" + sQueries[j]);
					String [] sHitLists= dsDES.listStoredQueryResultIDs(folders[i],sQueries[j]);
					if(sHitLists != null){
						for(int k=0; k<sHitLists.length; ++k){
							System.out.println("     - sHitLists=" + sHitLists[k]);
							displayResults(dsDES, folders[i], sQueries[j], sHitLists[k]);
						}
					}//hitLists
				}
			}//sQueries
		}
	}//folders

    dsDES.disconnect();
    System.out.println("datastore disconnected");
    userName = dsDES.userName();
    dsName = dsDES.datastoreName();
    System.out.println("user " + userName + " dsName " + dsName);
    dsDES.destroy();
    }
    catch (DKException exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
     dsDES.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }

  }

	public static void displayResults(DKDatastoreDES dsDES, String fName, String qName, String resultID) throws Exception, DKException {

	dkResultSetCursor pCur = null;
	DKDDO ddo = null;
    
    	pCur = dsDES.storedQueryResults(fName,qName,resultID);
    	System.out.println("*** Results Cardinality for " +resultID+ " is="+pCur.cardinality());

    	while (pCur.isValid())
     	{
     		ddo = pCur.fetchNext();
     		if (ddo != null)
       		{
       		System.out.println("");
       		System.out.println("=====> Item " + pCur.getPosition() + " <=====");
       		displayDDO(ddo);
       		}
     	}

	}


   // method to show DDO hit
   public static void displayDDO(DKDDO ddo)
                throws DKException, Exception
     {
     dkXDO pXDO = null;
     int i = 0;
     int numDataItems = 0;
     short k = 0;
     short j = 0;
     Integer valueCount = null;
     Object value = null;
     String dataName = null;
     dkCollection pCol = null;
     dkIterator pIter = null;
     Object anObject = null;
     String mimeType = "";
     numDataItems = ddo.dataCount();
     DKPid pid = ddo.getPidObject();
     System.out.println("pid string " + pid.pidString());

      System.out.println("Number of Attributes " + numDataItems);
      for (j = 1; j <= numDataItems; j++)
        {
        anObject = ddo.getData(j);
        dataName = ddo.getDataName(j);
        System.out.println(j + ": Name " + dataName );
        // determine if data item has a single value or multiple values
        Short type = (Short)ddo.getDataPropertyByName(j, DK_CM_PROPERTY_TYPE);
        k = type.shortValue();
        if ((k == DK_CM_COLLECTION || k == DK_CM_COLLECTION_XDO)
            && (!dataName.equals(DK_CM_DKPARTS)) )
          {
          pCol = (dkCollection)anObject;
          pIter = pCol.createIterator();
          i = 0;
          while (pIter.more() == true)
            {
            i++;
            value = pIter.next();
            System.out.println("   Value" + i + " " + value);
            }
          } // end DK_CM_COLLECTION
        else
        if (dataName.equals(DK_CM_DKPARTS))  // this is a DKParts with a URL and/or a doclink
          {
          pCol = (dkCollection)anObject;
          pIter = pCol.createIterator();
          while (pIter.more() == true)
            {
            pXDO = (dkXDO) pIter.next();
            mimeType = pXDO.getMimeType();
            if (mimeType.equals("text/html"))
              System.out.println("   URL: " + new String(pXDO.getContent()));
            else if(mimeType.equals("text/doclink"))
              System.out.println("   doclink: " + new String(pXDO.getContent()));
            }
          } // end DK_CM_COLLECTION_XDO
        else
          System.out.println("   Value " + anObject);
        } // end for j
      } // end displayDDO

}

