/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

 // sample to test combined Query, one Parametric Query, its scope, 
 //        and one Text Query without scope 

/*
 * Filename       : TCQPQTQIQOpt.java
 * Description    : Test combined Query, one Parametric Query, its scope, 
 *                  and one Text Query without scope
 */                   

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.lang.*;

public class TCQPQTQIQOpt implements DKConstant
{

  // Main method
  public static void main(String argv[])
  {
    // Checking for input parameters
    if (argv.length < 4) {
       System.out.println("Usage: " );
       System.out.println("  java TCQPQTQOpt <DLServer> <userid> <password> <TMServer>" );
       return;
    }
    
    // input parameters
    String DLServer = argv[0];  // DL Library Server name, e.g. LIBSRVRX
    String userid   = argv[1];
    String password = argv[2];
    String TMServer = argv[3];  // Text Search server name, case sensitive, e.g. TMSRVRX in uppercase

    String DEconnect = "SQLDSTYPE=DB2;SQLUID=db2inst1;SQLPWD=ibmdb2;SQLLSUID=lsadmin";
      
    DKDatastoreDL dsDL = null;
    DKDatastoreTS dsTS  = null;
    DKDatastoreQBIC dsQBIC = null;
    try {
       System.out.println(" *** Test Combined Query : Parametric Query, Text Query, and Image Query ");
       dsDL = new DKDatastoreDL();
       System.out.println("\n connecting to datastore DL :" + DLServer); 
       dsDL.connect(DLServer, userid, password,DEconnect);
       System.out.println(" datastore DL connected"); 

       dsTS  = new DKDatastoreTS();
       System.out.println("\n connecting to datastore TS :" + TMServer); 
       dsTS.connect(TMServer,"", ' ');
       System.out.println(" datastore TS connected"); 

       dsQBIC = new DKDatastoreQBIC();
       System.out.println("Datastore QBIC created");
       System.out.println("connecting to datastore");
       dsQBIC.connect("QBICSRV","frnadmin","password",
                "DATABASE = QBICDB; CATALOG = QBICCAT; LIBSERVER = LIBSRVRX ");
       System.out.println("datastore connected");
       dsQBIC.openCatalog("QBICDB", "QBICCAT");
       System.out.println("Catalog opened ");

       dkResultSetCursor pCur = null;
       String scmd = null;
       String pcmd = null;
       String tcmd = null;
       DKDDO pDDO = null;
       dkXDO pXDO = null;
       DKPidXDO pidXDO = null;
       int i = 0;
       int resultCounter = 0;
       int numDataItems = 0;
       short k = 0;
       short j = 0;
       String pidString;
       String strDataName;
       String className;  
       String className2;
       dkCollection pCol = null;
       dkIterator pIter = null;
       Object a = null;
       dkDataObjectBase pDO = null;
       DKNVPair[] par = new DKNVPair[4];    

       // prepare the parametric query string
       pcmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,COND=(DLSEARCH_Date >= '1994'));" +
		       "OPTION=(CONTENT=NO;" +
		       "TYPE_FILTER=FOLDERDOC)";
       System.out.println("\n parametric query string " + pcmd);

       DKParametricQuery pq = (DKParametricQuery) dsDL.createQuery(pcmd,DK_CM_PARAMETRIC_QL_TYPE, null);

       // prepare the text query string
       tcmd = "SEARCH=(COND=('IBM'));" +
              "OPTION=(SEARCH_INDEX=TMINDEX)";
       System.out.println("\n text query string " + tcmd);
       DKTextQuery     tq = (DKTextQuery) dsTS.createQuery(tcmd,DK_CM_TEXT_QL_TYPE, null);
       String icmd = "QbColorFeatureClass color=<255,0,0>";
       DKImageQuery iq = (DKImageQuery) dsQBIC.createQuery(icmd,DK_CM_IMAGE_QL_TYPE, null);

       DKCombinedQueryOptDL cqOpt = new DKCombinedQueryOptDL();
       par[0] = new DKNVPair(DK_CM_PARM_QUERY, pq);
       par[1] = new DKNVPair(DK_CM_TEXT_QUERY, tq);
       par[2] = new DKNVPair(DK_CM_IMAGE_QUERY, iq);
       par[3] = new DKNVPair(DK_CM_PARM_END, null);

       System.out.println("\n executing combined query, pq,  tq, and iq");
       cqOpt.execute(par);
       System.out.println(" cq query executed");

       pCur = cqOpt.resultSetCursor();

       if (pCur == null)
       {
        System.out.println("cursor null");
       }
       else if (pCur.cardinality() == 0) {
           System.out.println(" CQ results is empty ");  
       }
       else {
         System.out.println(" CQ number of results = " + pCur.cardinality());  
         System.out.println("\n process query results : \n");
         while (pCur.isValid())
         {
           pDDO = pCur.fetchNext();
           if (pDDO != null)
           {
    	     System.out.println(" =====> Item " + ++resultCounter + " <=====");
             i++;
    	     numDataItems = pDDO.dataCount();
    	     k = pDDO.propertyId(DK_CM_PROPERTY_ITEM_TYPE);
    	     if (k > 0)  {
    	   	   Short sVal = (Short)pDDO.getProperty(k);
    	      	   j = sVal.shortValue();
    	      	   switch (j) {
    	      	      case DK_CM_DOCUMENT :
    	   	           System.out.println(" Item is a document ");
    	   	           break;
    	      	      case DK_CM_FOLDER :
    	   	           System.out.println(" Item is a folder ");
    	   	           break;
		              default:
    	   	           System.out.println(" Item type is unknown ");
    	   	           break;
    	      	      } 
    	       }  
    	     System.out.println("Pid " + pDDO.getPidObject().pidString());
    	     System.out.println("Number of Data Items " + numDataItems);
    	     for (j = 1; j <= numDataItems; j++) {
    	      	 a = pDDO.getData(j);
    	      	 strDataName = pDDO.getDataName(j);
    	      	 System.out.println(j + ": Attribute Name " + strDataName );
    	      	 if (a instanceof String)
      	      	     System.out.println("   Attribute Value " + a);    
    	      	 else if (a instanceof dkCollection) {
    	      	    System.out.println("   Attribute Value is a collection");    
    	      	 }  // end if collection	   
                 else
      	      	    System.out.println("   Attribute Value " + a);    
    	     }       
    	  } // if pDDO == null
         }
       }
       if ((dsQBIC != null) && (dsQBIC.isConnected())) dsQBIC.disconnect();
       if ((dsTS != null) && (dsTS.isConnected())) dsTS.disconnect();
       if ((dsDL != null) && (dsDL.isConnected())) dsDL.disconnect();
       System.out.println("\n datastores disconnected"); 
       if (dsQBIC != null) dsQBIC.destroy();
       if (dsTS != null) dsTS.destroy();
       if (dsDL != null) dsDL.destroy();
    }
    catch (DKException exc)
    {
     try {
        if (dsDL != null)
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     try {
        if (dsTS != null)
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     try {
        if (dsQBIC != null)
         dsQBIC.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println(" Exception name : " + exc.name());
     System.out.println(" Exception message : " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
        if (dsDL != null)
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     try {
        if (dsTS != null)
         dsTS.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     try {
        if (dsQBIC != null)
         dsQBIC.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println(" Exception message : " + exc.getMessage());
     exc.printStackTrace();
    }
  System.out.println("\n Have a nice day!"); 
  }
}

