/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

 // Perform create and delete Text Search Index
 /*
  * Filename       : TCreateStructDocIndexTS.java
  * Description    : Perform index creation and doc model creation functions
  */


import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

// TCreateStructDocIndexTS class
public class TCreateStructDocIndexTS implements DKConstantDL {

  // Main method
  public static void main(String argv[]){
    
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //-------------------------------------------------------------- 
    if (argv.length < 2){
       System.out.println("Invoke Syntax: " );
       System.out.println("  java TCreateStructDocIndexTS <srchSrv> <srchIndex>" );
       return;
    }
    
    String srchSrv  = argv[0];
    String strIndex = argv[1];
      
    DKDatastoreTS dsTS = null;
    try {
        DKSearchIndexDefTS pEnt = null;
        int iIndexType = 0;
        String strIndexType = "";
        String strIndexName = "";
        Integer iOpt = null;
        int iIndexADD = 0;
        String strIndexADD = "";
        dsTS = new DKDatastoreTS();
        DKDatastoreDefTS dsDef = null;
        DKDatastoreAdminTS dsAdmin = null;
        DKIndexingRulesTS indexRules = new DKIndexingRulesTS();
        DKDocModelTS docModel = new DKDocModelTS();
        DKDocSectionTS docSection = new DKDocSectionTS();
        DKDocSectionTS docSection2 = new DKDocSectionTS();
    
        indexRules.setIndexName(strIndex);
        docModel.setCCSID(DK_TS_CCSID_00850);
        docModel.setName("SAMPCORPMOD");
        docSection.setName("SAMPCORPTITLE");
        docSection.setTag("TITLE");
        docModel.addDocSection(docSection);   
        docSection2.setName("SAMPCORPBODY");
        docSection2.setTag("BODY");
        docModel.addDocSection(docSection2);   
        indexRules.setDefaultDocFormat(DK_TS_DOCFMT_HTML);
        indexRules.setDefaultDocModel(docModel);
        System.out.println("connecting to datastore"); 
        dsTS.connect(srchSrv,"","","");
        System.out.println("datastore connected srchSrv " + srchSrv); 
        dsDef = (DKDatastoreDefTS)dsTS.datastoreDef();
        dsAdmin = (DKDatastoreAdminTS)dsDef.datastoreAdmin();
        System.out.println(" create doc model for instance ");
        dsAdmin.createDocModel("",docModel);
        System.out.println(" doc model created for instance ");

        pEnt = new DKSearchIndexDefTS((dkDatastore)dsTS);
        System.out.println(" create index " + strIndex);
        pEnt.setName(strIndex);
        pEnt.setIndexType(DK_TS_INDEX_TYPE_PRECISE);
        pEnt.setIndexProperty(DK_TS_PROPERTY_SECTIONS_ENABLED);
        pEnt.setLibraryId("LIBSUM");
        pEnt.setLibraryClientServices("IMLLSCDL");
        pEnt.setLibraryServerServices("IMLLSSDL");
        String strDefaultDir = "e:\\tsindex\\";
        String strIndexFileDir = strDefaultDir + "index\\" + strIndex;
        pEnt.setIndexDataArea(strIndexFileDir);
        String strWorkFileDir = strDefaultDir + "work\\" + strIndex;
        pEnt.setIndexWorkArea(strWorkFileDir);
        pEnt.addDocModel(docModel);
        dsDef.add((dkEntityDef)pEnt);
        System.out.println(" created index " + strIndex);

        System.out.println(" set indexing rules for " + strIndex);
        dsAdmin.setIndexingRules(indexRules);
        System.out.println(" indexing rules set for " + strIndex);
   
        dsTS.disconnect();
        System.out.println("datastore disconnected"); 
        dsTS.destroy();
    }
    catch (DKException exc){
     	try {
     	    dsTS.destroy();
     	}
     	catch (Exception e)
     	{
     	 e.printStackTrace();
     	}        
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
    catch (Exception exc){
     	try {
     	    dsTS.destroy();
     	}
     	catch (Exception e)
     	{
     	 e.printStackTrace();
     	}        
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
  }
}

