/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TQueryExprDL.java
 * Description    : Test execute query expression with schema-mapping on DL Datastore
 *                  The query expression is expected to come from a Federated Datastore
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.sql.*;
import java.util.*;

public class TQueryExprDL implements DKConstant {

  // Main method
  public static void main(String argv[]){
    
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

   try {

      // NOTE:
      // Fed. Entity, fed. Attr, schema mapping, and search template information
      // could have been obtained from the Federated Datastore.
      // For clarity of this example and for most practical purposes, 
      // they are created here in-memory.

      // create federated attributes :
         dkAttrDef fAuthor = new DKAttrDefFed();
           fAuthor.setEntityName("FedSample");
           fAuthor.setName("FedAuthor");
           fAuthor.setType(DK_CM_VSTRING);
           fAuthor.setId((short)95);
           fAuthor.setSize(50);
           fAuthor.setScale(1);
           fAuthor.setPrecision(50);
           fAuthor.setNullable(true);
           fAuthor.setStringType(DK_ATTR_ALPHA);
   
         dkAttrDef fTitle = new DKAttrDefFed();
           fTitle.setEntityName("FedSample");
           fTitle.setName("FedTitle");
           fTitle.setType(DK_CM_VSTRING);
           fTitle.setId((short)99);
           fTitle.setSize(100);
           fTitle.setScale(1);
           fTitle.setPrecision(100);
           fTitle.setNullable(true);
           fTitle.setStringType(DK_ATTR_ALPHA);
   
         dkAttrDef fSubject = new DKAttrDefFed();
           fSubject.setEntityName("FedSample");
           fSubject.setName("FedSubject");
           fSubject.setType(DK_CM_VSTRING);
           fSubject.setId((short)100);
           fSubject.setSize(100);
           fSubject.setScale(1);
           fSubject.setPrecision(100);
           fSubject.setNullable(true);
           fSubject.setStringType(DK_ATTR_ALPHA);
         
         System.out.println(" *** federated attributes created ***");
   
      // create native attributes : DL index_class DLSAMPLE at mufasa
         dkAttrDef nbAuthor = new DKAttrDefDL();
           nbAuthor.setEntityName("DLSAMPLE");
           nbAuthor.setName("DLSEARCH_Author");
           nbAuthor.setType(DK_CM_VSTRING);
           nbAuthor.setId((short)95);
           nbAuthor.setSize(50);
           nbAuthor.setScale(1);
           nbAuthor.setPrecision(50);
           nbAuthor.setNullable(true);
           nbAuthor.setStringType(DK_ATTR_ALPHA);
   
         dkAttrDef nbTitle = new DKAttrDefDL();
           nbTitle.setEntityName("DLSAMPLE");
           nbTitle.setName("DLSEARCH_Title");
           nbTitle.setType(DK_CM_VSTRING);
           nbTitle.setId((short)99);
           nbTitle.setSize(100);
           nbTitle.setScale(1);
           nbTitle.setPrecision(100);
           nbTitle.setNullable(true);
           nbTitle.setStringType(DK_ATTR_ALPHA);
   
         dkAttrDef nbSubject = new DKAttrDefDL();
           nbSubject.setEntityName("DLSAMPLE");
           nbSubject.setName("DLSEARCH_Subject");
           nbSubject.setType(DK_CM_VSTRING);
           nbSubject.setId((short)100);
           nbSubject.setSize(100);
           nbSubject.setScale(1);
           nbSubject.setPrecision(100);
           nbSubject.setNullable(true);
           nbSubject.setStringType(DK_ATTR_ALPHA);
   
         System.out.println(" *** native attributes created ***");
   
       //  create schema mapping : entity and attr mappings
           // maps "FedAuthor" to "DLSEARCH_Author"
         dkAttrMapping mapAuthor = new DKAttrMappingFed();
           mapAuthor.addMappedAttr(fAuthor);
           mapAuthor.addMapToAttr(nbAuthor);
   
            // maps "FedTitle" to "DLSEARCH_Title"
         dkAttrMapping mapTitle = new DKAttrMappingFed();
           mapTitle.addMappedAttr(fTitle);
           mapTitle.addMapToAttr(nbTitle);
   
            // maps "FedSubject" to "DLSEARCH_Subject"
         dkAttrMapping mapSubject = new DKAttrMappingFed();
           mapSubject.addMappedAttr(fSubject);
           mapSubject.addMapToAttr(nbSubject);
   
         System.out.println(" *** attribute mapping created *** ");
   
         dkSchemaMapping sm = new DKSchemaMappingFed();
           sm.setServerName("LSMUFASA");
           sm.setServerType(DK_DSTYPE_DL);
           sm.setEntityName("DLSAMPLE");
           sm.setMappedEntityName("FedSample");
   
           // add attribute mapping to this schema
           sm.addAttrMapping(mapAuthor);
           sm.addAttrMapping(mapTitle);
           sm.addAttrMapping(mapSubject);
   
       System.out.println(" ** schema mapping created ***");
   
       DKQTerm sTerm = new DKQTerm(new DKQTerm("FedAuthor", DKOpCode.EQ, "'Anonymous'"),
                                    DKOpCode.AND,
                                    new DKQTerm("FedTitle", DKOpCode.NEQ, "NULL"));
       sTerm = sTerm.or(new DKQTerm("FedAuthor", DKOpCode.LIKE, "'Alan%'"));

       System.out.println(" Search Term = " + sTerm.toString());
   
       // Create a query sub-expression
       DKQSubExpr qse = new DKQSubExpr();
       qse.setTranslation(true);
       qse.setMappingName(sm.getName());
       qse.setQueryTerm(sTerm);
       qse.setEntityName("FedSample");
       qse.setDisplayList(new String[]{"FedTitle","FedSubject","FedAuthor"});
   
       // Generic Options : MAX_RESULTS=6,CONTENT=YES
       DKNVPair[] opt = new DKNVPair[2];
       opt[0] = new DKNVPair("MAX_RESULTS", "6");
       opt[1] = new DKNVPair("CONTENT","YES");
       qse.setOptionList(opt);
       for (int i = 0; i < opt.length; i++)
           System.out.println(" option[" + i + "] : " + opt[i].getName() +
                                                  "=" + opt[i].getValue().toString());
   
       // Create a query expression
       DKQExpr qe = new DKQExpr();
       qe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
       qe.addSubQuery(qse);
   
       // Create a compound query expression
       DKCQExpr cqe = new DKCQExpr(qe);
       cqe.setQLType(DK_CM_PARAMETRIC_QL_TYPE);
   
       // All set; Connect and execute the compund query expression
       DKDatastoreDL dsDL = new DKDatastoreDL();
       System.out.println(" *** connecting to mufasa ***");
       dsDL.connect("LSMUFASA", "frnadmin", "password","");
       System.out.println(" \t -----> connected ");
   
       // Do not forget to register the schema mapping
       dsDL.registerMapping(new DKNVPair("SCHEMA", sm));
       dkResultSetCursor rsc = dsDL.execute(cqe);
   
       // Process the result set cursor :
       if (rsc == null) 
          System.out.println(" *** cursor is null");
       else {
            DKDDO ddo;
            while (rsc.isValid())  {
              ddo = rsc.fetchNext();
              if (ddo != null) {  
                System.out.println(" pidString : " + ddo.getPidObject().pidString());
                displayDDO(ddo);
              }
            }
       }
   
       // disconnects
       System.out.println("\n\n *** disconnect from the datastore DL ... ***");
       dsDL.disconnect();
       System.out.println(" *** datastore DL disconnected ***"); 
   
       // that is all, folks
    }
    catch (DKException exc)
    {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
    catch (Exception exc)
    {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
    }
  }

  // simple method to show DDO hit 
  public static void displayDDO(DKDDO ddo) 
                throws DKException, Exception
  {
    String   dataName;
    short    cnt;
    Object a = null;
 
    if (ddo != null) {
       cnt = ddo.dataCount();
       for (short i = 1; i <= cnt; i++) {
           a = ddo.getData(i);
           dataName = ddo.getDataName(i);
           System.out.print(i + ": Attribute Name :" + dataName );
           if (a != null) 
              System.out.println(",\t   Value :" + a.toString());    
           else
              System.out.println(",\t   Value is null");    
           
       }  
    }
  }

}

