/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TCallbackFed.java
 * Description    : Callback class to receive results from Fed
 *
 */

import com.ibm.mm.sdk.common.*;
import java.util.*;
import java.lang.*;


public class TCallbackFed 
	implements dkCallback, DKConstantFed, DKMessageIdFed, java.io.Serializable
{
 
  // constructor
 /**
  *  Constructs an callback for Fed
  */
  public TCallbackFed()
  {
     System.out.println("*** con called ");
     _threshold = 5;
  }

 /**
  *  Sets the number of returned hits for each data callback.
  *  This method should be set by applications for the datastore.
  *  @param 	nValue the number of hits in the result collection
  *  		for each callback.
  */
 public void setThreshold(int nValue)
 {
    _threshold = nValue;
 }
 
 /**
  *  Gets the number of returned hits that the server return
  *  for each callback.  The datastore will call this method for
  *  each query.
  *  @return	the threshold value,
  */
 public int getThreshold()
 {
    System.out.println("*** get threshold called ");
    return _threshold; 
 }
 
 /**
  *  Query done check
  *  @return	true when done
  */
 public boolean isDone()
 {
    return _isDone; 
 }
 
 /**
  *  Indicates that the user has canceled the query.
  *  It should be called by the application to let the server
  *  knows to stop the query.
  */
 public void cancelQuery()
 {
    _cancel = true;
 }
 
 /**
  *  Returns nHits from the backend datastore
  *  @param 	ddoList a collection of DKDDO objects
  *  @param 	isDone  a boolean value indicating whether the server is
  *  			done with the query.  That is, there is more hits.
  *  @return	true if query is done or user has canceled the query.
  */
 public boolean dataCallback(dkCollection ddoList, boolean isDone)
 {
    boolean fDone = false;
    _numCalled++;
    if (_cancel == true || isDone == true){
     fDone = true;
     _isDone = true;
     System.out.println("*** " + _numCalled + " time called ***");
     if (ddoList != null)
     {
      try
      {
       processResults(ddoList);
      }
      catch (DKException exc)
      {
       System.out.println("Exception name " + exc.name());
       System.out.println("Exception message " + exc.getMessage());
       System.out.println("Exception error code " + exc.errorCode());
       System.out.println("Exception error state " + exc.errorState());
       exc.printStackTrace();
      }
      catch (Exception exc)
      {
       System.out.println("Exception message " + exc.getMessage());
       exc.printStackTrace();
      }
     } 
    }
    else {
     System.out.println("*** " + _numCalled + " time called ***");
     if (ddoList != null)
     {
      try {
         processResults(ddoList);
      }
      catch (DKException exc){
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         System.out.println("Exception error code " + exc.errorCode());
         System.out.println("Exception error state " + exc.errorState());
         exc.printStackTrace();
      }
      catch (Exception exc){
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
     }
    }
    return fDone;
 }
  
 /**
  *  Report an exception from the backend datastore
  *  @param 	exc an exception from the backend datastore
  */
 public void reportException(DKException exc)
 {
  _exc = exc;
 }

 public DKException getException()
 {
  return _exc;
 }

 public void resetException()
 {
   _exc = null;
 }

  public static void processDDO(DKDDO p) 
                throws DKException, Exception
  {
   int numDataItems = 0;
   short k = 0;
   short j = 0;
   short sDataPropType = 0;
   Short sProp = null;
   String strDataName;
   dkCollection pCol = null;
   Object a = null;
   dkXDO pXDO = null;
   numDataItems = p.dataCount();
   DKPid pid = p.getPidObject();
   System.out.println("pid string " + pid.pidString());
   k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);
   if (k > 0)
   {
    Short sVal = (Short)p.getProperty(k);
    j = sVal.shortValue();
    switch (j)
    {
     case DK_CM_DOCUMENT :
     {
      System.out.println("Item is document ");
      break;
     }
     case DK_CM_FOLDER :
     {
      System.out.println("Item is folder ");
      break;
     }
     case DK_CM_ITEM :
     {
      System.out.println("Item found ");
      break;
     }
    }
   }

   System.out.println("Number of Data Items " + numDataItems);
   for (j = 1; j <= numDataItems; j++)
   {
    a = p.getData(j);
    strDataName = p.getDataName(j);
    System.out.println(j + ": Attribute Name " + strDataName );
    sProp = (Short)p.getDataPropertyByName(j,DK_CM_PROPERTY_TYPE);
    sDataPropType = sProp.shortValue();
    switch (sDataPropType)
    {
     case DK_CM_DATAITEM_TYPE_STRING :
     {
      if (a == null)
      {
       System.out.println("   String Attribute Value is null");    
      }
      else
      {
       System.out.println("   String Attribute Value " + a);    
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_LONG :
     {
      if (a == null)
      {
       System.out.println("   Int Attribute Value is null");    
      }
      else
      {
       System.out.println("   Int Attribute Value " + a);    
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_SHORT :
     {
      if (a == null)
      {
       System.out.println("   Short Attribute Value is null");    
      }
      else
      {
       System.out.println("   Short Attribute Value " + a);    
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_DOUBLE :
     {
      if (a == null)
      {
       System.out.println("   Double Attribute Value is null");    
      }
      else
      {
       System.out.println("   Double Attribute Value " + a);    
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_DATE :
     {
      if (a == null)
      {
       System.out.println("   Date Attribute Value is null");    
      }
      else
      {
       System.out.println("   Date Attribute Value " + a);    
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_TIME :
     {
      if (a == null)
      {
       System.out.println("   Time Attribute Value is null");    
      }
      else
      {
       System.out.println("   Time Attribute Value " + a);    
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_TIMESTAMP :
     {
      if (a == null)
      {
       System.out.println("   Timestamp Attribute Value is null");    
      }
      else
      {
       System.out.println("   Timestamp Attribute Value " + a);    
      }
      break;
     } 
     case DK_CM_DATAITEM_TYPE_XDOOBJECT :
     {
      if (a == null)
      {
       System.out.println("   XDO Object Attribute Value is null");    
      }
      else
      {
       System.out.println("   XDO Object Attribute Value " + a);
       pXDO = (dkXDO)a;
       processXDO(pXDO);
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_COLLECTION_XDO :
     {
      if (a == null)
      {
       System.out.println("   XDO Collection Attribute Value is null");    
      }
      else
      {
       System.out.println("   XDO Collection Attribute Value " + a);    
       pCol = (dkCollection)a;
       processXDOCollection(pCol);
      }
      break;
     }
     case DK_CM_DATAITEM_TYPE_COLLECTION_DDO :
     {
      if (a == null)
      {
       System.out.println("   DDO Collection Attribute Value is null");    
      }
      else
      {
       System.out.println("   DDO Collection Attribute Value " + a);    
       pCol = (dkCollection)a;
       processFolder(pCol);
      }
      break;
     }
     default :
     {
      if (a == null)
      {
       System.out.println("   Unknown type Attribute Value is null");    
      }
      else
      {
       System.out.println("   Unknown type Attribute Value " + a);    
      }
      break;
     }
    }
   }       
  }

  public static void processXDO(dkXDO pXDO) 
                throws DKException, Exception
  {
    DKPidXDO pid = (DKPidXDO)pXDO.getPidObject();
    System.out.println("XDO pid string " + pid.pidString());
  }

  public static void processResults(dkCollection results) 
                throws DKException, Exception
  {
    DKDDO    item = null;
    System.out.println( "Begin Results ******************" + results);
    int i = 0;
    if (results != null)
    {
     System.out.println( "number of items in DKResults " + results.cardinality());
     dkIterator iter = results.createIterator();
 
     while(iter.more()) {
       item = (DKDDO) iter.next();
       i++;
       if (item != null) {
          System.out.println("*** DDO number " + i + " ***");
	  // item.retrieve();
          processDDO(item);
       }
     }
    } 
    System.out.println( "End Results ****************** " + results);
  }

  public static void processFolder(dkCollection results) 
                throws DKException, Exception
  {
    DKDDO    item = null;
    System.out.println( "Begin folder ******************" + results);
    int i = 0;
    if (results != null)
    {
     System.out.println( "number of items in DKFolder " + results.cardinality());
     dkIterator iter = results.createIterator();
 
     while(iter.more()) {
       item = (DKDDO) iter.next();
       i++;
       if (item != null) {
          System.out.println("*** DDO number " + i + " ***");
	  // item.retrieve();
          processDDO(item);
       }
     }
    } 
    System.out.println( "End folder ****************** " + results);
  }

  public static void processXDOCollection(dkCollection results) 
                throws DKException, Exception
  {
    dkXDO pXDO = null;
    int i = 0;
    System.out.println( "Begin XDO Collection ******************" + results);
    if (results != null)
    {
     System.out.println( "number of items in XDO Collection " + results.cardinality());
     dkIterator iter = results.createIterator();
 
     while(iter.more()) {
       pXDO = (dkXDO) iter.next();
       i++;
       if (pXDO != null) {
        System.out.println("*** XDO number " + i + " ***");
	// pXDO.retrieve();
        processXDO(pXDO);
       }
     }
    } 
    System.out.println( "End XDO Collection ******************" + results);
  }

 private boolean _cancel = false;  
 private boolean _isDone = false;  
 private int _threshold;
 private int _numCalled = 0;
 private DKException _exc = null;
}
