/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample to create Federated entity 

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

// TCreateEntityFed class
/**
 * <pre>
 * Filename       : TCreateEntityFed.java
 * Description    : test the following classes and funtions in each:
 *		     	- DKDatastoreFed: 
 *                          + connect
 *                          + disconnect
 *			    + userName
 *			    + datastoreName
 *                      - DKDatastoreDefFed:
 *                          + getEntity
 *                      - DKEntityDefFed:
 *                          + setName
 *                          + setDescription
 *			    + add()
 *			    + add(dkAttrDef)
 *			- DKAttrDefFed:
 *			    + setName
 *			    + setEntityName
 *			    + setEntity
 *			    + setDescription
 *			    + setType
 *			    + setSize
 *			    + setNullable
 *  			    + setUpdatable
 *			    + setQueryable
 *			    + setStringType
 *		 
 * </pre>
 */

public class TCreateEntityFed implements DKConstantFed {

  // Main method
  public static void main(String argv[]){
    
   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    try {
	DKDatastoreFed dsFed = new DKDatastoreFed();
	DKDatastoreDefFed dsDef = (DKDatastoreDefFed)dsFed.datastoreDef();

	String userName;
    	String dsName;

    	System.out.println("connecting to datastore");

     	String url = "feddb";
	dsFed.connect(url,"edms1","edms1","");

	System.out.println("datastore connected"); 

	userName = dsFed.userName();
	dsName = dsFed.datastoreName();

	System.out.println("user " + userName + " dsName " + dsName);   
	
	DKEntityDefFed entityObj = new DKEntityDefFed(dsFed);	
	entityObj.setName("Demo1");
	entityObj.setDescription("Green demo federated entity created by mimis test case");
	entityObj.add();	//adding it to the federated database

	//create 1st attribute
	DKAttrDefFed attrObj = new DKAttrDefFed(dsFed);	
	attrObj.setName("Author");
	attrObj.setEntityName("Demo1");
	attrObj.setEntity(entityObj);
	attrObj.setDescription("Green Demo author field");
	attrObj.setType(DKConstant.DK_CM_VSTRING);
	attrObj.setSize(100);
	attrObj.setNullable(false);
	attrObj.setUpdatable(true);
	attrObj.setQueryable(true);
	attrObj.setStringType(DKConstant.DK_ATTR_ALPHA);
	entityObj.add(attrObj);
	
	attrObj = new DKAttrDefFed(dsFed);	
	attrObj.setName("Title");
	attrObj.setEntityName("Demo1");
	attrObj.setEntity(entityObj);
	attrObj.setDescription("Green Demo abstract/title field");
	attrObj.setType(DKConstant.DK_CM_VSTRING);
	attrObj.setSize(200);
	attrObj.setNullable(false);
	attrObj.setUpdatable(true);
	attrObj.setQueryable(true);
	attrObj.setStringType(DKConstant.DK_ATTR_ALPHA);
	entityObj.add(attrObj);

	attrObj = new DKAttrDefFed(dsFed);	
	attrObj.setName("Date Published");
	attrObj.setEntityName("Demo1");
	attrObj.setEntity(entityObj);
	attrObj.setDescription("Green Demo date published field");
	attrObj.setType(DKConstant.DK_CM_DATE);
	attrObj.setSize(10);
	attrObj.setNullable(true);
	attrObj.setUpdatable(true);
	attrObj.setQueryable(true);
	attrObj.setStringType(DKConstant.DK_ATTR_NUMERIC);
	entityObj.add(attrObj);

	dsFed.disconnect();
	System.out.println("datastore disconnected"); 
	userName = dsFed.userName();
	dsName = dsFed.datastoreName();
	System.out.println("user " + userName + " dsName " + dsName);   
    }
    catch (DKException exc)
    {
	System.out.println("Exception name " + exc.name());
     	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
    catch (Exception exc)
    {
    	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }

  }
}

