/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

// sample to create Federated search template and search criteria
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;

// TCreateSTCriteriaFed class
/**
 * <pre>
 * Filename       : TCreateSTCriteriaFed.java
 * Description    : test the following classes and funtions in each:
 *		     	- DKDatastoreFed:
 *                          + connect
 *                          + disconnect
 *                      - DKSearchTemplateFed:
 *                          + add()
 *                      - DKSearchCriteria:
 *                          + addCriteria
 *                          + setName
 *                          + setDisplayName
 *                          + setDefaultOperator
 *                          + setValidOperators
 *                          + setCriteriaPosition
 *                          + setDisplayPosition
 *                          + setQueryable
 *                          + setDisplayable
 *                          + setDisplayWidth
 * </pre>
 */

public class TCreateSTCriteriaFed implements DKConstantFed {

  // Main method
  public static void main(String argv[]){

   // Indicate this is a Console Application on Windows NT/95 for error
   // message display in dos console code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_CM_SS_WINDOWS is the default.
   DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    try {
	DKDatastoreFed dsFed = new DKDatastoreFed();
	DKDatastoreDefFed dsDef = (DKDatastoreDefFed)dsFed.datastoreDef();

	String userName;
    	String dsName;

    	System.out.println("connecting to datastore");
	dsFed.connect("feddb","JANEWAY","pw","");
	System.out.println("datastore connected");

	userName = dsFed.userName();
	dsName = dsFed.datastoreName();

	System.out.println("user " + userName + " dsName " + dsName);

	DKSearchTemplateFed stObj = new DKSearchTemplateFed(dsFed);
	stObj.setName("Green Driver Demo");
	stObj.setDescription("Green 2nd driver demo template created by meis test case");
        stObj.setFolderName("MeiFolder1");
        stObj.setParametricOp(DK_CM_OP_ALL_ORS);
        stObj.setServerParticipation(DK_CM_SOME_SERVERS);
        stObj.setTextSingleCharWildcard("?");
        stObj.setTextMultiCharWildcard("*");
        stObj.setParmWildcard("%");
	//stObj.add();	//adding it to the federated database
        //System.out.println("DKSearchTemplate add() success!!");

	DKSearchCriteria crit = new DKSearchCriteria("GreenDemo", "Author");
	short[] ops = { DKOpCode.EQ,
			DKOpCode.NEQ,
			DKOpCode.LIKE,
			DKOpCode.NOTLIKE,
			DKOpCode.IN,
			DKOpCode.NOTIN };
	crit.setName("Author");
	crit.setDisplayName("Document Author");
	crit.setDefaultOperator(DKOpCode.LIKE);
	crit.setValidOperators(ops);
	crit.setCriteriaPosition((short)2);
	crit.setDisplayPosition((short)1);
	crit.setDisplayWidth((short)40);
	crit.setQueryable(true);
	crit.setDisplayable(true);
	stObj.addCriteria(crit);
        System.out.println("DKSearchCriteria add() success!!");

	crit = new DKSearchCriteria("GreenDemo", "Title");
	short[] ops1 = { DKOpCode.EQ,
			 DKOpCode.NEQ,
			 DKOpCode.LIKE,
			 DKOpCode.NOTLIKE,
			 DKOpCode.IN,
			 DKOpCode.NOTIN }; 

	crit.setName("Abstract");
	crit.setDisplayName("Document Title");
	crit.setDefaultOperator(DKOpCode.LIKE);
	crit.setValidOperators(ops1);
	crit.setCriteriaPosition((short)1);
	crit.setDisplayPosition((short)2);
	crit.setDisplayWidth((short)40);
	crit.setQueryable(true);
	crit.setDisplayable(true);
	stObj.addCriteria(crit);
        System.out.println("DKSearchCriteria add() success!!");


	crit = new DKSearchCriteria("GreenDemo", "Date Published");
	short[] ops2 = {DKOpCode.EQ,
			 DKOpCode.NEQ,
			 DKOpCode.BETWEEN,
			 DKOpCode.NOTBETWEEN,
			 DKOpCode.IN,
			 DKOpCode.NOTIN,
			 DKOpCode.LT,
			 DKOpCode.LEQ,
			 DKOpCode.GT,
			 DKOpCode.GEQ };

	crit.setName("Date");
	crit.setDisplayName("Date Published");
	crit.setDefaultOperator(DKOpCode.EQ);
	crit.setValidOperators(ops2);
	crit.setCriteriaPosition((short)3);
	crit.setDisplayPosition((short)3);
	crit.setDisplayWidth((short)20);
	crit.setQueryable(true);
	crit.setDisplayable(true);
	stObj.addCriteria(crit);
        System.out.println("DKSearchCriteria add() success!!");

	stObj.add();	//adding it to the federated database
        System.out.println("DKSearchTemplate add() success!!");

	dsFed.disconnect();
	System.out.println("datastore disconnected");
	userName = dsFed.userName();
	dsName = dsFed.datastoreName();
	System.out.println("user " + userName + " dsName " + dsName);
    }
    catch (DKException exc)
    {
	System.out.println("Exception name " + exc.name());
     	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }
    catch (Exception exc)
    {
    	System.out.println("Exception message " + exc.getMessage());
     	exc.printStackTrace();
    }

  }
}

