/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Get Work from a Work List
    A Work List may be used to obtain Work Packages for one or more Work Nodes.  This 
    example demonstrates how to list Work Package information using a Work List.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.util.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SDocRoutingListingICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: List all Routing Process in System, List Work Nodes, List Work Lists,
 *                    Count number of Packages in a Work List, Obtain all Work Packages in
 *                    a Work List, Retrieve & List WOrk package Details, Print Work Package
 *                    Information.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: List Routing Processes
 *                    List Work Nodes
 *                    List Work Lists
 *                    Count the number of Packages currently in a Work List
 *                    Obtain all Work Packages in a Work List
 *                    Retrieve & List Work Package Details
 *                    Printing Work Package Information
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SDocRoutingListingICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SDocRoutingListingICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SDocRoutingListingICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SDocRoutingListingICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //---------------------------------------------------------------------
            // Connect to datastore
            //---------------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //---------------------------------------------------------------------
            // Routing Service Object
            //---------------------------------------------------------------------
            // DKDocRoutingServiceICM:  Provides the core routing services such as
            //                          like starting, terminating, continuing, 
            //                          suspending, and resuming a process.
            System.out.println("Creating Routing Service Object...");

                DKDocRoutingServiceICM routingService = new DKDocRoutingServiceICM(dsICM);     // Obtain the core routing service object.

            System.out.println("Created Routing Service & Management Objects.");

            //------------------------------------------------------------------------
            // Start a Process
            //------------------------------------------------------------------------
            // As shown in SDocRoutingProcessingICM, a process may easily be created
            // as shown below.
            System.out.println("Starting an Instance of Process 'S_claimProcess'...");

                DKDDO ddoFolder = dsICM.createDDO("S_simple", DKConstant.DK_CM_FOLDER); // Using Item Type Created in SItemTypeCreationICM.  May be replaced by any Item Type.
                ddoFolder.add();  // Save folder to the persistent datastore.

  	            String workPackagePidStr = routingService.startProcess("S_claimProcess", ddoFolder.getPidObject().pidString(), 1, userName); // Using the UserName specified by commandline to log on in this Sample.

            System.out.println("Started an Instance of Process 'S_claimProcess'.");

            //------------------------------------------------------------------------
            // List Routing Processes
            //------------------------------------------------------------------------
            // List all running routing processes in the System.
            System.out.println("Listing Routing Processes in System...");
            
                listRoutingProcesses(dsICM);  // Use wrapper provided in this sample.
            
            System.out.println("Listed Routing Processes in System.");

            //------------------------------------------------------------------------
            // List Work Nodes
            //------------------------------------------------------------------------
            // list all Work Nodes in the system.
            System.out.println("Listing all Work Nodes in System...");

                listWorkNodes(dsICM);  // Use wrapper provided in this sample.

            System.out.println("Listed all Work Nodes in System.");

            //------------------------------------------------------------------------
            // List Work Lists
            //------------------------------------------------------------------------
            // list all Work Lists in the system.
            System.out.println("Listing all Work Lists in System...");

                listWorkLists(dsICM);  // Use wrapper provided in this sample.

            System.out.println("Listed all Work Lists in System.");

            //------------------------------------------------------------------------
            // Count the number of Packages curreetly in a Work List
            //------------------------------------------------------------------------
            // Work Lists are managed by an administrator or process owner.
            System.out.println("Counting Packages Currently In Work List 'S_fillClaimWL'...");

                int count = routingService.getCount("S_fillClaimWL",userName);  // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.
            
                System.out.println("  # Work Packages in Work List:  "+count);

            System.out.println("Counted Packages Currently In Work List 'S_fillClaimWL'.");

            //------------------------------------------------------------------------
            // Obtain all Work Packages in a Work List
            //------------------------------------------------------------------------
            // The process owner may obtain all work packages in the Work Nodes covered
            // by the specified Work List.
            System.out.println("Obtaining all Work Packages in a Work List...");
            
                String[] workPackagePIDs = routingService.listWorkPackagePidStrings("S_fillClaimWL",userName); // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.

                // Print Work Package PIDs
                System.out.println("Work Packages in Work List:  ("+workPackagePIDs.length+")");
                for(int i=0; i< workPackagePIDs.length; i++)
                    System.out.println("     - PID:  "+workPackagePIDs[i]);
                    
            System.out.println("Obtained all Work Packages in a Work List.");

            //------------------------------------------------------------------------
            // Retrieve & List Work Package Details
            //------------------------------------------------------------------------
            // The User-Application is responsible for ordering priorities if desired.
            // The Work Package may be retrieved and printed.
            //
            // In this Sample, retrieving & printing the first Work Package in the
            // Work List.
            System.out.println("Obtaining / Retrieving New Work Package...");
        
                DKWorkPackageICM workPackage = routingService.retrieveWorkPackage(workPackagePidStr,false); // specifying false to not check it out.  'true' would check out the object as well.
                printWorkPackage(workPackage); // Print the Work Package using the wrapper provided in this Sample.

            System.out.println("Obtained / Retrieved New Work Package.");

            //------------------------------------------------------------------------
            // Terminate the process started at the beginning of this sample.
            //------------------------------------------------------------------------
            System.out.println("Terminating Routing Process...");

                routingService.terminateProcess(workPackagePidStr);
                
            System.out.println("Terminated Routing Process.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * List all running Routing Processes in the System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    public static void listRoutingProcesses(DKDatastoreICM dsICM) throws DKException, Exception{
        
        DKDocRoutingServiceMgmtICM routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM);   // Obtain the Routing Management object.
        
        dkCollection processes = routingMgmt.listProcesses();   // Obtain list of all routing processes running.
        
        System.out.println("Running Processes:  ("+processes.cardinality()+")");
        dkIterator iter = processes.createIterator();
        while(iter.more()){
            DKProcessICM proc = (DKProcessICM) iter.next();    // Move pointer to next element and obtain that next element.
            System.out.println("     - "+proc.getName()+":  "+proc.getDescription());
        }
    }//end listRoutingProcesses

   /**
    * List all Work Nodes in the System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    public static void listWorkNodes(DKDatastoreICM dsICM) throws DKException, Exception{

        DKDocRoutingServiceMgmtICM routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM);   // Obtain the Routing Management object.

        dkCollection workNodes = routingMgmt.listWorkNodes();      // Obtain all Work Nodes in the System.
        
        System.out.println("Work Nodes in System:  ("+workNodes.cardinality()+")");
        dkIterator iter = workNodes.createIterator();
        while(iter.more()){
            DKWorkNodeICM workNode = (DKWorkNodeICM) iter.next();  // Move pointer to next element and obtain that next element.   
            if(workNode.getType()==0)
                System.out.println("       Normal Node - "+workNode.getName()+":  "+workNode.getDescription());
            else
                System.out.println("     Collection Pt - "+workNode.getName()+":  "+workNode.getDescription());
        }
        
    }//end listWorkNodes

   /**
    * List all Work Lists in the System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    public static void listWorkLists(DKDatastoreICM dsICM) throws DKException, Exception{

        DKDocRoutingServiceMgmtICM routingMgmt = new DKDocRoutingServiceMgmtICM(dsICM);   // Obtain the Routing Management object.

        dkCollection workLists = routingMgmt.listWorkLists();      // Obtain all Work Lists in the System.

        System.out.println("Work Lists in System:  ("+workLists.cardinality()+")");
        dkIterator iter = workLists.createIterator();
        while(iter.more()){
            DKWorkListICM workList = (DKWorkListICM) iter.next();  // Move pointer to next element and obtain that next element.   
            System.out.println("     - "+workList.getName()+":  "+workList.getDescription());
        }
        
    }//end listWorkLists

   /**
    * Print all Work Package PIDs in a Work List
    * @param dsICM         Connected DKDatastoreICM object.
    * @param workListName  Name of the Work List.
    * @param processOwner  Owner of the process.
    **/
    public static void printWorkPackagePidsFromWorkList(DKDatastoreICM dsICM, String workListName, String processOwner) throws DKException, Exception{
    
        DKDocRoutingServiceICM routingService = new DKDocRoutingServiceICM(dsICM);                   // Obtain the Routing Management object.
            
        String[] workPackagePIDs = routingService.listWorkPackagePidStrings("workListName",processOwner); // The userName specified here must be the process owner.  In this case, the process is owned by the user running the sample.

        // Print Work Package PIDs
        System.out.println("Work Packages in Work List:  ("+workPackagePIDs.length+")");
        for(int i=0; i< workPackagePIDs.length; i++)
            System.out.println("     - PID:  "+workPackagePIDs[i]);
    
    }//end printWorkPackagePidsFromWorkList

   /**
    * Print the specified Work Package.
    * @param workPackage  The Work Package to be Printed.
    **/
    public static void printWorkPackage(DKWorkPackageICM workPackage) throws DKException, Exception{

        System.out.println("---------------------------------------------");
        System.out.println("                Work Package");
        System.out.println("---------------------------------------------");
        System.out.println("    Process Name:  " + workPackage.getProcessName());
        System.out.println("  work Node Name:  " + workPackage.getWorkNodeName());
        System.out.println("           Owner:  " + workPackage.getOwner());
        System.out.println("        Priority:  " + workPackage.getPriority());
        System.out.println(" User Last Moved:  " + workPackage.getUserLastMoved());
        System.out.println(" Time Last Moved:  " + workPackage.getTimeLastMoved());
        System.out.println("   Suspend State:  " + workPackage.getSuspendState());
        System.out.println("    Notify State:  " + workPackage.getNotifyState());
        System.out.println("     Notify Time:  " + workPackage.getNotifyTime());
        System.out.println("     Resume Time:  " + workPackage.getResumeTime());
        System.out.println("Work Package Pid:  " + workPackage.getPidString());
        System.out.println("        Item Pid:  " + workPackage.getItemPidString());

        // Print any requirements needed to move onto next node
        dkCollection requirements = workPackage.getResumeList();
        dkIterator iter = requirements.createIterator();
        System.out.println("     Resume List:  '"+requirements.cardinality()+"' Requirements:");
        while (iter.more()){
            DKResumeListEntryICM resumeRequirement = (DKResumeListEntryICM) iter.next();    // Move pointer to next element and obtain that next element.
            System.out.println("          Required Item Type:  " + resumeRequirement.getRequiredItemTypeName());
            System.out.println("              Quanity Needed:  " + resumeRequirement.getQuantityNeeded() + "\n");
        }
        
    }//end printWorkPackage()

}//end class SDocRoutingListingICM
              
