/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Updating Item Type or Component Type Definitions
    An Item Type may be modified just in the same way that values were set
    in the Item Type Creation sample.  Definition object may be accessed
    as shown in the Item Type Retrieval Sample.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemTypeUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying & Updating Item Type Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Item Types in Memory
 *                    Saving / Updating Item Type Definitions in Datastore Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemTypeUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types must be defined.  If not, please run Sample 
 *                    SItemTypeCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemTypeRetrievalICM.java
 ************************************************************************************************/
public class SItemTypeUpdateICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemTypeRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemTypeUpdateICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------------
            // Retrieving an Individual Item Type Definition
            //-------------------------------------------------------------------
            // Retrieve a single Item Type Definition given the Item Type Name.
            //
            // NOTE:  When retrieving Item Type, Component Type, or any Entity Definitions, if it 
            //        does not exist, null is returned.  Usage of the null object will result in 
            //        a NullPointerException.
            System.out.println("Retrieving Item Type Definition of 'S_simple'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKItemTypeDefICM itemType  = (DKItemTypeDefICM)  dsDefICM.retrieveEntity("S_simple"); // Retrieve the Item Type Definition.

            System.out.println("Retrieved Item Type Definition of 'S_simple'.");

            //-------------------------------------------------------------------
            // Modify Definition
            //-------------------------------------------------------------------
            // Any part of the Item Type Definition may be modified.  For more
            // information about accessing Attribute Definitions, Mulivalue Attribute
            // Definitions, Child Component Type Definitions, and all Sub-Entities,
            // please refer to the SItemTypeRetrievalICM Sample, which covered this
            // thoroughly.  Once it is shown how to access each part, it should be
            // very easy to modify and continue to the next section here.
            //
            // In this case, a property of the main Item Type Definition will be 
            // modified.
            System.out.println("Modifying Item Type Definition for 'S_simple'...");

                itemType.setDescription("My Simple Item Type's Description.");

            System.out.println("Modified Item Type Definition for 'S_simple'.");

            //-------------------------------------------------------------------
            // Save / Update Defintion Changes in Datastore Definition
            //-------------------------------------------------------------------
            // Like Attribute Definitions, its own update() method can be used or
            // the datastore definitions's update(entityType) method may be used.
            System.out.println("Saving / Updating Item Type Definitin Changes...");

                itemType.update(); // Save the Changes

            System.out.println("Saving / Updating Item Type Definitin Changes...");

            //-------------------------------------------------------------------
            // Printing Updated Item Type Definition
            //-------------------------------------------------------------------
            // Uses wrapper function defined in SItemTypeRetrievalICM.
            System.out.println("Printing the Updated Item Type Definition of 'S_simple'...");

                SItemTypeRetrievalICM.printItemType(itemType); // Uses wrapper funciton defined at end of sample file.

            System.out.println("Printed the Updated Item Type Definition of 'S_simple'.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SItemTypeUpdateICM
              
