/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Text Indexes
    A text index is used to perform text searches on text in attributes or in 
    document content.  The user may specify a list of search words and obtain 
    results.  Searching text this way is far more efficient and flexible than
    using LIKE in a SQL query.
    
Updateing Text Indexes
    When an Item (database row) is added (to an ItemType table) that has a 
    Text Index, that Item (row) is put in a queue to be added to the Text
    Index.  After some period of time, the information is processed, taken
    out of the queue, and can be found by searching the Text Index.
    
    Updating a text index is performed to force TIE to immediately process 
    all the Items (rows) in its queue.

Reorganizing Text Indexes
    Text indexes should be reorganized for optimal performance after updating
    the text index.

For detailed information about DB2 Text Information Extender, please refer to the
DB2 Text Information Extender documentation.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: STextIndexUpdateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Text Indexes, Updating Text Indexes, Reorganizing Text Indexes, Updating
 *                    all Text Indexes in System.  Listing all Component Type Definitions in
 *                    System.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Text Indexes
 *                    Updating Text Indexes
 *                    Reorganize Text Indexes
 *                    Listing all Component Type Definitions in System
 *                    Updating all Text Indexes in System
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java STestIndexUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: none
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class STextIndexUpdateICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java STextIndexUpdateICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  STextIndexUpdateICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Obtain a list of Component Type Definitions
            //-------------------------------------------------------------
            // First, we need a list of Component Type Definitions for
            // which we will update the Text Index for each.
            System.out.println("Obtaining List of All Component Type Definitions in System...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                       // Obtain the Datastore Definition Object
                dkCollection coll = dsDefICM.listComponentTypes(DKConstantICM.DK_ICM_ALL_COMPONENT_TYPES);  // Obtain all Component Type Definitions in the System (Including System-Defined)

            System.out.println("Obtained List of All Component Type Definitions in System.");

            //-------------------------------------------------------------
            // Update Text Index for Each
            //-------------------------------------------------------------
            // Go through all Component Types in the System obtained above
            // and update the text index on each.
            //
            // Each index should also be reorganized for optimal performance.
            System.out.println("Updating Text Indexes...");

                dkIterator iter = coll.createIterator();
                while (iter.more()) {
                    DKComponentTypeDefICM compTypeDef = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
                    dsDefICM.updateTextIndexes(compTypeDef.getComponentTypeId());            // Update the Text Index for Each.
                    System.out.println("     Updated Text Index for "+compTypeDef.getName()+":  "+compTypeDef.getDescription());
                    dsDefICM.reorgTextIndexes(compTypeDef.getComponentTypeId());             // Reorganize for Optimal Performance.
                    System.out.println(" Reorganized Text Index for "+compTypeDef.getName()+":  "+compTypeDef.getDescription());
                }

            System.out.println("Updated Text Indexes.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Update all Text Indexes in System.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    public static void updateAllTextIndexes(DKDatastoreICM dsICM) throws DKException, Exception{
        System.out.println("Updating All Text Indexes in System...");

            // Obtain a list of Component Type Definitions
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef(); // Obtain the Datastore Definition Object
            dkCollection coll = dsDefICM.listComponentTypes(DKConstantICM.DK_ICM_ALL_COMPONENT_TYPES);  // Obtain all Component Type Definitions in the System (Including System-Defined)

            // Update Text Index for Each
            dkIterator iter = coll.createIterator();
            while (iter.more()) {
                DKComponentTypeDefICM compTypeDef = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
                dsDefICM.updateTextIndexes(compTypeDef.getComponentTypeId());            // Update the Text Index for Each.
                System.out.println("     Updated Text Index for "+compTypeDef.getName()+":  "+compTypeDef.getDescription());
                dsDefICM.reorgTextIndexes(compTypeDef.getComponentTypeId());             // Reorganize for Optimal Performance.
                System.out.println(" Reorganized Text Index for "+compTypeDef.getName()+":  "+compTypeDef.getDescription());
            }

        System.out.println("Updated All Text Indexes in System.");
    }//end updateAllTextIndexes
    
}//end class STextIndexUpdateICM
              
