/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// sample connect/execute/retrieve/disconnect to OnDemand

/*
 * Filename       : TClientRetrieveFolderOD.java
 * Description    : Connect to OnDemand Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                  if folders found display them and their documents/
 *                  folders.
 *                  Disconnect from OD Datastore.
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;
import java.util.*;

// TClientRetrieveFolderOD class
public class TClientRetrieveFolderOD implements DKConstantOD
{
  static String folderName;

  static int       NumCriteria = 0;
  static String[]  CriteriaNames = null;
  static String[]  CriteriaValues = null;
  static short[]   CriteriaOperators = null;

  // Main method
  public static void main(String argv[])
  {
    int     MaxCount = 5;
    boolean retrieveToFile = false;

    DKDatastoreOD dsOD = null;
    DKFolderOD folder = null;

    try
    {
      //*** for input parameters
      String libSrv = new String(""),     // library server
             userid = new String(""),     //userid
             pw     = new String(""),     // pw
             connect_string = new String(""),
             configuration_string = new String(""),
             testcase_string = new String("");

      // Select a testcase by changing this number!!!
      int testcase = 1;

      //--------------------------------------------------------------
      // Checking for input parameters
      //--------------------------------------------------------------
      if (argv.length < 3)
      {
        System.out.println("Invoke Syntax: " );
        System.out.println("  java TClientRetrieveFolderOD " + "<libSrv> <userid> <pw> <connect string> <configuration string> <testcase>" );
        return;
      }
      else
      {
        libSrv = argv[0];
        userid = argv[1];
        pw = argv[2];
        if (argv.length > 3)
          connect_string = argv[3];
        if (argv.length > 4)
          configuration_string = argv[4];
        if (argv.length > 5)
        {
          testcase_string = argv[5];
          testcase = Integer.parseInt(testcase_string);
        }
      }
      System.out.println("connection string=>" + connect_string + "<=");
      System.out.println("configuration string=>" + configuration_string + "<=");
      System.out.println("testcase=" + testcase);

      if (configuration_string.length() > 0)
      {
        configuration_string += ";ENTITY_TYPE=TEMPLATES";
        dsOD = new DKDatastoreOD(configuration_string);
      }
      else
      {
        dsOD = new DKDatastoreOD("ENTITY_TYPE=TEMPLATES");
      }
      dkResultSetCursor pCur = null;

      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      // Set a testcase
      //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      setFolderQueryCriteria(testcase, libSrv);

      System.out.println("\nUsing folder =>"+folderName+"<=");

      System.out.println("connecting to datastore");
      dsOD.connect(libSrv,userid,pw,connect_string);
      System.out.println("datastore connected, libSrv: " +
                         libSrv + ", userid: " + userid );

      DKNVPair[] parms = new DKNVPair[2];
      parms[0] = new DKNVPair("MAX_RESULTS",
                              new String(Integer.toString(MaxCount)));
      parms[1] = new DKNVPair("CONTENT", new String("ATTRONLY"));

      // Get datastore definition object
      DKDatastoreDefOD datastoreDefOD = new DKDatastoreDefOD(dsOD);
      folder = (DKFolderOD)datastoreDefOD.getEntity(folderName);

      int i = 0;
      if (folder != null)
      {
        // Set the search criteria
        if (NumCriteria > 0)
        {
          for (i = 0; i < NumCriteria; i++)
          {
            DKSearchCriteria crit = folder.getCriteria(CriteriaNames[i]);
            System.out.println("CriteriaNames["+i+"]=>"+CriteriaNames[i]+"<=");
            if (crit != null)
            {
              crit.setValue(CriteriaValues[i]);
              crit.setOperator(CriteriaOperators[i]);
              System.out.println("  =>value: " + CriteriaValues[i]);
              System.out.println("  =>operator: " + Op2Str(CriteriaOperators[i]));

              //System.out.println("\n******");
              //DisplayCritValues(crit);
              //System.out.println("******\n");
            }
          }
        }
        else
        {
          DKSearchCriteria crit;
          NumCriteria = folder.getNumQueryCrit();
          for (i = 0; i < NumCriteria; i++)
          {
            crit = folder.getQueryCriteria(i);
            if (crit != null)
            {
              String[] defValues = crit.getDefaultValues();
              if (defValues != null)
              {
                crit.setValues(defValues);
                System.out.println("  =>using default values:");
                for (i=0;i<defValues.length;i++)
                {
                  System.out.println(" =>"+defValues[i]+"<=");
                }
              }
              short defOp = crit.getDefaultOperator();
              crit.setOperator(defOp);
              System.out.println("  =>using default operator: " + Op2Str(defOp));

              //System.out.println("\n******");
              //DisplayCritValues(crit);
              //System.out.println("******\n");
            }
          }
        }

        DKDDO p = null;
        i = 0;
        long  t = 0;
        int cnt = 0;
        System.out.println("executing query");

        Date date;
        date  = new Date();
        System.out.println("Before query, time is " + date.toString());
        pCur = dsOD.execute(folder,parms,null); //parms,null);
        t = (new Date()).getTime() - t;
        System.out.println("Execute elapsed time: " + t + " msec");
        date  = new Date();
        System.out.println("After query, time is " + date.toString());
        System.out.println("cardinality: " + pCur.cardinality());
        System.out.println("datastore executed query");

        if (pCur == null)
        {
          System.out.println("cursor is null!!!");
        }

        Vector hitListVector = new Vector();
        t = (new Date()).getTime();
        while (pCur.isValid())
        {
          p = pCur.fetchNext();
          if (p != null)
          {
            i = pCur.getPosition();
            hitListVector.addElement((Object)p);
            displayHit(i, p);
          }
        } // while (pCur.isValid())
        t = (new Date()).getTime() - t;
        System.out.println("\nFetch elapsed time: " + t + " msec\n");

        t = (new Date()).getTime();
        for (i=0; i<hitListVector.size(); i++)
        {
          cnt++;
          p = (DKDDO)hitListVector.elementAt(i);

          // the retrieval action
          String idstr = ((DKPid)p.getPidObject()).pidString();
          System.out.println(" pidString : " + idstr);
          DKPid pid = new DKPid (idstr);
          DKDDO ddoold = p;
          short id, docType = 0;
          if ((id = ddoold.propertyId(DK_CM_PROPERTY_ITEM_TYPE)) > 0)
            docType = ((Short)ddoold.getProperty(id)).shortValue();
          if (docType == DK_CM_DOCUMENT)
          {
            System.out.println("create a new DDO with a cloned pid to retrieve!");
            p = dsOD.createDDO(ddoold.getObjectType(), DK_CM_DOCUMENT);
            p.setPidObject(pid);
            dsOD.retrieveObject((dkDataObject)p);
            displayDDO(cnt, dsOD, p, retrieveToFile);
          }
        }
        hitListVector.removeAllElements();

        t = (new Date()).getTime() - t;
        System.out.println("Retrieval elapsed time: " + t + " msec");

        pCur.destroy(); // Finished with the cursor

        date  = new Date();
        System.out.println("\nAfter results processed " + date.toString());
        System.out.println("\nquery results processed");
        System.out.println("Number of items retrieved: " + cnt );
      }
      else
      {
        System.out.println("Folder =>"+folderName+"<= not found on "+libSrv+"!!!");
      }

      dsOD.disconnect();
      System.out.println("\ndatastore disconnected");

      dsOD.destroy(); // Finished with the datstore
    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }

  }

  // --------------------------------------------------------------------
  //   Op2Str
  // --------------------------------------------------------------------
  public static String Op2Str(short Op)
  {
    String OpStr = new String("");
    switch (Op)
    {
      case DK_CM_OPCODE_EQ         : OpStr = "EQ"         ; break;
      case DK_CM_OPCODE_NEQ        : OpStr = "NEQ"        ; break;
      case DK_CM_OPCODE_GT         : OpStr = "GT"         ; break;
      case DK_CM_OPCODE_LT         : OpStr = "LT"         ; break;
      case DK_CM_OPCODE_LIKE       : OpStr = "LIKE"       ; break;
      case DK_CM_OPCODE_NOTLIKE    : OpStr = "NOTLIKE"    ; break;
      case DK_CM_OPCODE_BETWEEN    : OpStr = "BETWEEN"    ; break;
      case DK_CM_OPCODE_IN         : OpStr = "IN"         ; break;
      case DK_CM_OPCODE_NOTIN      : OpStr = "NOTIN"      ; break;
      case DK_CM_OPCODE_GEQ        : OpStr = "GEQ"        ; break;
      case DK_CM_OPCODE_LEQ        : OpStr = "LEQ"        ; break;
      case DK_CM_OPCODE_NOTBETWEEN : OpStr = "NOTBETWEEN" ; break;
      case DK_CM_OPCODE_AND        : OpStr = "AND"        ; break;
      case DK_CM_OPCODE_OR         : OpStr = "OR"         ; break;
      case DK_CM_OPCODE_NOT        : OpStr = "NOT"        ; break;
      case DK_CM_OPCODE_UNDEFINED  : OpStr = "UNDEFINED"  ; break;
      default                      : OpStr = "Unknown!!!" ; break;
    }
    return OpStr;
  }

  // --------------------------------------------------------------------
  // display hits from the search
  // --------------------------------------------------------------------
  public static void displayHit(int cnt, DKDDO p)
  {
    short j = 0;
    if (p != null)
    {
      try
      {
        int numDataItems = numDataItems = p.dataCount();
        // display attribute (field) names (1st time only)
        if (cnt == 1)
        {
          System.out.println("");
          for (j = 1; j <= numDataItems; j++)
          {
            String strDataName = p.getDataName(j);
            if (strDataName.equals(DK_CM_DKPARTS) == false &&
                strDataName.equals("DKResource") == false &&
                strDataName.equals("DKViews") == false &&
                strDataName.equals("DKLargeObject") == false &&
                strDataName.equals("DKFixedView") == false &&
                strDataName.equals("DKAnnotations") == false)
            {
              System.out.print("  \t\""+strDataName+"\"");
            }
          }
          System.out.println("");
        }
        // display attribute (field) values
        for (j = 1; j <= numDataItems; j++)
        {
          Object a = p.getData(j);
          String strDataName = p.getDataName(j);
          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            if (j == 1)
              System.out.print(" "+cnt+".\t");
            else
              System.out.print("\t");

            if ((a instanceof String)  ||
                (a instanceof Integer) ||
                (a instanceof Short)   ||
                (a instanceof Long)    ||
                (a instanceof Float)   ||
                (a instanceof Double)  ||
                (a instanceof DKDate)  ||
                (a instanceof DKTime)  ||
                (a instanceof DKTimestamp))
            {
              System.out.print("\"" + a + "\"");
            }
            else
              System.out.print("");
          }
        }
        System.out.println("");
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  // --------------------------------------------------------------------
  // Retrieve and display results
  // --------------------------------------------------------------------
  public static void displayDDO(int           cnt,
                                DKDatastoreOD dsOD,
                                DKDDO         p,
                                boolean       retrieveToFile)
  {
    String DocFile, DataFile, LineDataFile, ResourceFile, TocFile, LogViewFile;

    FileOutputStream myLogViewFile, myResourceFile, myTocFile, myDataFile, myDocFile;
    String extension = new String("");
    String MimeType = new String("");
    String noteString = new String("");
    String segmentFile;
    String cnt_str;
    DKAnnotationOD annotation = null;
    int numNotes = 0;
    int numBytes = 0;

    DKDDO pDDO = null;
    dkXDO pXDO = null;
    int i = 0;
    int numDataItems = 0;
    short k = 0;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;
    DKPid pid = null;

    try
    {
      // create the output directory if it does not exist
      String OutDir;
      if (File.separator.equals("\\"))
        OutDir = new String("c:\\cmbuild\\output\\");
      else
        OutDir = new String("/archive/bulldog/eip/samples/output/");

      File outputPath = new File(OutDir);
      if (outputPath.exists() == false)
      {
        // try this first
        if (outputPath.mkdirs() == false)
        {
          // If mkdirs did not work, try this
          outputPath.mkdir();
        }
      }

      if (p != null)
      {
        System.out.println("\n=====> Item " + cnt + " <=====");

        numDataItems = p.dataCount();
        pid = p.getPidObject();

        // Retrieve privileges
        DKDatastoreExtOD dsODExt = (DKDatastoreExtOD)dsOD.getExtension(DK_CM_DATASTORE_EXT);
        DKCommonPrivilege priv = dsODExt.getCommonItemPrivilege(p);
        System.out.println("privileges are:");
        if (priv.checkAuthorization(DK_CM_PRIV_PRINT) == DK_CM_PRIV_YES)
          System.out.println(" +++can print document");
        else
          System.out.println(" ---cannot print document");
        if (priv.checkAuthorization(DK_CM_PRIV_VIEW_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can view annotations");
        else
          System.out.println(" ---cannot view annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_CREATE_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can create annotations");
        else
          System.out.println(" ---cannot create annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_DELETE_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can delete annotations");
        else
          System.out.println(" ---cannot delete annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_EDIT_TEXT_ANNOTATION) == DK_CM_PRIV_YES)
          System.out.println(" +++can edit annotations");
        else
          System.out.println(" ---cannot edit annotations");
        if (priv.checkAuthorization(DK_CM_PRIV_MODIFY_MULTI_VIEWDATA) == DK_CM_PRIV_YES)
          System.out.println(" +++can change logical views");
        else
          System.out.println(" ---cannot change logical views");

        System.out.println("object type is: "+pid.getObjectType());

        System.out.println("\npid string: " + pid.pidString());
        k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);

        if (k > 0)
        {
          Short sVal = (Short)p.getProperty(k);
          j = sVal.shortValue();
          switch (j)
          {
            case DK_CM_DOCUMENT :
              System.out.println("  Item is a document ");
              break;
            default:
              System.out.println("  Unknown property: "+k);
          }
        }

        cnt_str = new String(Integer.toString(cnt));
        DataFile     = OutDir + new String("Docdata")  + cnt_str;
        LineDataFile = OutDir + new String("Linedata") + cnt_str;
        ResourceFile = OutDir + new String("resource") + cnt_str + new String(".dat");
        TocFile = OutDir + new String("Toc") + cnt_str + new String(".dat");
        LogViewFile  = OutDir + new String("LogView")  + cnt_str + new String(".dat");

        System.out.println("  DataFile=>"+DataFile+"<=");
        System.out.println("  ResourceFile=>"+ResourceFile+"<=");
        System.out.println("  LineDataFile=>"+LineDataFile+"<=");
        System.out.println("  TocFile=>"+TocFile+"<=");
        System.out.println("  LogViewFile=>"+LogViewFile+"<=");

        // Removed existing output files
        File LineFile = new File(LineDataFile + new String(".lin"));
        if (LineFile.exists())
          LineFile.delete();
        File ResFile = new File(ResourceFile);
        if (ResFile.exists())
          ResFile.delete();
        File TOCFile = new File(TocFile);
        if (TOCFile.exists())
          TOCFile.delete();
        File ViewFile = new File(LogViewFile);
        if (ViewFile.exists())
          ViewFile.delete();

        System.out.println("  Number of Data Items: " + numDataItems);

        for (j = 1; j <= numDataItems; j++)
        {
          a = p.getData(j);
          strDataName = p.getDataName(j);
          System.out.println("    " + j + ". Attribute Name: " + strDataName );
          System.out.println("       type: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_TYPE));
          System.out.println("       nullable: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE));

          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            System.out.println("       attribute id: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID));
          }

          if (a instanceof String)
          {
            System.out.println("       Attribute Value(String): =>" + a + "<=");
          }
          else if (a instanceof Integer)
          {
            System.out.println("       Attribute Value(Integer): " + a);
          }
          else if (a instanceof Short)
          {
            System.out.println("       Attribute Value(Short): " + a);
          }
          else if (a instanceof Long)
          {
            System.out.println("       Attribute Value(Long): " + a);
          }
          else if (a instanceof Float)
          {
            System.out.println("       Attribute Value(Float): " + a);
          }
          else if (a instanceof Double)
          {
            System.out.println("       Attribute Value(Double): " + a);
          }
          else if (a instanceof DKDate)
          {
            System.out.println("      Attribute Value(DKDate): " + a);
          }
          else if (a instanceof DKTime)
          {
            System.out.println("       Attribute Value(DKTime): " + a);
          }
          else if (a instanceof DKTimestamp)
          {
            System.out.println("       Attribute Value(DKTimestamp): " + a);
          }
          else if (a instanceof dkCollection)
          {
            System.out.println("       Attribute Value is collection");
            pCol = (dkCollection)a;
            pIter = pCol.createIterator();
            i = 0;

            while (pIter.more() == true)
            {
              i++;
              a = pIter.next();
              pDO = (dkDataObjectBase)a;

              if (pDO.protocol() == DK_CM_XDO)
              {
                System.out.println("         dkXDO object " + i + " in collection");
                pXDO = (dkXDO)pDO;
                DKPid pid2 = pXDO.getPidObject();
                System.out.println("           XDO pid string: " + pid2.pidString());

                // Get the MIME type
                MimeType = pXDO.getMimeType();
                System.out.println("MimeType = " + MimeType);
                if ( MimeType.equals("application/ondemand line") == true )
                {
                  // Set the EBCDIC->ASCII conversion
                  a = (new Integer(DK_CM_TEXT_ASCII));
                  pXDO.setOption( DK_CM_OPT_XDO_TEXT_CONVERSION, a);
                  System.out.println("EBCDIC->ASCII conversion set!");
                }

                byte[] DataBytes = null;
                String DataString = "";
                long   DataSize  = 0;
                // Retrieve and open instance handler for an XDO
                if (!retrieveToFile)
                {
                  pXDO.retrieve();
                  DataBytes = pXDO.getContent();
                  DataString = new String(DataBytes);
                  DataSize = DataString.length();
                }
                System.out.println("Content Begin:");
                long item_type = ((DKPidXDOOD)pXDO.getPidObject()).getType();
                long part_id = ((DKPidXDOOD)pXDO.getPidObject()).getPartId();
                switch ((short)item_type)
                {
                  case DK_CM_ANNOTATION:
                    // binary
                    System.out.println("======DK_OD_ANNOTATION (Notes)======");
                    annotation = (DKAnnotationOD)((DKBlobOD)pXDO).getExtension("dkAnnotationExt");
                    noteString  = "NOTE: " + annotation.getTimestamp();
                    noteString += " by " + annotation.getUserID();
                    noteString += " for page " + Integer.toString(annotation.getPageNumber());
                    noteString += "\n";
                    noteString += annotation.isPublic() ? "Public" : "Private";
                    noteString += "--";
                    noteString += annotation.isOkToCopy() ? "Can" : "Cannot";
                    noteString += " be copied to another server";
                    noteString += new String("\n") + annotation.getText();
                    System.out.println( noteString );
                    break;
                  case DK_OD_VIEW:
                    // binary
                    System.out.println("======DK_OD_VIEW (Logical Views)======");
                    if (retrieveToFile)
                    {
                      pXDO.retrieve(LogViewFile);
                      DataSize = pXDO.size();
                    }
                    else
                    {
                      myLogViewFile = new FileOutputStream(LogViewFile, true);  // append
                      myLogViewFile.write(DataBytes);
                      myLogViewFile.close();
                    }
                    break;
                  case DK_OD_SEGMENT:
                    // binary
                    if ( extension.equals("") == true )
                    {
                      if ( MimeType.equals("application/afp") == true )
                        extension = new String("afp");
                      else if ( MimeType.equals("application/ondemand line") == true )
                        extension = new String("lin");
                      else if ( MimeType.equals("text/plain") == true )
                        extension = new String("txt");
                      else if ( MimeType.equals("image/bmp") == true )
                        extension = new String("bmp");
                      else if ( MimeType.equals("image/gif") == true )
                        extension = new String("gif");
                      else if ( MimeType.equals("image/jpeg") == true )
                        extension = new String("jpg");
                      else if ( MimeType.equals("image/pcx") == true )
                        extension = new String("pcx");
                      else if ( MimeType.equals("image/tif") == true )
                        extension = new String("tif");
                      else if ( MimeType.equals("application/pdf") == true )
                        extension = new String("pdf");
                      else
                        extension = new String("dat");
                    }
                    System.out.println("======DK_OD_SEGMENT (Doc Data) part_id=" + part_id + "======");
                    if (extension.equals("lin"))
                      segmentFile = LineDataFile + new String(".") + extension;
                    else
                      segmentFile = DataFile + new String(".") + extension;
                    if ( part_id == 0 )
                    {
                      File SegFile = new File(segmentFile);
                      if (SegFile.exists())
                        SegFile.delete();
                    }
                    // The first segment starts at part_id 0
                    if ( part_id >= 0)
                    {
                      if (retrieveToFile)
                      {
                        pXDO.retrieve(segmentFile);
                        DataSize = pXDO.size();
                      }
                      else
                      {
                        myDataFile = new FileOutputStream(segmentFile, true);  // append
                        myDataFile.write(DataBytes);
                        myDataFile.close();
                      }
                    }
                    break;
                }
                System.out.println("\nContent End");
              }
            }
          }
          else if (a != null)
          {
            System.out.println("       Attribute Value: " + a.toString());
            if (strDataName.equals("DKResource")  ||
                strDataName.equals("DKFixedView") ||
                strDataName.equals("DKLargeObject"))
            {
              pDO = (dkDataObjectBase)a;

              if (pDO.protocol() == DK_CM_XDO)
              {
                System.out.println("         dkXDO object ");
                pXDO = (dkXDO)pDO;
                DKPid pid2 = pXDO.getPidObject();
                System.out.println("           XDO pid string: " + pid2.pidString());

                byte[] DataBytes = null;
                String DataString = "";
                long DataSize = 0;
                if (!retrieveToFile)
                {
                  // Retrieve and open instance handler for an XDO
                  pXDO.retrieve();
                  // pXDO.open();

                  DataBytes = pXDO.getContent();
                  DataString = new String(DataBytes);
                  DataSize = DataString.length();
                }
                long item_type = ((DKPidXDOOD)pXDO.getPidObject()).getType();

                if (strDataName.equals("DKResource") )
                {
                  // binary
                  System.out.println("======DK_OD_RESOURCE (AFP Resource Group)======");
                  if (retrieveToFile)
                  {
                    pXDO.retrieve(ResourceFile);
                    DataSize = pXDO.size();
                  }
                  else
                  {
                    myResourceFile = new FileOutputStream(ResourceFile, true);  // append
                    myResourceFile.write(DataBytes);
                    myResourceFile.close();
                  }
                }
                else if (strDataName.equals("DKFixedView"))
                {
                  // string
                  System.out.println("======DK_OD_FIXED_VIEW (Fixed View)======");
                  if (!retrieveToFile)
                    System.out.println( DataString );
                }
                else if (item_type == DK_OD_TOC)
                {
                  // binary
                  System.out.println("======DK_OD_TOC (Large Object table of contents)======");
                  if (retrieveToFile)
                  {
                    pXDO.retrieve(TocFile);
                    DataSize = pXDO.size();
                  }
                  else
                  {
                    myTocFile = new FileOutputStream(TocFile, true);  // append
                    myTocFile.write(DataBytes);
                    myTocFile.close();
                  }
                }
              }
            }
          }
          else System.out.println("       Attribute Value is null");
        }

        // If the resource file exists, concatenate the resource with data
        ResFile = new File(ResourceFile);
        if (ResFile.exists())
        {
          DocFile = OutDir + new String("sample") + cnt_str + new String(".") + extension;
          File DocumentFile = new File(DocFile);
          if (DocumentFile.exists())
            DocumentFile.delete();

          int BytesTotal = 0;
          byte[] buffer = new byte[100000];
          myDocFile = new FileOutputStream(DocFile);
          FileInputStream myResFile = new FileInputStream(ResourceFile);
          if ( myResFile != null)
          {
            while ( myResFile.available() > 0 )
            {
              numBytes = myResFile.read( buffer );
              if ( numBytes == -1  )
              {
                System.err.println( "Resource read error" );
                break;
              }
              myDocFile.write(buffer, 0, numBytes);
              BytesTotal += numBytes;
            }
            myResFile.close();
            ResFile.delete();
          }
          segmentFile = DataFile + new String(".") + extension;
          FileInputStream myInDataFile = new FileInputStream(segmentFile);

          // copy data
          if ( myInDataFile != null)
          {
            int DateBytes = 0;
            while ( myInDataFile.available() > 0 )
            {
              numBytes = myInDataFile.read(buffer);
              if ( numBytes == -1  )
              {
                System.err.println( "Data read error" );
                break;
              }
              DateBytes += numBytes;
              myDocFile.write(buffer, 0, numBytes);
              BytesTotal += numBytes;
            }
            myInDataFile.close();
            File SegFile = new File(segmentFile);
            SegFile.delete();
          }
          myDocFile.close();

        } // if (ResFile.exists())

      }  // if (p != null)

    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }

  // --------------------------------------------------------------------
  // Set the folder query criteria
  // --------------------------------------------------------------------
  public static void setFolderQueryCriteria(int testcase, String libSrv)
  {
    // Folder and search criteria
    // OD/390 v2.1
    if (libSrv.equalsIgnoreCase("9.91.121.31") == true)
    {
      NumCriteria = 3;
      CriteriaNames     = new String[NumCriteria];
      CriteriaValues    = new String[NumCriteria];
      CriteriaOperators = new short[NumCriteria];

      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "CHKSTMN1";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
        case 2:
          folderName = "COMPAT2";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
        case 3:
          folderName = "COMPATD";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
        case 4:
          folderName = "DELINQCY";
          CriteriaNames[0] = "LOAN NUMBER";
          break;
        case 5:
          folderName = "FSINV";
          CriteriaNames[0] = "INVOICE#";
          break;
        case 6:
          folderName = "INSURE";
          CriteriaNames[0] = "POLICY NUMBER";
          break;
        case 7:
          folderName = "INSURTST";
          CriteriaNames[0] = "POLICY NUMBER";
          break;
        case 8:
          folderName = "K0560";
          CriteriaNames[0] = "FIL#";
          break;
        case 9:
          folderName = "KEYBANK";
          CriteriaNames[0] = "ACCOUNT NUMBER";
          break;
      }
      CriteriaValues   [0] = "%";
      CriteriaOperators[0] = DK_CM_OPCODE_EQ;
      CriteriaNames    [1] = "FROM (YYYY-MM-DD)";
      CriteriaValues   [1] = "1990-05-26";
      CriteriaOperators[1] = DK_CM_OPCODE_GEQ;
      CriteriaNames    [2] = "TO   (YYYY-MM-DD)";
      CriteriaValues   [2] = "2001-12-31";
      CriteriaOperators[2] = DK_CM_OPCODE_LEQ;
    }
    // OD/390 v7.1
    else if (libSrv.equalsIgnoreCase("9.91.121.34") == true)
    {
      switch (testcase)
      {
        default:
        case 1: // AFP data
          folderName = "Credit Card Statements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account";
          CriteriaValues   [0] = "%"; //"000-000-010";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 2:
          folderName = "CHKSTMNT";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "ACCOUNT NUM";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 3:  // line data
          folderName = "Delinquency";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Page Number";
          CriteriaValues   [0] = "0";
          CriteriaOperators[0] = DK_CM_OPCODE_NEQ;
          break;
        case 4:  // line data large object
          folderName = "LOAN LO";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Date";
          CriteriaValues   [0] = "10/01/94";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
        case 5:  // pdf
          folderName = "PDF Landscape";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Region";
          CriteriaValues   [0] = "'%'";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
        case 6:  // AFP
          folderName = "afpdata";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Tax Year";
          CriteriaValues   [0] = "1995";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
      }
    }
    // OD/400 v4.5
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400c") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "CHECKSTMTS";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account Number";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    // OD/400 v5.1
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("rdr400g") == true)
    {
      switch (testcase)
      {
        default:
        case 1:
          folderName = "CheckStatements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Customer Name";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 6 &&
             libSrv.substring(0,6).equalsIgnoreCase("reagan") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "load-db2-CRD";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account Name";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 7 &&
             libSrv.substring(0,7).equalsIgnoreCase("everest") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // PDF
          folderName = "43MBPDF";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Report Title";
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("akira") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "rl-DFAC";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "nomcli"; //"exploit";
          CriteriaValues   [0] = "MR PEREZ ERIC"; //"%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 2:
          folderName = "LARpmr94348";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Customer Id"; ;
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
        case 3:
          folderName = "Credit Card Statements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account"; ;
          CriteriaValues   [0] = "%";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
    else if (libSrv.length() >= 5 &&
             libSrv.substring(0,5).equalsIgnoreCase("banff") == true)
    {
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "load-db2-afpdata";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Tax Year";
          CriteriaValues   [0] = "1995-01-01";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
        case 2:  // Line
          folderName = "load-db2-LOAN";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Date";
          CriteriaValues   [0] = "1994-10-01";
          CriteriaOperators[0] = DK_CM_OPCODE_EQ;
          break;
      }
    }
    else //if (libSrv.equalsIgnoreCase("gunnar") == true)
    {
      // Single object AFP
      switch (testcase)
      {
        default:
        case 1:  // AFP
          folderName = "Credit Card Statements";

          NumCriteria = 1;
          CriteriaNames     = new String[NumCriteria];
          CriteriaValues    = new String[NumCriteria];
          CriteriaOperators = new short[NumCriteria];

          CriteriaNames    [0] = "Account";
          CriteriaValues   [0] = "%"; //"000-000-010";
          CriteriaOperators[0] = DK_CM_OPCODE_LIKE;
          break;
      }
    }
  }

  // --------------------------------------------------------------------
  // Display search criteria
  // --------------------------------------------------------------------
  public static void DisplayCritValues(DKSearchCriteria crit)
  {
    int i = 0;
    System.out.println("Name          =>"+crit.getName()+"<=");
    System.out.println("DisplayName   =>"+crit.getDisplayName()+"<=");
    System.out.println("EntityName    =>"+crit.getEntityName()+"<=");
    System.out.println("AttrName      =>"+crit.getAttrName()+"<=");
    System.out.println("CritId        = "+crit.getId());

    short[] validOps = crit.getValidOperators();
    if (validOps != null)
    {
      System.out.println("\nNumber of valid operators = "+validOps.length);
      for (i=0;i<validOps.length;i++)
      {
        System.out.println((i+1)+". =>"+Op2Str(validOps[i])+"<= \t("+validOps[i]+")");
      }
    }
    System.out.println("\nDefaultOperator=>"+Op2Str(crit.getDefaultOperator())+"<= \t("+crit.getDefaultOperator()+")");
    String[] defValues = crit.getDefaultValues();
    if (defValues != null)
    {
      System.out.println("\nNumber of default values = "+defValues.length);
      for (i=0;i<defValues.length;i++)
      {
        System.out.println((i+1)+". =>"+defValues[i]+"<=");
      }
    }
    System.out.println("\nOperator=>"+Op2Str(crit.getOperator())+"<= \t("+crit.getOperator()+")");
    System.out.println("Value()=>"+crit.getValue()+"<=");

    String[] values = crit.getValues();
    if (values != null)
    {
      System.out.println("\nNumber of Values = "+values.length);
      for (i=0;i<values.length;i++)
      {
        System.out.println((i+1)+". =>"+values[i]+"<=");
      }
    }
    System.out.println("\n");
    System.out.println("DisplayPosition = "+crit.getDisplayPosition());
    System.out.println("DisplayWidth    = "+crit.getDisplayWidth());
    System.out.println("CriteriaPosition= "+crit.getCriteriaPosition());
    System.out.println("isDisplayable   = "+crit.isDisplayable());
    System.out.println("isQueryable     = "+crit.isQueryable());
    System.out.println("getType         = "+crit.getType());
  }
}
