/**
@copyright(disclaimer)
Licensed Materials - Property of IBM

IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE
IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR
SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE
SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS LICENSORS AND SUPPLIERS
BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE
DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS HAVE BEEN
ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

package com.ibm.mm.viewer.annotation;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.geom.*;

/**
 * TImageAnnotation sample shows how to plugin a simple image annotation type into the annotation services.
 * Shows the basic steps needed to build a custom annotation - derive from CMBPageAnnotation, override methods needed
 * to correctly render the annotation and also build an appropriate properties dialog for the annotation.
 */
public class TImageAnnotation extends CMBPageAnnotation {
  private static CMBPropertiesPanel annotationPropertiesPanel = null;
  private Vector propList = new Vector();
  private static Hashtable compTable = new Hashtable();
  private static boolean isDefaultSet = false;
  private static boolean defaultVWD;
  private static String defaultFileName = "Help_normal.gif";
  private String imageFileName = "Help_normal.gif";
  private Image image;
  public static final int ANN_IMAGE = 101;

  public TImageAnnotation() {
      super();
      init();
  }

  public TImageAnnotation(Rectangle r)
  {
       super(r);
       init();
  }

  public TImageAnnotation(Rectangle r, int pageNumber)
  {
       super(r, pageNumber);
       init();
  }

  public void init()
  {
       type=ANN_IMAGE;
       if( isDefaultSet )
          updateValues(UPDATE_ANN_WITH_DEFAULTS);
       else
          updateValues(UPDATE_DEFAULTS_WITH_ANN);

      setImageFileName(imageFileName);

  }

  public void setImageFileName(String filename )
  {
    imageFileName = filename;
    System.out.println("image file name:" + imageFileName);
    java.net.URL imageUrl  = this.getClass().getClassLoader().getResource(imageFileName);
    image = Toolkit.getDefaultToolkit().createImage(imageUrl);
       try
        {
          MediaTracker tracker = new MediaTracker(new Frame());
          tracker.addImage(image,0);
          tracker.waitForID(0);
        } catch (InterruptedException e) {
          System.out.println(e);
        }
  }

  public String getImageFileName()
  {
    return imageFileName;
  }

   public void draw(Graphics2D g2)
    {
       if (!allowedToDraw)
            return;
       if (isInMotion && !isVisibleWhileDragging)
       {
            drawOutline(g2);
            return;
       }

       Color oldColor=g2.getColor();
       Rectangle drawRect=getDrawRect();
       if( image != null )
           g2.drawImage(image,drawRect.x, drawRect.y, drawRect.width,drawRect.height, new Frame() );

       super.draw(g2);
  }

  public void drawOutline(Graphics2D g2)
  {
    //save all the ones that will be modified...
       Color oldColor=g2.getColor();
       Stroke oldStroke=g2.getStroke();
       Rectangle drawRect=getDrawRect();

       //change the graphics to do what we want....
       g2.setColor(Color.black);
       g2.setStroke(getOutlineStroke());
       Rectangle2D.Float rect=new Rectangle2D.Float(drawRect.x,drawRect.y,drawRect.width,drawRect.height);
       //draw
       g2.draw(rect);
       //restore the graphics into its previous state
       g2.setStroke(oldStroke);
       g2.setColor(oldColor);
       super.drawOutline(g2);

  }

  public CMBPropertiesPanel getAnnotationPropertiesPanel()
     {
        if( annotationPropertiesPanel == null ){
            propList.addElement("Image File Name");
            propList.addElement(new String());
            propList.addElement("Annotation.visibleWhileDragging");
            propList.addElement(new Boolean(true));
            annotationPropertiesPanel = new CMBPropertiesPanel( this, propList );
        }
        return annotationPropertiesPanel;
     }

     public JPanel getPropertiesPanel(){
        getAnnotationPropertiesPanel().setAnnotation( this );
        return annotationPropertiesPanel.getPanel();
     }
       public void handlePropertyChange( String propName, JComponent comp, int action)
      {
         if( action == UPDATE_DEFAULTS_WITH_GUI )
           isDefaultSet = true;
           if( propName.equals("Image File Name" ) ){
              switch( action ){
                case UPDATE_ANN_WITH_GUI:
                    setImageFileName( ((JTextField)comp).getText() );
                    break;
                case UPDATE_GUI_WITH_ANN:
                    ((JTextField)comp).setText( getImageFileName() );
                    break;
                case UPDATE_DEFAULTS_WITH_ANN:
                    defaultFileName = getImageFileName();
                    break;
                case UPDATE_ANN_WITH_DEFAULTS:
                    setImageFileName( defaultFileName );
                    break;
                case UPDATE_GUI_WITH_DEFAULTS:
                     ((JTextField)comp).setText( defaultFileName );
                    break;
                case UPDATE_DEFAULTS_WITH_GUI:
                    defaultFileName =  ((JTextField)comp).getText() ;
                    break;
              }
          }
          else  if( propName.equals("Annotation.visibleWhileDragging" ) ){
              switch( action ){
                case UPDATE_ANN_WITH_GUI:
                    setVisibleWhileDragging( ((JCheckBox)comp).isSelected() );
                    break;
                case UPDATE_GUI_WITH_ANN:
                    ((JCheckBox)comp).setSelected( getVisibleWhileDragging() );
                    break;
                case UPDATE_DEFAULTS_WITH_ANN:
                    defaultVWD =  getVisibleWhileDragging();
                    break;
                case UPDATE_ANN_WITH_DEFAULTS:
                    setVisibleWhileDragging( defaultVWD );
                    break;
                case UPDATE_GUI_WITH_DEFAULTS:
                   ((JCheckBox)comp).setSelected( defaultVWD );
                    break;
                case UPDATE_DEFAULTS_WITH_GUI:
                    defaultVWD =  ((JCheckBox)comp).isSelected();
                    break;
              }
          }

     }
     public void updateValues( int action )
     {
       if( compTable.isEmpty() ) {
          return;
       }
       Enumeration props = compTable.keys();
       String propName;
       JComponent comp;
       firePropertiesWillChangeEvent();
       while( props.hasMoreElements() ){
          propName = (String)props.nextElement();
          comp = (JComponent)compTable.get( propName );
          this.handlePropertyChange( propName, comp, action);
       }
       firePropertiesChangeEvent();
     }


     public void addPropertyComponent( String propertyName, JComponent component ){
        compTable.put( propertyName , component );
     }




}
